@@ -1,5 +1,4 @@
-
-// FGAIAircraft - FGAIBase-derived class creates an AI airplane
+// // // FGAIAircraft - FGAIBase-derived class creates an AI airplane
 //
 // Written by David Culp, started October 2003.
 //
@@ -30,6 +29,7 @@
 #include <Main/viewer.hxx>
 #include <Scenery/scenery.hxx>
 #include <Scenery/tilemgr.hxx>
+#include <Airports/dynamics.hxx>
 
 #include <string>
 #include <math.h>
@@ -41,37 +41,24 @@
 #  include <ieeefp.h>
 #endif
 
-SG_USING_STD(string);
+using std::string;
 
 #include "AIAircraft.hxx"
+#include "performancedata.hxx"
+#include "performancedb.hxx"
+
 //#include <Airports/trafficcontroller.hxx>
 
 static string tempReg;
-//
-// accel, decel, climb_rate, descent_rate, takeoff_speed, climb_speed,
-// cruise_speed, descent_speed, land_speed
-//
 
-const FGAIAircraft::PERF_STRUCT FGAIAircraft::settings[] = {
-            // light aircraft
-            {2.0, 2.0,  450.0, 1000.0,  70.0,  80.0, 100.0,  80.0,  60.0},
-            // ww2_fighter
-            {4.0, 2.0, 3000.0, 1500.0, 110.0, 180.0, 250.0, 200.0, 100.0},
-            // jet_transport
-            {5.0, 2.0, 3000.0, 1500.0, 140.0, 300.0, 430.0, 300.0, 130.0},
-            // jet_fighter
-            {7.0, 3.0, 4000.0, 2000.0, 150.0, 350.0, 500.0, 350.0, 150.0},
-            // tanker
-            {5.0, 2.0, 3000.0, 1500.0, 140.0, 300.0, 430.0, 300.0, 130.0},
-            // ufo (extreme accel/decel)
-            {30.0, 30.0, 6000.0, 6000.0, 150.0, 300.0, 430.0, 300.0, 130.0}
-        };
+class AI_OutOfSight{};
+class FP_Inactive{};
 
 FGAIAircraft::FGAIAircraft(FGAISchedule *ref) : FGAIBase(otAircraft) {
     trafficRef = ref;
     if (trafficRef) {
         groundOffset = trafficRef->getGroundOffset();
-        setCallSign(trafficRef->getCallSign());
+	setCallSign(trafficRef->getCallSign());
     }
     else
         groundOffset = 0;
@@ -95,6 +82,9 @@ FGAIAircraft::FGAIAircraft(FGAISchedule *ref) : FGAIBase(otAircraft) {
     headingChangeRate = 0.0;
 
     holdPos = false;
+
+    _performance = 0; //TODO initialize to JET_TRANSPORT from PerformanceDB
+    
 }
 
 
@@ -140,63 +130,58 @@ void FGAIAircraft::update(double dt) {
     Transform();
 }
 
-
 void FGAIAircraft::setPerformance(const std::string& acclass) {
-    if (acclass == "light") {
-        SetPerformance(&FGAIAircraft::settings[FGAIAircraft::LIGHT]);
-    } else if (acclass == "ww2_fighter") {
-        SetPerformance(&FGAIAircraft::settings[FGAIAircraft::WW2_FIGHTER]);
-    } else if (acclass == "jet_transport") {
-        SetPerformance(&FGAIAircraft::settings[FGAIAircraft::JET_TRANSPORT]);
-    } else if (acclass == "jet_fighter") {
-        SetPerformance(&FGAIAircraft::settings[FGAIAircraft::JET_FIGHTER]);
-    } else if (acclass == "tanker") {
-        SetPerformance(&FGAIAircraft::settings[FGAIAircraft::JET_TRANSPORT]);
-    } else if (acclass == "ufo") {
-        SetPerformance(&FGAIAircraft::settings[FGAIAircraft::UFO]);
-    } else {
-        SetPerformance(&FGAIAircraft::settings[FGAIAircraft::JET_TRANSPORT]);
-    }
+     static PerformanceDB perfdb; //TODO make it a global service
+     setPerformance(perfdb.getDataFor(acclass));
+  }
+
+
+ void FGAIAircraft::setPerformance(PerformanceData *ps) {
+     _performance = ps;
+  }
+
+
+ void FGAIAircraft::Run(double dt) {
+      FGAIAircraft::dt = dt;
+
+     try {
+         updatePrimaryTargetValues(); // target hdg, alt, speed
+     }
+     catch (AI_OutOfSight) {
+         return;
+     }
+     catch (FP_Inactive) {
+         //return;
+         groundTargetSpeed = 0;
+     }
+
+     handleATCRequests(); // ATC also has a word to say
+     updateSecondaryTargetValues(); // target roll, vertical speed, pitch
+     updateActualState(); 
+     UpdateRadar(manager);
+     checkVisibility();
+  }
+
+void FGAIAircraft::checkVisibility() 
+{
+     double visibility_meters = fgGetDouble("/environment/visibility-m");
+
+     FGViewer* vw = globals->get_current_view();
+     double course, distance;
+
+     SGWayPoint current(pos.getLongitudeDeg(), pos.getLatitudeDeg(), 0);
+     SGWayPoint view (vw->getLongitude_deg(), vw->getLatitude_deg(), 0);
+     view.CourseAndDistance(current, &course, &distance);
+     if (distance > visibility_meters) {
+         //aip.getSGLocation()->set_cur_elev_m(aptElev);
+         //return;
+         invisible = true;
+      } else {
+         invisible = false;
+      }
 }
 
 
-void FGAIAircraft::SetPerformance(const PERF_STRUCT *ps) {
-
-    performance = ps;
-}
-
-
-void FGAIAircraft::Run(double dt) {
-
-    FGAIAircraft::dt = dt;
-    if (!updateTargetValues())
-        return;
-
-    if (controller) {
-        controller->update(getID(),
-                           pos.getLatitudeDeg(),
-                           pos.getLongitudeDeg(),
-                           hdg,
-                           speed,
-                           altitude_ft, dt);
-        processATC(controller->getInstruction(getID()));
-    }
-
-    if (no_roll) {
-        adjustSpeed(groundTargetSpeed);
-    } else {
-        adjustSpeed(tgt_speed);
-    }
-
-    updatePosition();
-    updateHeading();
-    updateBankAngles();
-    updateAltitudes();
-    updateVerticalSpeed();
-    matchPitchAngle();
-    UpdateRadar(manager);
-}
-
 
 void FGAIAircraft::AccelTo(double speed) {
     tgt_speed = speed;
@@ -257,9 +242,6 @@ void FGAIAircraft::SetFlightPlan(FGAIFlightPlan *f) {
 
 void FGAIAircraft::ProcessFlightPlan( double dt, time_t now ) {
 
-    //if (! fpExecutable(now))
-    //      return;
-
     // the one behind you
     FGAIFlightPlan::waypoint* prev = 0;
     // the one ahead
@@ -280,7 +262,8 @@ void FGAIAircraft::ProcessFlightPlan( double dt, time_t now ) {
         handleFirstWaypoint();
         return;
     }                            // end of initialization
-
+    if (! fpExecutable(now))
+          return;
     dt_count = 0;
 
     if (! leadPointReached(curr)) {
@@ -305,7 +288,10 @@ void FGAIAircraft::ProcessFlightPlan( double dt, time_t now ) {
         //TODO let the fp handle this (loading of next leg)
         fp->IncrementWaypoint((bool) trafficRef);
         if (!(fp->getNextWaypoint()) && trafficRef)
-            loadNextLeg();
+            if (!loadNextLeg()) {
+                setDie(true);
+                return;
+            }
 
         prev = fp->getPreviousWaypoint();
         curr = fp->getCurrentWaypoint();
@@ -329,7 +315,7 @@ void FGAIAircraft::ProcessFlightPlan( double dt, time_t now ) {
         }
 
         if (next) {
-            fp->setLeadDistance(speed, tgt_heading, curr, next);
+            fp->setLeadDistance(tgt_speed, tgt_heading, curr, next);
         }
 
         if (!(prev->on_ground))  // only update the tgt altitude from flightplan if not on the ground
@@ -356,18 +342,19 @@ void FGAIAircraft::initializeFlightPlan() {
 
 
 bool FGAIAircraft::_getGearDown() const {
-    return ((props->getFloatValue("position/altitude-agl-ft") < 900.0)
-            && (props->getFloatValue("velocities/airspeed-kt")
-                < performance->land_speed*1.25));
+    return _performance->gearExtensible(this);
 }
 
 
-void FGAIAircraft::loadNextLeg() {
+bool FGAIAircraft::loadNextLeg() {
 
     int leg;
     if ((leg = fp->getLeg())  == 10) {
-        trafficRef->next();
+        if (!trafficRef->next()) {
+            return false;
+        }
         setCallSign(trafficRef->getCallSign());
+	//props->setStringValue("callsign", callsign.c_str());
         leg = 1;
         fp->setLeg(leg);
     }
@@ -392,7 +379,9 @@ void FGAIAircraft::loadNextLeg() {
                     trafficRef->getFlightType(),
                     acType,
                     company);
+       //cerr << "created  leg " << leg << " for " << trafficRef->getCallSign() << endl;
     }
+    return true;
 }
 
 
@@ -405,11 +394,11 @@ void FGAIAircraft::getGroundElev(double dt) {
     dt_elev_count += dt;
 
     // Update minimally every three secs, but add some randomness
-    // to prevent all IA objects doing this in synchrony
+    // to prevent all AI objects doing this in synchrony
     if (dt_elev_count < (3.0) + (rand() % 10))
         return;
-    else
-        dt_elev_count = 0;
+
+    dt_elev_count = 0;
 
     // Only do the proper hitlist stuff if we are within visible range of the viewer.
     if (!invisible) {
@@ -453,10 +442,14 @@ void FGAIAircraft::announcePositionToController() {
     if (trafficRef) {
         int leg = fp->getLeg();
 
-        // Note that leg was been incremented after creating the current leg, so we should use
+        // Note that leg has been incremented after creating the current leg, so we should use
         // leg numbers here that are one higher than the number that is used to create the leg
         //
         switch (leg) {
+          case 2:              // Startup and Push back
+            if (trafficRef->getDepartureAirport()->getDynamics())
+                controller = trafficRef->getDepartureAirport()->getDynamics()->getStartupController();
+            break;
         case 3:              // Taxiing to runway
             if (trafficRef->getDepartureAirport()->getDynamics()->getGroundNetwork()->exists())
                 controller = trafficRef->getDepartureAirport()->getDynamics()->getGroundNetwork();
@@ -465,12 +458,14 @@ void FGAIAircraft::announcePositionToController() {
             if (trafficRef->getDepartureAirport()->getDynamics()) {
                 controller = trafficRef->getDepartureAirport()->getDynamics()->getTowerController();
                 //if (trafficRef->getDepartureAirport()->getId() == "EHAM") {
-                //cerr << trafficRef->getCallSign() << " at runway " << fp->getRunway() << "Ready for departure "
-                //   << trafficRef->getFlightType() << " to " << trafficRef->getArrivalAirport()->getId() << endl;
+                //string trns = trafficRef->getCallSign() + " at runway " + fp->getRunway() + 
+                //              ". Ready for departure. " + trafficRef->getFlightType() + " to " +
+                //              trafficRef->getArrivalAirport()->getId();
+                //fgSetString("/sim/messages/atc", trns.c_str());
                 //  if (controller == 0) {
                 //cerr << "Error in assigning controller at " << trafficRef->getDepartureAirport()->getId() << endl;
                 //}
-
+                //}
             } else {
                 cerr << "Error: Could not find Dynamics at airport : " << trafficRef->getDepartureAirport()->getId() << endl;
             }
@@ -486,31 +481,30 @@ void FGAIAircraft::announcePositionToController() {
 
         if ((controller != prevController) && (prevController != 0)) {
             prevController->signOff(getID());
-            string callsign =  trafficRef->getCallSign();
-            if ( trafficRef->getHeavy())
-                callsign += "Heavy";
-            switch (leg) {
-            case 3:
-                //cerr << callsign << " ready to taxi to runway " << fp->getRunway() << endl;
-                break;
-            case 4:
-                //cerr << callsign << " at runway " << fp->getRunway() << "Ready for take-off. "
-                //     << trafficRef->getFlightRules() << " to " << trafficRef->getArrivalAirport()->getId()
-                //     << "(" << trafficRef->getArrivalAirport()->getName() << ")."<< endl;
-                break;
-            }
         }
         prevController = controller;
         if (controller) {
             controller->announcePosition(getID(), fp, fp->getCurrentWaypoint()->routeIndex,
                                          _getLatitude(), _getLongitude(), hdg, speed, altitude_ft,
-                                         trafficRef->getRadius(), leg, trafficRef->getCallSign());
+                                         trafficRef->getRadius(), leg, this);
         }
     }
 }
 
+// Process ATC instructions and report back
 
 void FGAIAircraft::processATC(FGATCInstruction instruction) {
+    if (instruction.getCheckForCircularWait()) {
+        // This is not exactly an elegant solution, 
+        // but at least it gives me a chance to check
+        // if circular waits are resolved.
+        // For now, just take the offending aircraft 
+        // out of the scene
+	setDie(true);
+        // a more proper way should be - of course - to
+        // let an offending aircraft take an evasive action
+        // for instance taxi back a little bit.
+    }
     //cerr << "Processing ATC instruction (not Implimented yet)" << endl;
     if (instruction.getHoldPattern   ()) {}
 
@@ -523,7 +517,7 @@ void FGAIAircraft::processATC(FGATCInstruction instruction) {
     } else {
         if (holdPos) {
             //if (trafficRef)
-            //	cerr << trafficRef->getCallSign() << " Resuming Taxi " << endl;
+            //	cerr << trafficRef->getCallSign() << " Resuming Taxi." << endl;
             holdPos = false;
         }
         // Change speed Instruction. This can only be excecuted when there is no
@@ -551,6 +545,7 @@ void FGAIAircraft::processATC(FGATCInstruction instruction) {
 
 void FGAIAircraft::handleFirstWaypoint() {
     bool eraseWaypoints;         //TODO YAGNI
+    headingError = 0;
     if (trafficRef) {
         eraseWaypoints = true;
     } else {
@@ -567,7 +562,10 @@ void FGAIAircraft::handleFirstWaypoint() {
     //TODO fp should handle this
     fp->IncrementWaypoint(eraseWaypoints);
     if (!(fp->getNextWaypoint()) && trafficRef)
-        loadNextLeg();
+        if (!loadNextLeg()) {
+            setDie(true);
+            return;
+        }
 
     prev = fp->getPreviousWaypoint();   //first waypoint
     curr = fp->getCurrentWaypoint();    //second waypoint
@@ -638,16 +636,21 @@ bool FGAIAircraft::leadPointReached(FGAIFlightPlan::waypoint* curr) {
 
     //cerr << "2" << endl;
     double lead_dist = fp->getLeadDistance();
-    //cerr << " Distance : " << dist_to_go << ": Lead distance " << lead_dist << endl;
     // experimental: Use fabs, because speed can be negative (I hope) during push_back.
 
     if (lead_dist < fabs(2*speed)) {
-        //don't skip over the waypoint
-        lead_dist = fabs(2*speed);
-        //cerr << "Extending lead distance to " << lead_dist << endl;
+      //don't skip over the waypoint
+      lead_dist = fabs(2*speed);
+      //cerr << "Extending lead distance to " << lead_dist << endl;
     }
 
     //prev_dist_to_go = dist_to_go;
+    //if (dist_to_go < lead_dist)
+    //     cerr << trafficRef->getCallSign() << " Distance : " 
+    //          << dist_to_go << ": Lead distance " 
+    //          << lead_dist << " " << curr->name 
+    //          << " Ground target speed " << groundTargetSpeed << endl;
+         
     return dist_to_go < lead_dist;
 }
 
@@ -662,7 +665,7 @@ bool FGAIAircraft::aiTrafficVisible() {
 
     user.CourseAndDistance(current, &course, &distance);
 
-    return ((distance * SG_METER_TO_NM) <= TRAFFICTOAIDIST);
+    return ((distance * SG_METER_TO_NM) <= TRAFFICTOAIDISTTODIE);
 }
 
 
@@ -685,8 +688,13 @@ bool FGAIAircraft::handleAirportEndPoints(FGAIFlightPlan::waypoint* prev, time_t
 
     // This waypoint marks the fact that the aircraft has passed the initial taxi
     // departure waypoint, so it can release the parking.
-    if (prev->name == "park2")
+    //cerr << trafficRef->getCallSign() << " has passed waypoint " << prev->name << " at speed " << speed << endl;
+    if (prev->name == "PushBackPoint") {
         dep->getDynamics()->releaseParking(fp->getGate());
+        time_t holdUntil = now + 120;
+	fp->setTime(holdUntil);
+	//cerr << _getCallsign() << "Holding at pushback point" << endl;
+    }
 
     // This is the last taxi waypoint, and marks the the end of the flight plan
     // so, the schedule should update and wait for the next departure time.
@@ -696,7 +704,7 @@ bool FGAIAircraft::handleAirportEndPoints(FGAIFlightPlan::waypoint* prev, time_t
         if (nextDeparture < (now+1200)) {
             nextDeparture = now + 1200;
         }
-        fp->setTime(nextDeparture);
+        fp->setTime(nextDeparture); // should be "next departure"
     }
 
     return true;
@@ -719,7 +727,7 @@ void FGAIAircraft::controlHeading(FGAIFlightPlan::waypoint* curr) {
 
     if (finite(calc_bearing)) {
         double hdg_error = calc_bearing - tgt_heading;
-        if (fabs(hdg_error) > 1.0) {
+        if (fabs(hdg_error) > 0.01) {
             TurnTo( calc_bearing );
         }
 
@@ -746,7 +754,9 @@ void FGAIAircraft::controlSpeed(FGAIFlightPlan::waypoint* curr, FGAIFlightPlan::
 
     if (fabs(speed_diff) > 10) {
         prevSpeed = speed;
-        fp->setLeadDistance(speed, tgt_heading, curr, next);
+        if (next) {
+            fp->setLeadDistance(speed, tgt_heading, curr, next);
+        }
     }
 }
 
@@ -754,36 +764,38 @@ void FGAIAircraft::controlSpeed(FGAIFlightPlan::waypoint* curr, FGAIFlightPlan::
 /**
  * Update target values (heading, alt, speed) depending on flight plan or control properties
  */
-bool FGAIAircraft::updateTargetValues() {
+void FGAIAircraft::updatePrimaryTargetValues() {
     if (fp)                      // AI object has a flightplan
     {
         //TODO make this a function of AIBase
         time_t now = time(NULL) + fgGetLong("/sim/time/warp");
         //cerr << "UpateTArgetValues() " << endl;
         ProcessFlightPlan(dt, now);
-        if (! fp->isActive(now)) {
-            // Do execute Ground elev for inactive aircraft, so they
-            // Are repositioned to the correct ground altitude when the user flies within visibility range.
-            // In addition, check whether we are out of user range, so this aircraft
-            // can be deleted.
-            if (no_roll) {
+
+        // Do execute Ground elev for inactive aircraft, so they
+        // Are repositioned to the correct ground altitude when the user flies within visibility range.
+        // In addition, check whether we are out of user range, so this aircraft
+        // can be deleted.
+        if (onGround()) {
                 Transform();     // make sure aip is initialized.
-                if (trafficRef) {
-                    //cerr << trafficRef->getRegistration() << " Setting altitude to " << altitude_ft;
-                    if (! aiTrafficVisible()) {
-                        setDie(true);
-                        return false;
-                    }
-                    getGroundElev(dt);
-                    doGroundAltitude();
-                    // Transform();
-                    pos.setElevationFt(altitude_ft);
-                }
-            }
-            return false;
+                getGroundElev(dt);
+                doGroundAltitude();
+                // Transform();
+                pos.setElevationFt(altitude_ft);
         }
-    }
-    else {
+        if (trafficRef) {
+           //cerr << trafficRef->getRegistration() << " Setting altitude to " << altitude_ft;
+            if (! aiTrafficVisible()) {
+                setDie(true);
+                //cerr << trafficRef->getRegistration() << " is set to die " << endl;
+                throw AI_OutOfSight();
+            }
+        }
+        timeElapsed = now - fp->getStartTime();
+        if (! fp->isActive(now)) { 
+            throw FP_Inactive();
+        }
+    } else {
         // no flight plan, update target heading, speed, and altitude
         // from control properties.  These default to the initial
         // settings in the config file, but can be changed "on the
@@ -812,39 +824,8 @@ bool FGAIAircraft::updateTargetValues() {
 
         AccelTo( props->getDoubleValue("controls/flight/target-spd" ) );
     }
-    return true;
 }
 
-
-/**
- * Adjust the speed (accelerate/decelerate) to tgt_speed.
- */
-void FGAIAircraft::adjustSpeed(double tgt_speed) {
-    double speed_diff = tgt_speed - speed;
-    speed_diff = groundTargetSpeed - speed;
-
-    if (speed_diff > 0.0)        // need to accelerate
-    {
-        speed += performance->accel * dt;
-        if ( speed > tgt_speed )
-            speed = tgt_speed;
-
-    } else if (speed_diff < 0.0) {
-        if (no_roll) {
-            // on ground (aircraft can't roll)
-            // deceleration performance is better due to wheel brakes.
-            speed -= performance->decel * dt * 3;
-        } else {
-            speed -= performance->decel * dt;
-        }
-
-        if ( speed < tgt_speed )
-            speed = tgt_speed;
-
-    }
-}
-
-
 void FGAIAircraft::updatePosition() {
     // convert speed to degrees per second
     double speed_north_deg_sec = cos( hdg * SGD_DEGREES_TO_RADIANS )
@@ -870,32 +851,65 @@ void FGAIAircraft::updateHeading() {
         //else
         //  turnConstant = 0.088362;
         // If on ground, calculate heading change directly
-        if (no_roll) {
+        if (onGround()) {
             double headingDiff = fabs(hdg-tgt_heading);
+            double bank_sense = 0.0;
+        /*
+        double diff = fabs(hdg - tgt_heading);
+        if (diff > 180)
+            diff = fabs(diff - 360);
 
+        double sum = hdg + diff;
+        if (sum > 360.0)
+            sum -= 360.0;
+        if (fabs(sum - tgt_heading) < 1.0) {
+            bank_sense = 1.0;    // right turn
+        } else {
+            bank_sense = -1.0;   // left turn
+        }*/
             if (headingDiff > 180)
                 headingDiff = fabs(headingDiff - 360);
-
-            groundTargetSpeed = tgt_speed - (tgt_speed * (headingDiff/45));
+            double sum = hdg + headingDiff;
+            if (sum > 360.0) 
+                sum -= 360.0;
+            if (fabs(sum - tgt_heading) > 0.0001) {
+                bank_sense = -1.0;
+            } else {
+                bank_sense = 1.0;
+            }
+            //if (trafficRef)
+            	//cerr << trafficRef->getCallSign() << " Heading " 
+                //     << hdg << ". Target " << tgt_heading <<  ". Diff " << fabs(sum - tgt_heading) << ". Speed " << speed << endl;
+            //if (headingDiff > 60) {
+            groundTargetSpeed = tgt_speed; // * cos(headingDiff * SG_DEGREES_TO_RADIANS);
+                //groundTargetSpeed = tgt_speed - tgt_speed * (headingDiff/180);
+            //} else {
+            //    groundTargetSpeed = tgt_speed;
+            //}
             if (sign(groundTargetSpeed) != sign(tgt_speed))
                 groundTargetSpeed = 0.21 * sign(tgt_speed); // to prevent speed getting stuck in 'negative' mode
 
             if (headingDiff > 30.0) {
                 // invert if pushed backward
-                headingChangeRate += dt * sign(roll);
-
-                if (headingChangeRate > 30)
-                    headingChangeRate = 30;
-                else if (headingChangeRate < -30)
-                    headingChangeRate = -30;
+                headingChangeRate += 10.0 * dt * sign(roll);
 
+                // Clamp the maximum steering rate to 30 degrees per second,
+                // But only do this when the heading error is decreasing.
+                if ((headingDiff < headingError)) {
+                    if (headingChangeRate > 30)
+                        headingChangeRate = 30;
+                    else if (headingChangeRate < -30)
+                        headingChangeRate = -30;
+                }
             } else {
-                if (fabs(headingChangeRate) > headingDiff)
-                    headingChangeRate = headingDiff*sign(roll);
-                else
-                    headingChangeRate += dt * sign(roll);
+                   if (fabs(headingChangeRate) > headingDiff)
+                       headingChangeRate = headingDiff*sign(roll);
+                   else
+                       headingChangeRate += dt * sign(roll);
             }
-            hdg += headingChangeRate * dt;
+
+	    hdg += headingChangeRate * dt * (fabs(speed) / 15);
+            headingError = headingDiff;
         } else {
             if (fabs(speed) > 1.0) {
                 turn_radius_ft = 0.088362 * speed * speed
@@ -921,7 +935,7 @@ void FGAIAircraft::updateHeading() {
 }
 
 
-void FGAIAircraft::updateBankAngles() {
+void FGAIAircraft::updateBankAngleTarget() {
     // adjust target bank angle if heading lock engaged
     if (hdg_lock) {
         double bank_sense = 0.0;
@@ -937,55 +951,37 @@ void FGAIAircraft::updateBankAngles() {
         } else {
             bank_sense = -1.0;   // left turn
         }
-        if (diff < 30) {
+        if (diff < _performance->maximumBankAngle()) {
             tgt_roll = diff * bank_sense;
         } else {
-            tgt_roll = 30.0 * bank_sense;
+            tgt_roll = _performance->maximumBankAngle() * bank_sense;
         }
-        if ((fabs((double) spinCounter) > 1) && (diff > 30)) {
+        if ((fabs((double) spinCounter) > 1) && (diff > _performance->maximumBankAngle())) {
             tgt_speed *= 0.999;  // Ugly hack: If aircraft get stuck, they will continually spin around.
             // The only way to resolve this is to make them slow down.
         }
     }
-
-    // adjust bank angle, use 9 degrees per second
-    double bank_diff = tgt_roll - roll;
-    if (fabs(bank_diff) > 0.2) {
-        if (bank_diff > 0.0)
-            roll += 9.0 * dt;
-
-        if (bank_diff < 0.0)
-            roll -= 9.0 * dt;
-        //while (roll > 180) roll -= 360;
-        //while (roll < 180) roll += 360;
-    }
 }
 
 
-void FGAIAircraft::updateAltitudes() {
-    // adjust altitude (meters) based on current vertical speed (fpm)
-    altitude_ft += vs / 60.0 * dt;
-    pos.setElevationFt(altitude_ft);
-
+void FGAIAircraft::updateVerticalSpeedTarget() {
     // adjust target Altitude, based on ground elevation when on ground
-    if (no_roll) {
+    if (onGround()) {
         getGroundElev(dt);
         doGroundAltitude();
-    } else {
-        // find target vertical speed if altitude lock engaged
-        if (alt_lock && use_perf_vs) {
+    } else if (alt_lock) {
+        // find target vertical speed
+        if (use_perf_vs) {
             if (altitude_ft < tgt_altitude_ft) {
                 tgt_vs = tgt_altitude_ft - altitude_ft;
-                if (tgt_vs > performance->climb_rate)
-                    tgt_vs = performance->climb_rate;
+                if (tgt_vs > _performance->climbRate())
+                    tgt_vs = _performance->climbRate();
             } else {
                 tgt_vs = tgt_altitude_ft - altitude_ft;
-                if (tgt_vs  < (-performance->descent_rate))
-                    tgt_vs = -performance->descent_rate;
+                if (tgt_vs  < (-_performance->descentRate()))
+                    tgt_vs = -_performance->descentRate();
             }
-        }
-
-        if (alt_lock && !use_perf_vs) {
+        } else {
             double max_vs = 4*(tgt_altitude_ft - altitude_ft);
             double min_vs = 100;
             if (tgt_altitude_ft < altitude_ft)
@@ -997,34 +993,78 @@ void FGAIAircraft::updateAltitudes() {
             if (fabs(tgt_vs) < fabs(min_vs))
                 tgt_vs = min_vs;
         }
-    }
+    } //else 
+    //    tgt_vs = 0.0;
 }
 
+void FGAIAircraft::updatePitchAngleTarget() {
+    // if on ground and above vRotate -> initial rotation
+    if (onGround() && (speed > _performance->vRotate()))
+        tgt_pitch = 8.0; // some rough B737 value 
 
-void FGAIAircraft::updateVerticalSpeed() {
-    // adjust vertical speed
-    double vs_diff = tgt_vs - vs;
-    if (fabs(vs_diff) > 10.0) {
-        if (vs_diff > 0.0) {
-            vs += (performance->climb_rate / 3.0) * dt;
-
-            if (vs > tgt_vs)
-                vs = tgt_vs;
-        } else {
-            vs -= (performance->descent_rate / 3.0) * dt;
-
-            if (vs < tgt_vs)
-                vs = tgt_vs;
-        }
-    }
-}
-
-
-void FGAIAircraft::matchPitchAngle() {
+    //TODO pitch angle on approach and landing
+    
     // match pitch angle to vertical speed
-    if (vs > 0) {
-        pitch = vs * 0.005;
+    else if (tgt_vs > 0) {
+        tgt_pitch = tgt_vs * 0.005;
     } else {
-        pitch = vs * 0.002;
+        tgt_pitch = tgt_vs * 0.002;
     }
 }
+
+string FGAIAircraft::atGate() {
+     string tmp("");
+     if (fp->getLeg() < 3) {
+         if (trafficRef) {
+             if (fp->getGate() > 0) {
+                 FGParking *park =
+                     trafficRef->getDepartureAirport()->getDynamics()->getParking(fp->getGate());
+                 tmp = park->getName();
+             }
+         }
+     }
+     return tmp;
+}
+
+void FGAIAircraft::handleATCRequests() {
+    //TODO implement NullController for having no ATC to save the conditionals
+    if (controller) {
+        controller->update(getID(),
+                           pos.getLatitudeDeg(),
+                           pos.getLongitudeDeg(),
+                           hdg,
+                           speed,
+                           altitude_ft, dt);
+        processATC(controller->getInstruction(getID()));
+    }
+}
+
+void FGAIAircraft::updateActualState() {
+    //update current state
+    //TODO have a single tgt_speed and check speed limit on ground on setting tgt_speed
+    updatePosition();
+
+    if (onGround())
+        speed = _performance->actualSpeed(this, groundTargetSpeed, dt);
+    else
+        speed = _performance->actualSpeed(this, tgt_speed, dt);
+
+    updateHeading();
+    roll = _performance->actualBankAngle(this, tgt_roll, dt);
+
+    // adjust altitude (meters) based on current vertical speed (fpm)
+    altitude_ft += vs / 60.0 * dt;
+    pos.setElevationFt(altitude_ft);
+
+    vs = _performance->actualVerticalSpeed(this, tgt_vs, dt);
+    pitch = _performance->actualPitch(this, tgt_pitch, dt);
+}
+
+void FGAIAircraft::updateSecondaryTargetValues() {
+    // derived target state values
+    updateBankAngleTarget();
+    updateVerticalSpeedTarget();
+    updatePitchAngleTarget();
+
+    //TODO calculate wind correction angle (tgt_yaw)
+}
