@@ -17,9 +17,15 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  **************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include "AIFlightPlan.hxx"
 #include <simgear/math/sg_geodesy.hxx>
 #include <Airports/runways.hxx>
+#include <Airports/dynamics.hxx>
 
 #include <Environment/environment_mgr.hxx>
 #include <Environment/environment.hxx>
@@ -53,16 +59,16 @@ void FGAIFlightPlan::create(FGAirport *dep, FGAirport *arr, int legNr,
 		 radius, fltType, aircraftType, airline);
       break;
     case 3: 
-      createTakeOff(firstFlight, dep, speed);
+      createTakeOff(firstFlight, dep, speed, fltType);
       break;
     case 4: 
-      createClimb(firstFlight, dep, speed, alt);
+      createClimb(firstFlight, dep, speed, alt, fltType);
       break;
     case 5: 
-      createCruise(firstFlight, dep,arr, latitude, longitude, speed, alt);
+      createCruise(firstFlight, dep,arr, latitude, longitude, speed, alt, fltType);
       break;
     case 6: 
-      createDecent(arr);
+      createDecent(arr, fltType);
       break;
     case 7: 
       createLanding(arr);
@@ -83,100 +89,10 @@ void FGAIFlightPlan::create(FGAirport *dep, FGAirport *arr, int legNr,
   leg++;
 }
 
-/*******************************************************************
- * createPushBack
- * initialize the Aircraft at the parking location
- ******************************************************************/
-void FGAIFlightPlan::createPushBack(bool firstFlight, FGAirport *dep, 
- 				    double latitude,
- 				    double longitude,
- 				    double radius,
- 				    const string& fltType,
- 				    const string& aircraftType,
- 				    const string& airline)
-{
-  double heading;
-  double lat;
-  double lon;
-  double lat2;
-  double lon2;
-  double az2;
-  
-  //int currWpt = wpt_iterator - waypoints.begin();
-  // Erase all existing waypoints.
-  //resetWaypoints();
-  
-  // We only need to get a valid parking if this is the first leg. 
-  // Otherwise use the current aircraft position.
-  if (firstFlight)
-    {
-      if (!(dep->getDynamics()->getAvailableParking(&lat, &lon, 
- 						    &heading, &gateId, 
- 						    radius, fltType, 
- 						    aircraftType, airline)))
- 	{
- 	  SG_LOG(SG_INPUT, SG_ALERT, "Could not find parking for a " << 
- 		 aircraftType <<
- 		 " of flight type " << fltType << 
- 		 " of airline     " << airline <<
- 		 " at airport     " << dep->getId());
-	  //exit(1);
- 	}
-    }
-  else
-    {
-      dep->getDynamics()->getParking(gateId, &lat, &lon, &heading);
-    }
-  heading += 180.0;
-  if (heading > 360)
-    heading -= 360;
-  waypoint *wpt = new waypoint;
-  wpt->name      = "park";
-  wpt->latitude  = lat;
-  wpt->longitude = lon;
-  wpt->altitude  = dep->getElevation();
-  wpt->speed     = -10; 
-  wpt->crossat   = -10000;
-  wpt->gear_down = true;
-  wpt->flaps_down= true;
-  wpt->finished  = false;
-  wpt->on_ground = true;
 
-  waypoints.push_back(wpt); 
-  
-  geo_direct_wgs_84 ( 0, lat, lon, heading, 
- 		      10, 
- 		      &lat2, &lon2, &az2 );
-  wpt = new waypoint;
-  wpt->name      = "park2";
-  wpt->latitude  = lat2;
-  wpt->longitude = lon2;
-  wpt->altitude  = dep->getElevation();
-  wpt->speed     = -10; 
-  wpt->crossat   = -10000;
-  wpt->gear_down = true;
-  wpt->flaps_down= true;
-  wpt->finished  = false;
-  wpt->on_ground = true;
-  wpt->routeIndex = 0;
-  waypoints.push_back(wpt); 
-  geo_direct_wgs_84 ( 0, lat, lon, heading, 
- 		      2.2*radius,           
- 		      &lat2, &lon2, &az2 );
-  wpt = new waypoint;
-  wpt->name      = "taxiStart";
-  wpt->latitude  = lat2;
-  wpt->longitude = lon2;
-  wpt->altitude  = dep->getElevation();
-  wpt->speed     = 10; 
-  wpt->crossat   = -10000;
-  wpt->gear_down = true;
-  wpt->flaps_down= true;
-  wpt->finished  = false;
-  wpt->on_ground = true;
-  wpt->routeIndex = 0;
-  waypoints.push_back(wpt);  
-}
+
+
+
 
 /*******************************************************************
  * createCreate Taxi. 
@@ -225,26 +141,16 @@ void FGAIFlightPlan::createTaxi(bool firstFlight, int direction,
 	  //wpt->on_ground = true;
 	  //waypoints.push_back(wpt);  
 	}
-      // "NOTE: this is currently fixed to "com" for commercial traffic
-      // Should be changed to be used dynamically to allow "gen" and "mil"
-      // as well
-      apt->getDynamics()->getActiveRunway("com", 1, activeRunway);
-      if (!(globals->get_runways()->search(apt->getId(), 
-					    activeRunway, 
-					    &rwy)))
-	{
-	   SG_LOG(SG_INPUT, SG_ALERT, "Failed to find runway " << 
-		  activeRunway << 
-		  " at airport     " << apt->getId());
-	   exit(1);
-	} 
-
+      string rwyClass = getRunwayClassFromTrafficType(fltType);
+      apt->getDynamics()->getActiveRunway(rwyClass, 1, activeRunway);
+      rwy = apt->getRunwayByIdent(activeRunway);
+      
       // Determine the beginning of he runway
-      heading = rwy._heading;
+      heading = rwy->headingDeg();
       double azimuth = heading + 180.0;
       while ( azimuth >= 360.0 ) { azimuth -= 360.0; }
-      geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, azimuth, 
-			  rwy._length * SG_FEET_TO_METER * 0.5 - 5.0,
+      geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), azimuth, 
+			  rwy->lengthM() * 0.5 - 5.0,
 			  &lat2, &lon2, &az2 );
 
       if (apt->getDynamics()->getGroundNetwork()->exists())
@@ -261,11 +167,29 @@ void FGAIFlightPlan::createTaxi(bool firstFlight, int direction,
 	  //FGTaxiRoute route;
 	  delete taxiRoute;
 	  taxiRoute = new FGTaxiRoute;
-	  if (gateId >= 0)
-	    *taxiRoute = apt->getDynamics()->getGroundNetwork()->findShortestRoute(gateId, 
-									      runwayId);
-	  else
-	    *taxiRoute = apt->getDynamics()->getGroundNetwork()->findShortestRoute(0, runwayId);
+
+          // Determine which node to start from.
+          int node;
+          // Find out which node to start from
+          FGParking *park = apt->getDynamics()->getParking(gateId);
+          if (park)
+          	node = park->getPushBackPoint();
+          else
+		node = 0;
+          if (node == -1)
+	     node = gateId;
+
+          // HAndle case where parking doens't have a node
+          if ((node == 0) && park) {
+                if (firstFlight) {
+                    node = gateId;
+                } else {
+		    node = lastNodeVisited;
+                }
+          }
+
+          //cerr << "Using node " << node << endl;
+	    *taxiRoute = apt->getDynamics()->getGroundNetwork()->findShortestRoute(node, runwayId);
 	  intVecIterator i;
 	 
 	  if (taxiRoute->empty()) {
@@ -301,30 +225,27 @@ void FGAIFlightPlan::createTaxi(bool firstFlight, int direction,
 	  } else {
 	    int node;
 	    taxiRoute->first();
-	    bool isPushBackPoint = false;
+	    //bool isPushBackPoint = false;
 	    if (firstFlight) {
+
 	      // If this is called during initialization, randomly
 	      // skip a number of waypoints to get a more realistic
 	      // taxi situation.
-	      isPushBackPoint = true;
+	      //isPushBackPoint = true;
 	      int nrWaypoints = taxiRoute->size();
 	      nrWaypointsToSkip = rand() % nrWaypoints;
 	      // but make sure we always keep two active waypoints
 	      // to prevent a segmentation fault
 	      for (int i = 0; i < nrWaypointsToSkip-2; i++) {
-		isPushBackPoint = false;
+		//isPushBackPoint = false;
 		taxiRoute->next(&node);
 	      }
+	      apt->getDynamics()->releaseParking(gateId);
 	    } else {
-	      //chop off the first two waypoints, because
-	      // those have already been created
-	      // by create pushback
-	      int size = taxiRoute->size();
-	      if (size > 2) {
-		taxiRoute->next(&node);
-		taxiRoute->next(&node);
-	      }
-	    }
+               if (taxiRoute->size() > 1) {
+                   taxiRoute->next(&node); // chop off the first waypoint, because that is already the last of the pushback route
+               }
+            }
 	    int route;
 	    while(taxiRoute->next(&node, &route))
 	      {
@@ -340,13 +261,7 @@ void FGAIFlightPlan::createTaxi(bool firstFlight, int direction,
 		// Elevation is currently disregarded when on_ground is true
 		// because the AIModel obtains a periodic ground elevation estimate.
 		wpt->altitude  = apt->getElevation();
-		if (isPushBackPoint) {
-		  wpt->speed = -10;
-		  isPushBackPoint = false;
-		}
-		else {
-		  wpt->speed     = 15; 
-		}
+		wpt->speed     = 15; 
 		wpt->crossat   = -10000;
 		wpt->gear_down = true;
 		wpt->flaps_down= true;
@@ -355,6 +270,7 @@ void FGAIFlightPlan::createTaxi(bool firstFlight, int direction,
 		wpt->routeIndex = route;
 		waypoints.push_back(wpt);
 	      }
+             /*
 	    //cerr << endl;
 	    // finally, rewind the taxiRoute object to the point where we started
 	    // generating the Flightplan, for AI use.
@@ -371,7 +287,7 @@ void FGAIFlightPlan::createTaxi(bool firstFlight, int direction,
 		//taxiRoute->next(&node);	
 		//taxiRoute->next(&node);
 	      }
-	    }
+	    }*/
 	  } // taxiRoute not empty
 	}
       else 
@@ -570,7 +486,7 @@ void FGAIFlightPlan::createTaxi(bool firstFlight, int direction,
  * CreateTakeOff 
  * initialize the Aircraft at the parking location
  ******************************************************************/
-void FGAIFlightPlan::createTakeOff(bool firstFlight, FGAirport *apt, double speed)
+void FGAIFlightPlan::createTakeOff(bool firstFlight, FGAirport *apt, double speed, const string &fltType)
 {
   double heading;
   double lat, lon, az;
@@ -583,26 +499,16 @@ void FGAIFlightPlan::createTakeOff(bool firstFlight, FGAirport *apt, double spee
   if (firstFlight)
     {
       //string name;
-       // "NOTE: this is currently fixed to "com" for commercial traffic
-      // Should be changed to be used dynamically to allow "gen" and "mil"
-      // as well
-      apt->getDynamics()->getActiveRunway("com", 1, activeRunway);
-	if (!(globals->get_runways()->search(apt->getId(), 
-					      activeRunway, 
-					      &rwy)))
-	  {
-	    SG_LOG(SG_INPUT, SG_ALERT, "Failed to find runway " << 
-		   activeRunway << 
-		   " at airport     " << apt->getId());
-	    exit(1);
-	  }
+      string rwyClass = getRunwayClassFromTrafficType(fltType);
+      apt->getDynamics()->getActiveRunway(rwyClass, 1, activeRunway);
+      rwy = apt->getRunwayByIdent(activeRunway);
     }
   // Acceleration point, 105 meters into the runway,
-  heading = rwy._heading;
+  heading = rwy->headingDeg();
   double azimuth = heading + 180.0;
   while ( azimuth >= 360.0 ) { azimuth -= 360.0; }
-  geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, azimuth, 
-		      rwy._length * SG_FEET_TO_METER * 0.5 - 105.0,
+  geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), azimuth, 
+		      rwy->lengthM() * 0.5 - 105.0,
 		      &lat2, &lon2, &az2 );
   wpt = new waypoint; 
   wpt->name      = "accel"; 
@@ -631,8 +537,8 @@ void FGAIFlightPlan::createTakeOff(bool firstFlight, FGAirport *apt, double spee
   
   wpt = new waypoint;
   wpt->name      = "SOC";
-  wpt->latitude  = rwy._lat;
-  wpt->longitude = rwy._lon;
+  wpt->latitude  = rwy->latitude();
+  wpt->longitude = rwy->longitude();
   wpt->altitude  = apt->getElevation()+1000;
   wpt->speed     = speed; 
   wpt->crossat   = -10000;
@@ -644,8 +550,8 @@ void FGAIFlightPlan::createTakeOff(bool firstFlight, FGAirport *apt, double spee
   waypoints.push_back(wpt);
 
 
- geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, heading, 
-  rwy._length * SG_FEET_TO_METER,
+ geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), heading, 
+  rwy->lengthM(),
   &lat2, &lon2, &az2 );
 
   wpt = new waypoint;
@@ -666,7 +572,7 @@ void FGAIFlightPlan::createTakeOff(bool firstFlight, FGAirport *apt, double spee
   // Tower control until they have reached the 3000 ft climb point
 
 
- geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, heading, 
+ geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), heading, 
   5000,
   &lat2, &lon2, &az2 );
 
@@ -676,7 +582,7 @@ void FGAIFlightPlan::createTakeOff(bool firstFlight, FGAirport *apt, double spee
   wpt->latitude  = lat2;
   wpt->longitude = lon2;
   wpt->altitude  = apt->getElevation()+5000;
-  wpt->speed     = speed; 
+  wpt->speed     = speed;
   wpt->crossat   = -10000;
   wpt->gear_down = true;
   wpt->flaps_down= true;
@@ -684,8 +590,13 @@ void FGAIFlightPlan::createTakeOff(bool firstFlight, FGAirport *apt, double spee
   wpt->on_ground = false;
   wpt->routeIndex = 0;
   waypoints.push_back(wpt);  
+  //cerr << "Created takeoff plan : " << endl;
+  //for (wpt_vector_iterator i = waypoints.begin(); i != waypoints.end(); i++) {
+  //     cerr << "Waypoint Name: " << (*i)->name << ". Speed = " << (*i)->speed << endl;
+  //}
 
- //  geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, heading, 
+
+ //  geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), heading, 
 //   100000,
 //   &lat2, &lon2, &az2 );
 //   wpt = new waypoint;
@@ -707,7 +618,7 @@ void FGAIFlightPlan::createTakeOff(bool firstFlight, FGAirport *apt, double spee
  * CreateClimb
  * initialize the Aircraft at the parking location
  ******************************************************************/
-void FGAIFlightPlan::createClimb(bool firstFlight, FGAirport *apt, double speed, double alt)
+void FGAIFlightPlan::createClimb(bool firstFlight, FGAirport *apt, double speed, double alt, const string &fltType)
 {
   double heading;
   //FGRunway rwy;
@@ -719,27 +630,17 @@ void FGAIFlightPlan::createClimb(bool firstFlight, FGAirport *apt, double speed,
   if (firstFlight)
     {
       //string name;
-      // "NOTE: this is currently fixed to "com" for commercial traffic
-      // Should be changed to be used dynamically to allow "gen" and "mil"
-      // as well
-      apt->getDynamics()->getActiveRunway("com", 1, activeRunway);
-	if (!(globals->get_runways()->search(apt->getId(), 
-					      activeRunway, 
-					      &rwy)))
-	  {
-	    SG_LOG(SG_INPUT, SG_ALERT, "Failed to find runway " << 
-		   activeRunway << 
-		   " at airport     " << apt->getId());
-	    exit(1);
-	  }
+      string rwyClass = getRunwayClassFromTrafficType(fltType);
+      apt->getDynamics()->getActiveRunway(rwyClass, 1, activeRunway);
+      rwy = apt->getRunwayByIdent(activeRunway);
     }
   
   
-  heading = rwy._heading;
+  heading = rwy->headingDeg();
   double azimuth = heading + 180.0;
   while ( azimuth >= 360.0 ) { azimuth -= 360.0; }
   //cerr << "Creating climb at : " << rwy._id << " " << rwy._rwy_no << endl;
-  geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, heading, 
+  geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), heading, 
 		      10*SG_NM_TO_METER,
 		      &lat2, &lon2, &az2 );
   wpt = new waypoint;
@@ -757,7 +658,7 @@ void FGAIFlightPlan::createClimb(bool firstFlight, FGAirport *apt, double speed,
   waypoints.push_back(wpt); 
   
 
-  geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, heading, 
+  geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), heading, 
 		      20*SG_NM_TO_METER,
 		      &lat2, &lon2, &az2 );
   wpt = new waypoint;
@@ -818,12 +719,12 @@ void FGAIFlightPlan::createClimb(bool firstFlight, FGAirport *apt, double speed,
 // 	     " at airport     " << arr->getId());
 //       exit(1);
 //     }
-//   heading = rwy._heading;
+//   heading = rwy->headingDeg();
 //   azimuth = heading + 180.0;
 //   while ( azimuth >= 360.0 ) { azimuth -= 360.0; }
   
   
-//   geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, azimuth, 
+//   geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), azimuth, 
 // 		      110000,
 // 		      &lat2, &lon2, &az2 );
 //   wpt = new waypoint;
@@ -844,7 +745,7 @@ void FGAIFlightPlan::createClimb(bool firstFlight, FGAirport *apt, double speed,
  * CreateDecent
  * initialize the Aircraft at the parking location
  ******************************************************************/
-void FGAIFlightPlan::createDecent(FGAirport *apt)
+void FGAIFlightPlan::createDecent(FGAirport *apt, const string &fltType)
 {
 
   // Ten thousand ft. Slowing down to 240 kts
@@ -858,21 +759,14 @@ void FGAIFlightPlan::createDecent(FGAirport *apt)
   //Beginning of Decent
   //string name;
   // allow "mil" and "gen" as well
-  apt->getDynamics()->getActiveRunway("com", 2, activeRunway);
-  if (!(globals->get_runways()->search(apt->getId(), 
-				       activeRunway, 
-				       &rwy)))
-    {
-      SG_LOG(SG_INPUT, SG_ALERT, "Failed to find runway " << 
-	     activeRunway << 
-	     " at airport     " << apt->getId());
-      exit(1);
-    }
-  
-  heading = rwy._heading;
+  string rwyClass = getRunwayClassFromTrafficType(fltType);
+  apt->getDynamics()->getActiveRunway(rwyClass, 2, activeRunway);
+  rwy = apt->getRunwayByIdent(activeRunway);
+     
+  heading = rwy->headingDeg();
   azimuth = heading + 180.0;
   while ( azimuth >= 360.0 ) { azimuth -= 360.0; }
-  geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, azimuth, 
+  geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), azimuth, 
 		      100000,
 		      &lat2, &lon2, &az2 );
   
@@ -891,7 +785,7 @@ void FGAIFlightPlan::createDecent(FGAirport *apt)
   waypoints.push_back(wpt);  
   
   // Three thousand ft. Slowing down to 160 kts
-  geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, azimuth, 
+  geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), azimuth, 
 		      8*SG_NM_TO_METER,
 		      &lat2, &lon2, &az2 );
   wpt = new waypoint;
@@ -923,13 +817,13 @@ void FGAIFlightPlan::createLanding(FGAirport *apt)
   waypoint *wpt;
 
   
-  heading = rwy._heading;
+  heading = rwy->headingDeg();
   azimuth = heading + 180.0;
   while ( azimuth >= 360.0 ) { azimuth -= 360.0; }
 
   //Runway Threshold
- geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, azimuth, 
-		     rwy._length*0.45 * SG_FEET_TO_METER,
+ geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), azimuth, 
+		     rwy->lengthM() *0.45,
 		     &lat2, &lon2, &az2 );
   wpt = new waypoint;
   wpt->name      = "Threshold"; //wpt_node->getStringValue("name", "END");
@@ -946,13 +840,13 @@ void FGAIFlightPlan::createLanding(FGAirport *apt)
   waypoints.push_back(wpt); 
 
  //Full stop at the runway centerpoint
- geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, azimuth, 
-		     rwy._length*0.45,
+ geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), azimuth, 
+		     rwy->lengthFt() *0.45,
 		      &lat2, &lon2, &az2 );
   wpt = new waypoint;
   wpt->name      = "Center"; //wpt_node->getStringValue("name", "END");
-  wpt->latitude  = rwy._lat;
-  wpt->longitude = rwy._lon;
+  wpt->latitude  = rwy->latitude();
+  wpt->longitude = rwy->longitude();
   wpt->altitude  = apt->getElevation();
   wpt->speed     = 30; 
   wpt->crossat   = -10000;
@@ -963,8 +857,8 @@ void FGAIFlightPlan::createLanding(FGAirport *apt)
   wpt->routeIndex = 0;
   waypoints.push_back(wpt);
 
- geo_direct_wgs_84 ( 0, rwy._lat, rwy._lon, heading, 
-		     rwy._length*0.45 * SG_FEET_TO_METER,
+ geo_direct_wgs_84 ( 0, rwy->latitude(), rwy->longitude(), heading, 
+		     rwy->lengthM() *0.45,
 		     &lat2, &lon2, &az2 );
   wpt = new waypoint;
   wpt->name      = "Threshold"; //wpt_node->getStringValue("name", "END");
@@ -1043,3 +937,43 @@ void FGAIFlightPlan::createParking(FGAirport *apt, double radius)
   wpt->routeIndex = 0;
   waypoints.push_back(wpt);
 }
+
+/**
+ *
+ * @param fltType a string describing the type of
+ * traffic, normally used for gate assignments
+ * @return a converted string that gives the runway
+ * preference schedule to be used at aircraft having
+ * a preferential runway schedule implemented (i.e.
+ * having a rwyprefs.xml file
+ * 
+ * Currently valid traffic types for gate assignment:
+ * - gate (commercial gate)
+ * - cargo (commercial gargo),
+ * - ga (general aviation) ,
+ * - ul (ultralight),
+ * - mil-fighter (military - fighter),
+ * - mil-transport (military - transport)
+ *
+ * Valid runway classes:
+ * - com (commercial traffic: jetliners, passenger and cargo)
+ * - gen (general aviation)
+ * - ul (ultralight: I can imagine that these may share a runway with ga on some airports)
+ * - mil (all military traffic)
+ */
+string FGAIFlightPlan::getRunwayClassFromTrafficType(string fltType)
+{
+    if ((fltType == "gate") || (fltType == "cargo")) { 
+	return string("com");
+    }
+    if (fltType == "ga") {
+        return string ("gen");
+    }
+    if (fltType == "ul") {
+        return string("ul");
+    }
+    if ((fltType == "mil-fighter") || (fltType == "mil-transport")) { 
+	return string("mil");
+    }
+   return string("com");
+}
