@@ -18,6 +18,10 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+#include <simgear/math/sg_geodesy.hxx>
+#include <simgear/props/props_io.hxx>
+#include <simgear/structure/exception.hxx>
+
 #include <Main/globals.hxx>
 
 #include <Airports/simple.hxx>
@@ -33,9 +37,7 @@
 #include "AIStatic.hxx"
 #include "AIMultiplayer.hxx"
 #include "AITanker.hxx"
-
-#include <simgear/math/sg_geodesy.hxx>
-
+#include "AIWingman.hxx"
 
 FGAIManager::FGAIManager() {
     _dt = 0.0;
@@ -73,6 +75,7 @@ FGAIManager::init() {
     user_heading_node   = fgGetNode("/orientation/heading-deg", true);
     user_pitch_node     = fgGetNode("/orientation/pitch-deg", true);
     user_yaw_node       = fgGetNode("/orientation/side-slip-deg", true);
+    user_roll_node      = fgGetNode("/orientation/roll-deg", true);
     user_speed_node     = fgGetNode("/velocities/uBody-fps", true);
 }
 
@@ -143,7 +146,7 @@ FGAIManager::update(double dt) {
             tmgr->release((*ai_list_itr)->getID());
             --mNumAiModels;
             --(mNumAiTypeModels[(*ai_list_itr)->getType()]);
-            FGAIBase *base = *ai_list_itr;
+            FGAIBase *base = (*ai_list_itr).get();
             SGPropertyNode *props = base->_getProps();
 
             props->setBoolValue("valid", false);
@@ -160,7 +163,7 @@ FGAIManager::update(double dt) {
         } else {
             fetchUserState();
             if ((*ai_list_itr)->isa(FGAIBase::otThermal)) {
-                FGAIBase *base = *ai_list_itr;
+                FGAIBase *base = (*ai_list_itr).get();
                 processThermal((FGAIThermal*)base);
             } else {
                 (*ai_list_itr)->update(_dt);
@@ -173,7 +176,7 @@ FGAIManager::update(double dt) {
 }
 
 void
-FGAIManager::attach(SGSharedPtr<FGAIBase> model)
+FGAIManager::attach(FGAIBase *model)
 {
     //unsigned idx = mNumAiTypeModels[model->getType()];
     const char* typeString = model->getTypeString();
@@ -238,6 +241,7 @@ FGAIManager::fetchUserState( void ) {
     user_pitch     = user_pitch_node->getDoubleValue();
     user_yaw       = user_yaw_node->getDoubleValue();
     user_speed     = user_speed_node->getDoubleValue() * 0.592484;
+    user_roll      = user_roll_node->getDoubleValue();
     wind_from_east = wind_from_east_node->getDoubleValue();
     wind_from_north = wind_from_north_node->getDoubleValue();
 }
@@ -274,12 +278,17 @@ FGAIManager::processScenario( const string &filename ) {
             continue;
         std::string type = scEntry->getStringValue("type", "aircraft");
 
-        if (type == "tanker") { // refueling scenarios
-            FGAITanker* tanker = new FGAITanker;
+         if (type == "tanker") { // refueling scenarios
+	    FGAITanker* tanker = new FGAITanker;
             tanker->readFromScenario(scEntry);
             attach(tanker);
 
-        } else if (type == "aircraft") {
+        } else if (type == "wingman") {
+            FGAIWingman* wingman = new FGAIWingman;
+            wingman->readFromScenario(scEntry);
+            attach(wingman);
+
+    } else if (type == "aircraft") {
             FGAIAircraft* aircraft = new FGAIAircraft;
             aircraft->readFromScenario(scEntry);
             attach(aircraft);
@@ -327,51 +336,13 @@ FGAIManager::loadScenarioFile(const std::string& filename)
         SGPropertyNode_ptr root = new SGPropertyNode;
         readProperties(path.str(), root);
         return root;
-    } catch (const sg_exception &e) {
+    } catch (const sg_exception &) {
         SG_LOG(SG_GENERAL, SG_DEBUG, "Incorrect path specified for AI "
             "scenario: \"" << path.str() << "\"");
         return 0;
     }
 }
 
-// This code keeps track of models that have already been loaded
-// Eventually we'd prbably need to find a way to keep track of models
-// that are unloaded again
-ssgBranch * FGAIManager::getModel(const string& path)
-{
-    ModelVecIterator i = loadedModels.begin();
-    int count = 0;
-
-    while (i != loadedModels.end())
-    {
-        count += i->getNumRefs() -1;
-
-        if (i->getNumRefs() == 1){
-            i = loadedModels.erase(i);
-            //cerr << "[ Deleted ]" << endl;
-        } else
-            i++;
-
-    }
-    //cerr << "Reference summary end : " << count << "models allocated" << endl;
-    i = loadedModels.begin();
-
-    while (i != loadedModels.end())
-    {
-        if (i->getPath() == path)
-            return i->getModelId();
-        i++;
-    }
-
-    return 0;
-}
-
-void
-FGAIManager::setModel(const string& path, ssgBranch *model)
-{
-    loadedModels.push_back(FGModelID(path,model));
-}
-
 bool
 FGAIManager::getStartPosition(const string& id, const string& pid,
                               SGGeod& geodPos, double& hdng, SGVec3d& uvw)
@@ -395,7 +366,7 @@ FGAIManager::getStartPosition(const string& id, const string& pid,
                         std::string pnumber = scEntry->getStringValue("pennant-number");
                         std::string name = scEntry->getStringValue("name");
                         if (type == "carrier" && (pnumber == id || name == id)) {
-                            SGSharedPtr<FGAICarrier> carrier = new FGAICarrier;
+                            osg::ref_ptr<FGAICarrier> carrier = new FGAICarrier;
                             carrier->readFromScenario(scEntry);
 
                             if (carrier->getParkPosition(pid, geodPos, hdng, uvw)) {
@@ -415,9 +386,8 @@ const FGAIBase *
 FGAIManager::calcCollision(double alt, double lat, double lon, double fuse_range)
 {
     // we specify tgt extent (ft) according to the AIObject type
-    double tgt_ht[]     = {0, 50 ,100, 250, 0, 100, 0, 0, 50, 50};
-    double tgt_length[] = {0, 100, 200, 750, 0, 50, 0, 0, 200, 100};
-
+    double tgt_ht[]     = {0, 50 ,100, 250, 0, 100, 0, 0, 50, 50, 50};
+    double tgt_length[] = {0, 100, 200, 750, 0, 50, 0, 0, 200, 100, 100};
     ai_list_iterator ai_list_itr = ai_list.begin();
     ai_list_iterator end = ai_list.end();
 
@@ -461,7 +431,7 @@ FGAIManager::calcCollision(double alt, double lat, double lon, double fuse_range
                 << " range " << range
                 << " alt " << tgt_alt
                 );
-            return *ai_list_itr;
+            return (*ai_list_itr).get();
         }
         ++ai_list_itr;
     }
