@@ -174,9 +174,10 @@ void FGAIShip::unbind() {
     props->untie("position/waypoint-range-rate-nm-sec");
     props->untie("position/waypoint-new");
     props->untie("position/waypoint-missed");
+    props->untie("position/waypoint-missed-count");
+    props->untie("position/waypoint-missed-time-sec");
     props->untie("position/waypoint-wait-count");
     props->untie("position/waypoint-waiting");
-    props->untie("position/waypoint-missed-time-sec");
     props->untie("submodels/serviceable");
 }
 
@@ -334,13 +335,9 @@ void FGAIShip::Run(double dt) {
 
         }
 
+        // do calculations for radar
+        UpdateRadar(manager);
     }
-
-    //###########################//
-    // do calculations for radar //
-    //###########################//
-    double range_ft2 = UpdateRadar(manager);
-
 }//end function
 
 void FGAIShip::AccelTo(double speed) {
