@@ -18,14 +18,14 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: aircraft.cxx,v 1.15 2006-02-21 01:16:05 mfranz Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-#include <stdio.h>
-#include <string.h>		// strdup
+#include <cstdio>
+#include <cstring>		// strdup
 
 #include <plib/ul.h>
 
@@ -36,6 +36,7 @@
 #include <simgear/structure/exception.hxx>
 
 #include <Main/globals.hxx>
+#include <Main/fg_init.hxx>
 #include <Main/fg_props.hxx>
 #include <Main/viewmgr.hxx>
 #include <Cockpit/panel.hxx>
