@@ -19,7 +19,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: apt_loader.cxx,v 1.8.2.3 2007-10-14 10:05:43 mfranz Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
@@ -36,19 +36,62 @@
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sgstream.hxx>
 #include <simgear/misc/strutils.hxx>
+#include <simgear/structure/exception.hxx>
 
-#include STL_STRING
+#include <string>
 
 #include "simple.hxx"
 #include "runways.hxx"
 
 #include "apt_loader.hxx"
 
+static FGPositioned::Type fptypeFromRobinType(int aType)
+{
+  switch (aType) {
+  case 1: return FGPositioned::AIRPORT;
+  case 16: return FGPositioned::SEAPORT;
+  case 17: return FGPositioned::HELIPORT;
+  default:
+    SG_LOG(SG_GENERAL, SG_ALERT, "unsupported type:" << aType);
+    throw sg_range_exception("Unsupported airport type", "fptypeFromRobinType");
+  }
+}
+
+FGAirport* addAirport(FGAirportList *airports, const string& apt_id, const string& apt_name,
+    int rwy_count, double rwy_lat_accum, double rwy_lon_accum, double last_rwy_heading,
+    double apt_elev, SGGeod& tower, bool got_tower, int type)
+{
+    if (apt_id.empty())
+        return NULL;
+
+    if (!rwy_count) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "ERROR: No runways for " << apt_id
+                << ", skipping." );
+        return NULL;
+    }
+
+    double lat = rwy_lat_accum / (double)rwy_count;
+    double lon = rwy_lon_accum / (double)rwy_count;
+
+    if (!got_tower) {
+        // tower height hard coded for now...
+        const float tower_height = 50.0f;
+        // make a little off the heading for 1 runway airports...
+        float fudge_lon = fabs(sin(last_rwy_heading * SGD_DEGREES_TO_RADIANS)) * .003f;
+        float fudge_lat = .003f - fudge_lon;
+        tower = SGGeod::fromDegFt(lon + fudge_lon, lat + fudge_lat, apt_elev + tower_height);
+    }
+
+  
+
+    return airports->add(apt_id, SGGeod::fromDegFt(lon, lat, apt_elev), tower, apt_name, false,
+        fptypeFromRobinType(type));
+}
 
 // Load the airport data base from the specified aptdb file.  The
 // metar file is used to mark the airports as having metar available
 // or not.
-bool fgAirportDBLoad( FGAirportList *airports, FGRunwayList *runways,
+bool fgAirportDBLoad( FGAirportList *airports, 
                       const string &aptdb_file, const string &metar_file )
 {
     //
@@ -67,15 +110,19 @@ bool fgAirportDBLoad( FGAirportList *airports, FGRunwayList *runways,
     string last_apt_name = "";
     string last_apt_info = "";
     int last_apt_type = 0;
+    SGGeod last_tower;
+    bool got_tower = false;
     string line;
     char tmp[2049];
     tmp[2048] = 0;
-
+    vector<FGRunwayPtr> runways;
+    
     unsigned int line_id = 0;
     unsigned int line_num = 0;
     double rwy_lon_accum = 0.0;
     double rwy_lat_accum = 0.0;
     int rwy_count = 0;
+    double last_rwy_heading = 0.0;
 
     while ( ! in.eof() ) {
 	in.getline(tmp, 2048);
@@ -119,25 +166,19 @@ bool fgAirportDBLoad( FGAirportList *airports, FGRunwayList *runways,
             SG_LOG( SG_GENERAL, SG_BULK, "Next airport = " << id << " "
                     << elev );
 
-            if ( !last_apt_id.empty()) {
-                if ( rwy_count > 0 ) {
-                    double lat = rwy_lat_accum / (double)rwy_count;
-                    double lon = rwy_lon_accum / (double)rwy_count;
-                    airports->add( last_apt_id, lon, lat, last_apt_elev,
-                                   last_apt_name, false, last_apt_type == 1,
-                                   last_apt_type == 16, last_apt_type == 17 );
-		} else {
-		    if ( !last_apt_id.length() ) {
-			SG_LOG(SG_GENERAL, SG_ALERT,
-                               "ERROR: No runways for " << last_apt_id
-                               << " skipping." );
-		    }
-		}
+            FGAirport* apt = addAirport(airports, last_apt_id, last_apt_name, rwy_count, rwy_lat_accum, rwy_lon_accum,
+                last_rwy_heading, last_apt_elev, last_tower, got_tower, last_apt_type);
+
+            for (unsigned int r=0; r< runways.size(); ++r) {
+              apt->addRunway(runways[r]);
             }
 
+            runways.clear();
+            
             last_apt_id = id;
-            last_apt_elev = atof( token[1].c_str() );
+            last_apt_elev = elev;
             last_apt_name = "";
+            got_tower = false;
 
             // build the name
             for ( unsigned int i = 5; i < token.size() - 1; ++i ) {
@@ -169,6 +210,8 @@ bool fgAirportDBLoad( FGAirportList *airports, FGRunwayList *runways,
             double heading = atof( token[4].c_str() );
             double length = atoi( token[5].c_str() );
             double width = atoi( token[8].c_str() );
+            
+            last_rwy_heading = heading;
 
             string rwy_displ_threshold = token[6];
             vector<string> displ
@@ -182,22 +225,29 @@ bool fgAirportDBLoad( FGAirportList *airports, FGRunwayList *runways,
             double stopway1 = atof( stop[0].c_str() );
             double stopway2 = atof( stop[1].c_str() );
 
-            string lighting_flags = token[9];
             int surface_code = atoi( token[10].c_str() );
-            string shoulder_code = token[11];
-            int marking_code = atoi( token[12].c_str() );
-            double smoothness = atof( token[13].c_str() );
-            bool dist_remaining = (atoi( token[14].c_str() ) == 1 );
 
-            runways->add( last_apt_id, rwy_no, lon, lat, heading, length,
-                          width, displ_thresh1, displ_thresh2,
-                          stopway1, stopway2, lighting_flags, surface_code,
-                          shoulder_code, marking_code, smoothness,
-                          dist_remaining );
+            FGRunway* rwy = new FGRunway(NULL, rwy_no, lon, lat, heading, length,
+                          width, displ_thresh1, stopway1, surface_code, false);
+            
+            FGRunway* reciprocal = new FGRunway(NULL, FGRunway::reverseIdent(rwy_no), 
+                          lon, lat, heading + 180.0, length, width, 
+                          displ_thresh2, stopway2, surface_code, true);
+            runways.push_back(rwy);
+            runways.push_back(reciprocal);
+            
         } else if ( line_id == 18 ) {
             // beacon entry (ignore)
         } else if ( line_id == 14 ) {
-            // control tower entry (ignore)
+            // control tower entry
+            token.clear();
+            token = simgear::strutils::split(line);
+
+            double lat = atof( token[1].c_str() );
+            double lon = atof( token[2].c_str() );
+            double elev = atof( token[3].c_str() );
+            last_tower = SGGeod::fromDegFt(lon, lat, elev + last_apt_elev);
+            got_tower = true;
         } else if ( line_id == 19 ) {
             // windsock entry (ignore)
         } else if ( line_id == 15 ) {
@@ -215,20 +265,10 @@ bool fgAirportDBLoad( FGAirportList *airports, FGRunwayList *runways,
         }
     }
 
-    if ( !last_apt_id.empty()) {
-        if ( rwy_count > 0 ) {
-            double lat = rwy_lat_accum / (double)rwy_count;
-            double lon = rwy_lon_accum / (double)rwy_count;
-            airports->add( last_apt_id, lon, lat, last_apt_elev,
-                           last_apt_name, false, false, false, false );
-        } else {
-            if ( !last_apt_id.length() ) {
-                SG_LOG(SG_GENERAL, SG_ALERT,
-                       "ERROR: No runways for " << last_apt_id
-                       << " skipping." );
-            }
-        }
-    }
+    // add the last airport being processed if any
+    addAirport(airports, last_apt_id, last_apt_name, rwy_count, rwy_lat_accum, rwy_lon_accum,
+        last_rwy_heading, last_apt_elev, last_tower, got_tower, last_apt_type);
+
 
     //
     // Load the metar.dat file and update apt db with stations that
