@@ -20,14 +20,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: parking.cxx,v 1.3.2.1 2007-11-27 19:28:27 durk Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
 //#include <algorithm>
-#include <stdlib.h>
 
 #include <simgear/compiler.h>
 
@@ -44,56 +43,29 @@
 //#include <Main/fg_props.hxx>
 //#include <Airports/runways.hxx>
 
-#include STL_STRING
+#include <string>
 
 #include "parking.hxx"
-
-
-/*****************************************************************************
- * Helper function for parsing position string
- ****************************************************************************/
-double processPosition(const string &pos)
-{
-  string prefix;
-  string subs;
-  string degree;
-  string decimal;
-  int sign = 1;
-  double value;
-  subs = pos;
-  prefix= subs.substr(0,1);
-  if (prefix == string("S") || (prefix == string("W")))
-    sign = -1;
-  subs    = subs.substr(1, subs.length());
-  degree  = subs.substr(0, subs.find(" ",0));
-  decimal = subs.substr(subs.find(" ",0), subs.length());
-  
-	      
-  //cerr << sign << " "<< degree << " " << decimal << endl;
-  value = sign * (atof(degree.c_str()) + atof(decimal.c_str())/60.0);
-  //cerr << value <<endl;
-  //exit(1);
-  return value;
-}
-
+#include "groundnetwork.hxx"
 
 /*********************************************************************************
  * FGParking
  ********************************************************************************/
-FGParking::FGParking(double lat,
-		     double lon,
-		     double hdg,
-		     double rad,
-		     int idx,
-		     const string &name,
-		     const string &tpe,
-		     const string &codes)
-{
-  latitude     = lat;
-  longitude    = lon;
-  heading      = hdg;
-  parkingName  = name;
-  index        = idx;
-  type         = tpe;
-  airlineCodes = codes;
+// FGParking::FGParking(double lat,
+// 		     double lon,
+// 		     double hdg,
+// 		     double rad,
+// 		     int idx,
+// 		     const string &name,
+// 		     const string &tpe,
+// 		     const string &codes)
+//   : FGTaxiNode(lat,lon,idx)
+// {
+//   heading      = hdg;
+//   parkingName  = name;
+//   type         = tpe;
+//   airlineCodes = codes;
+// }
+FGParking::~FGParking() {
+     delete pushBackRoute; 
 }
