@@ -19,15 +19,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: runwayprefs.cxx,v 1.7.2.3 2007-11-27 19:28:27 durk Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
 #include <math.h>
-#include <stdlib.h>
-#include <cstring>
 //#include <algorithm>
 
 #include <simgear/compiler.h>
@@ -46,6 +44,7 @@
 #include <Airports/runways.hxx>
 
 #include "runwayprefs.hxx"
+#include "simple.hxx"
 
 /******************************************************************************
  * ScheduleTime
@@ -119,7 +118,7 @@ string ScheduleTime::getName(time_t dayStart)
       //couldn't find one so return 0;
       //cerr << "Returning 0 " << endl;
     }
-    return string(0);
+    return string("");
 }			      
 /******************************************************************************
  * RunwayList
@@ -195,7 +194,7 @@ RunwayGroup& RunwayGroup:: operator= (const RunwayGroup &other)
   return *this;
 }
 
-void RunwayGroup::setActive(const string &aptId, 
+void RunwayGroup::setActive(const FGAirport* airport, 
 			    double windSpeed, 
 			    double windHeading, 
 			    double maxTail, 
@@ -203,7 +202,7 @@ void RunwayGroup::setActive(const string &aptId,
 			    stringVec *currentlyActive)
 {
 
-  FGRunway rwy;
+  FGRunway* rwy;
   int activeRwys = rwyList.size(); // get the number of runways active
   int nrOfPreferences;
   // bool found = true;
@@ -235,16 +234,11 @@ void RunwayGroup::setActive(const string &aptId,
 	  //
 	  validSelection = true;
 	  for (int j = 0; j < activeRwys; j++)
-	    {
-	     
-	      name = rwyList[j].getRwyList(i);
-	      //cerr << "Name of Runway: " << name << endl;
-	      if (globals->get_runways()->search( aptId, 
-						  name, 
-						  &rwy))
-		{
-		  //cerr << "Succes" << endl;
-		  hdgDiff = fabs(windHeading - rwy._heading);
+    {
+	     rwy = airport->getRunwayByIdent(rwyList[j].getRwyList(i));
+       
+	    		  //cerr << "Succes" << endl;
+		  hdgDiff = fabs(windHeading - rwy->headingDeg());
 		  //cerr << "Wind Heading: " << windHeading << "Runway Heading: " <<rwy._heading << endl;
 		  //cerr << "Wind Speed  : " << windSpeed << endl;
 		  if (hdgDiff > 180)
@@ -253,22 +247,20 @@ void RunwayGroup::setActive(const string &aptId,
 		  hdgDiff *= ((2*M_PI)/360.0); // convert to radians
 		  crossWind = windSpeed * sin(hdgDiff);
 		  tailWind  = -windSpeed * cos(hdgDiff);
-		  //cerr << "Tailwind : " << tailWind << endl;
-		  //cerr << "Crosswnd : " << crossWind << endl;
+		  //cerr << ". Tailwind : " << tailWind;
+		  //cerr << ". Crosswnd : " << crossWind;
 		  if ((tailWind > maxTail) || (crossWind > maxCross))
 		    {
-		      //cerr << "Invalid : " << endl;
+		      //cerr << ". [Invalid] " << endl;
 		      validSelection = false;
 		   }
 		  else 
 		    {
-		      //cerr << "Valid   : " << endl;;
+		      //cerr << ". [Valid] ";
 		  }
-		}else {
-		SG_LOG( SG_GENERAL, SG_INFO, "Failed to find runway " << name << " at " << aptId );
-		exit(1);
-	      }
-	    }
+		              //cerr << endl;
+    } // of active runways iteration
+      
 	  if (validSelection) 
 	    {
 	      //cerr << "Valid selection  : " << i << endl;;
@@ -311,15 +303,11 @@ void RunwayGroup::setActive(const string &aptId,
 	  bool validSelection = true;
 	  for (int j = 0; j < 2; j++)
 	    {
-	      //cerr << "I J " << i << " " << j << endl;
-	      name = rwyList[choice[j]].getRwyList(i);
-	      //cerr << "Name of Runway: " << name << endl;
-	      if (globals->get_runways()->search( aptId, 
-						  name, 
-						  &rwy))
-		{
+        name = rwyList[choice[j]].getRwyList(i);
+        rwy = airport->getRunwayByIdent(name);
+        
 		  //cerr << "Succes" << endl;
-		  hdgDiff = fabs(windHeading - rwy._heading);
+		  hdgDiff = fabs(windHeading - rwy->headingDeg());
 		  if (hdgDiff > 180)
 		    hdgDiff = 360 - hdgDiff;
 		  hdgDiff *= ((2*M_PI)/360.0); // convert to radians
@@ -327,10 +315,7 @@ void RunwayGroup::setActive(const string &aptId,
 		  tailWind  = -windSpeed * cos(hdgDiff);
 		  if ((tailWind > maxTail) || (crossWind > maxCross))
 		    validSelection = false;
-		}else {
-		  SG_LOG( SG_GENERAL, SG_INFO, "Failed to find runway " << name << " at " << aptId );
-		  exit(1);
-		}
+		
 
 	    }
 	  if (validSelection)
@@ -366,7 +351,8 @@ void RunwayGroup::getActive(int i, string &name, string &type)
 /*****************************************************************************
  * FGRunway preference
  ****************************************************************************/
-FGRunwayPreference::FGRunwayPreference()
+FGRunwayPreference::FGRunwayPreference(FGAirport* ap) : 
+  _ap(ap)
 {
   //cerr << "Running default Constructor" << endl;
   initialized = false;
@@ -375,16 +361,11 @@ FGRunwayPreference::FGRunwayPreference()
 FGRunwayPreference::FGRunwayPreference(const FGRunwayPreference &other)
 {
   initialized = other.initialized;
-  value = other.value;
-  scheduleName = other.scheduleName;
 
   comTimes = other.comTimes; // Commercial Traffic;
   genTimes = other.genTimes; // General Aviation;
   milTimes = other.milTimes; // Military Traffic;
-  currTimes= other.currTimes; // Needed for parsing;
 
-  rwyList = other.rwyList;
-  rwyGroup = other.rwyGroup;
   PreferenceListConstIterator i;
   for (i = other.preferences.begin(); i != other.preferences.end(); i++)
     preferences.push_back(*i);
@@ -393,16 +374,11 @@ FGRunwayPreference::FGRunwayPreference(const FGRunwayPreference &other)
 FGRunwayPreference & FGRunwayPreference::operator= (const FGRunwayPreference &other)
 {
   initialized = other.initialized;
-  value = other.value;
-  scheduleName = other.scheduleName;
   
   comTimes = other.comTimes; // Commercial Traffic;
   genTimes = other.genTimes; // General Aviation;
   milTimes = other.milTimes; // Military Traffic;
-  currTimes= other.currTimes; // Needed for parsing;
   
-  rwyList = other.rwyList;
-  rwyGroup = other.rwyGroup;
   PreferenceListConstIterator i;
   preferences.clear();
   for (i = other.preferences.begin(); i != other.preferences.end(); i++)
@@ -437,140 +413,6 @@ RunwayGroup *FGRunwayPreference::getGroup(const string &groupName)
     return 0;
 }
 
-void  FGRunwayPreference::startXML () {
-  //  cout << "Start XML" << endl;
-}
-
-void  FGRunwayPreference::endXML () {
-  //  cout << "End XML" << endl;
-}
-
-void  FGRunwayPreference::startElement (const char * name, const XMLAttributes &atts) {
-  //cout << "StartElement " << name << endl;
-  value = string("");
-  if (!(strcmp(name, "wind"))) {
-    //cerr << "Will be processing Wind" << endl;
-    for (int i = 0; i < atts.size(); i++)
-      {
-  	//cout << "  " << atts.getName(i) << '=' << atts.getValue(i) << endl; 
-  	//attname = atts.getName(i);
-  	if (atts.getName(i) == string("tail")) {
-  	  //cerr << "Tail Wind = " << atts.getValue(i) << endl;
-  	  currTimes.setTailWind(atof(atts.getValue(i)));
-  	}	
-  	if (atts.getName(i) == string("cross")) {
-  	  //cerr << "Cross Wind = " << atts.getValue(i) << endl;
-  	  currTimes.setCrossWind(atof(atts.getValue(i)));
-  	}
-     }
-  }
-    if (!(strcmp(name, "time"))) {
-      //cerr << "Will be processing time" << endl;	
-    for (int i = 0; i < atts.size(); i++)
-      {
-  	if (atts.getName(i) == string("start")) {
-  	  //cerr << "Start Time = " << atts.getValue(i) << endl;
-  	  currTimes.addStartTime(processTime(atts.getValue(i)));
-  	}
-  	if (atts.getName(i) == string("end")) {
-  	  //cerr << "End time = " << atts.getValue(i) << endl;
-  	  currTimes.addEndTime(processTime(atts.getValue(i)));
-  	}
-  	if (atts.getName(i) == string("schedule")) {
-  	  //cerr << "Schedule Name  = " << atts.getValue(i) << endl;
-  	  currTimes.addScheduleName(atts.getValue(i));
-  	}	
-    }
-  }
-  if (!(strcmp(name, "takeoff"))) {
-    rwyList.clear();
-  }
-  if  (!(strcmp(name, "landing")))
-    {
-      rwyList.clear();
-    }
-  if (!(strcmp(name, "schedule"))) {
-    for (int i = 0; i < atts.size(); i++)
-      {
-  	//cout << "  " << atts.getName(i) << '=' << atts.getValue(i) << endl; 
-  	//attname = atts.getName(i);
-  	if (atts.getName(i) == string("name")) {
-  	  //cerr << "Schedule name = " << atts.getValue(i) << endl;
-  	  scheduleName = atts.getValue(i);
-  	}
-      }
-  }
-}
-
-//based on a string containing hour and minute, return nr seconds since day start.
-time_t FGRunwayPreference::processTime(const string &tme)
-{
-  string hour   = tme.substr(0, tme.find(":",0));
-  string minute = tme.substr(tme.find(":",0)+1, tme.length());
-
-  //cerr << "hour = " << hour << " Minute = " << minute << endl;
-  return (atoi(hour.c_str()) * 3600 + atoi(minute.c_str()) * 60);
-}
-
-void  FGRunwayPreference::endElement (const char * name) {
-  //cout << "End element " << name << endl;
-  if (!(strcmp(name, "rwyuse"))) {
-    initialized = true;
-  }
-  if (!(strcmp(name, "com"))) { // Commercial Traffic
-    //cerr << "Setting time table for commerical traffic" << endl;
-    comTimes = currTimes;
-    currTimes.clear();
-  }
-  if (!(strcmp(name, "gen"))) { // General Aviation
-    //cerr << "Setting time table for general aviation" << endl;
-    genTimes = currTimes;
-    currTimes.clear();
-  }  
-  if (!(strcmp(name, "mil"))) { // Military Traffic
-    //cerr << "Setting time table for military traffic" << endl;
-    genTimes = currTimes;
-    currTimes.clear();
-  }
-
-  if (!(strcmp(name, "takeoff"))) {
-    //cerr << "Adding takeoff: " << value << endl;
-    rwyList.set(name, value);
-    rwyGroup.add(rwyList);
-  }
-  if (!(strcmp(name, "landing"))) {
-    //cerr << "Adding landing: " << value << endl;
-    rwyList.set(name, value);
-    rwyGroup.add(rwyList);
-  }
-  if (!(strcmp(name, "schedule"))) {
-    //cerr << "Adding schedule" << scheduleName << endl;
-    rwyGroup.setName(scheduleName);
-    //rwyGroup.addRunways(rwyList);
-    preferences.push_back(rwyGroup);
-    rwyGroup.clear();
-    //exit(1);
-  }
-}
-
-void  FGRunwayPreference::data (const char * s, int len) {
-  string token = string(s,len);
-  //cout << "Character data " << string(s,len) << endl;
-  //if ((token.find(" ") == string::npos && (token.find('\n')) == string::npos))
-  //  value += token;
-  //else
-  //  value = string("");
-  value += token;
-}
-
-void  FGRunwayPreference::pi (const char * target, const char * data) {
-  //cout << "Processing instruction " << target << ' ' << data << endl;
-}
-
-void  FGRunwayPreference::warning (const char * message, int line, int column) {
-  SG_LOG(SG_IO, SG_WARN, "Warning: " << message << " (" << line << ',' << column << ')');
-}
-
-void  FGRunwayPreference::error (const char * message, int line, int column) {
-  SG_LOG(SG_IO, SG_ALERT, "Error: " << message << " (" << line << ',' << column << ')');
-}
+ string FGRunwayPreference::getId() { 
+   return _ap->getId(); 
+ };
