@@ -22,7 +22,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: simple.cxx,v 1.34.2.5 2007-10-11 07:44:35 mfranz Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -33,26 +33,24 @@
 
 #include <simgear/compiler.h>
 
-#include <plib/sg.h>
-#include <plib/ul.h>
-
 #include <Environment/environment_mgr.hxx>
 #include <Environment/environment.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/structure/subsystem_mgr.hxx>
-//#include <simgear/route/waypoint.hxx>
 #include <simgear/debug/logstream.hxx>
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 #include <Airports/runways.hxx>
+#include <Airports/dynamics.hxx>
 
-#include STL_STRING
+#include <string>
 
 #include "simple.hxx"
+#include "xmlloader.hxx"
 
-SG_USING_STD(sort);
-SG_USING_STD(random_shuffle);
+using std::sort;
+using std::random_shuffle;
 
 
 
@@ -60,23 +58,13 @@ SG_USING_STD(random_shuffle);
 /***************************************************************************
  * FGAirport
  ***************************************************************************/
-FGAirport::FGAirport() : _longitude(0), _latitude(0), _elevation(0), _dynamics(0)
-{
-}
 
-
-FGAirport::FGAirport(const string &id, double lon, double lat, double elev,
-        const string &name, bool has_metar, bool is_airport, bool is_seaport,
-        bool is_heliport) :
-    _id(id),
-    _longitude(lon),
-    _latitude(lat),
-    _elevation(elev),
+FGAirport::FGAirport(const string &id, const SGGeod& location, const SGGeod& tower_location,
+        const string &name, bool has_metar, Type aType) :
+    FGPositioned(aType, id, location),
+    _tower_location(tower_location),
     _name(name),
     _has_metar(has_metar),
-    _is_airport(is_airport),
-    _is_seaport(is_seaport),
-    _is_heliport(is_heliport),
     _dynamics(0)
 {
 }
@@ -87,58 +75,163 @@ FGAirport::~FGAirport()
     delete _dynamics;
 }
 
+bool FGAirport::isAirport() const
+{
+  return type() == AIRPORT;
+}
+
+bool FGAirport::isSeaport() const
+{
+  return type() == SEAPORT;
+}
+
+bool FGAirport::isHeliport() const
+{
+  return type() == HELIPORT;
+}
 
 FGAirportDynamics * FGAirport::getDynamics()
 {
     if (_dynamics != 0) {
         return _dynamics;
     } else {
-        FGRunwayPreference rwyPrefs;
         //cerr << "Trying to load dynamics for " << _id << endl;
-        _dynamics = new FGAirportDynamics(_latitude, _longitude, _elevation, _id);
+        _dynamics = new FGAirportDynamics(this);
+        XMLLoader::load(_dynamics);
 
-        SGPath parkpath( globals->get_fg_root() );
-        parkpath.append( "/AI/Airports/" );
-        parkpath.append(_id);
-        parkpath.append("parking.xml");
-
-        SGPath rwyPrefPath( globals->get_fg_root() );
-        rwyPrefPath.append( "AI/Airports/" );
-        rwyPrefPath.append(_id);
-        rwyPrefPath.append("rwyuse.xml");
-
-        //if (ai_dirs.find(id.c_str()) != ai_dirs.end()
-        //  && parkpath.exists())
-        if (parkpath.exists()) {
-            try {
-                readXML(parkpath.str(),*_dynamics);
-		//cerr << "Initializing " << getId() << endl;
-                _dynamics->init();
-		_dynamics->getGroundNetwork()->setParent(this);
-            } catch (const sg_exception &e) {
-                //cerr << "unable to read " << parkpath.str() << endl;
-            }
-        }
-
-        //if (ai_dirs.find(id.c_str()) != ai_dirs.end()
-        //  && rwyPrefPath.exists())
-        if (rwyPrefPath.exists()) {
-            try {
-                readXML(rwyPrefPath.str(), rwyPrefs);
-                _dynamics->setRwyUse(rwyPrefs);
-            } catch (const sg_exception &e) {
-                //cerr << "unable to read " << rwyPrefPath.str() << endl;
-                //exit(1);
-            }
-        }
-        //exit(1);
-    }
+        FGRunwayPreference rwyPrefs(this);
+        XMLLoader::load(&rwyPrefs);
+        _dynamics->setRwyUse(rwyPrefs);
+   }
     return _dynamics;
 }
 
+unsigned int FGAirport::numRunways() const
+{
+  return mRunways.size();
+}
 
+FGRunway* FGAirport::getRunwayByIndex(unsigned int aIndex) const
+{
+  assert(aIndex >= 0 && aIndex < mRunways.size());
+  return mRunways[aIndex];
+}
 
+bool FGAirport::hasRunwayWithIdent(const string& aIdent) const
+{
+  return (getIteratorForRunwayIdent(aIdent) != mRunways.end());
+}
 
+FGRunway* FGAirport::getRunwayByIdent(const string& aIdent) const
+{
+  Runway_iterator it = getIteratorForRunwayIdent(aIdent);
+  if (it == mRunways.end()) {
+    SG_LOG(SG_GENERAL, SG_ALERT, "no such runway '" << aIdent << "' at airport " << ident());
+    throw sg_range_exception("unknown runway " + aIdent + " at airport:" + ident(), "FGAirport::getRunwayByIdent");
+  }
+  
+  return *it;
+}
+
+FGAirport::Runway_iterator
+FGAirport::getIteratorForRunwayIdent(const string& aIdent) const
+{
+  string ident(aIdent);
+  if ((aIdent.size() == 1) || !isdigit(aIdent[1])) {
+    ident = "0" + aIdent;
+  }
+
+  Runway_iterator it = mRunways.begin();
+  for (; it != mRunways.end(); ++it) {
+    if ((*it)->ident() == ident) {
+      return it;
+    }
+  }
+
+  return it; // end()
+}
+
+static double normaliseBearing(double aBearing)
+{
+  while (aBearing < -180) {
+    aBearing += 360.0;
+  }
+  
+  while (aBearing > 180.0) {
+    aBearing -= 360.0;
+  }
+  
+  return aBearing;
+}
+
+FGRunway* FGAirport::findBestRunwayForHeading(double aHeading) const
+{
+  Runway_iterator it = mRunways.begin();
+  FGRunway* result = NULL;
+  double currentBestQuality = 0.0;
+  
+  SGPropertyNode *param = fgGetNode("/sim/airport/runways/search", true);
+  double lengthWeight = param->getDoubleValue("length-weight", 0.01);
+  double widthWeight = param->getDoubleValue("width-weight", 0.01);
+  double surfaceWeight = param->getDoubleValue("surface-weight", 10);
+  double deviationWeight = param->getDoubleValue("deviation-weight", 1);
+    
+  for (; it != mRunways.end(); ++it) {
+    double good = (*it)->score(lengthWeight, widthWeight, surfaceWeight);
+    
+    double dev = normaliseBearing(aHeading - (*it)->headingDeg());
+    double bad = fabs(deviationWeight * dev) + 1e-20;
+    double quality = good / bad;
+    
+    if (quality > currentBestQuality) {
+      currentBestQuality = quality;
+      result = *it;
+    }
+  }
+
+  return result;
+}
+
+unsigned int FGAirport::numTaxiways() const
+{
+  return mTaxiways.size();
+}
+
+FGRunway* FGAirport::getTaxiwayByIndex(unsigned int aIndex) const
+{
+  assert(aIndex >= 0 && aIndex < mTaxiways.size());
+  return mTaxiways[aIndex];
+}
+
+void FGAirport::addRunway(FGRunway* aRunway)
+{
+  aRunway->setAirport(this);
+  
+  if (aRunway->isTaxiway()) {
+    mTaxiways.push_back(aRunway);
+  } else {
+    mRunways.push_back(aRunway);
+  }
+}
+
+FGRunway* FGAirport::getActiveRunwayForUsage() const
+{
+  static FGEnvironmentMgr* envMgr = NULL;
+  if (!envMgr) {
+    envMgr = (FGEnvironmentMgr *) globals->get_subsystem("environment");
+  }
+  
+  FGEnvironment stationWeather(envMgr->getEnvironment(mPosition));
+  
+  double windSpeed = stationWeather.get_wind_speed_kt();
+  double hdg = stationWeather.get_wind_from_heading_deg();
+  if (windSpeed <= 0.0) {
+    hdg = 270;	// This forces West-facing rwys to be used in no-wind situations
+    // which is consistent with Flightgear's initial setup.
+  }
+  
+  return findBestRunwayForHeading(hdg);
+}
 
 /******************************************************************************
  * FGAirportList
@@ -178,21 +271,18 @@ FGAirportList::~FGAirportList( void )
 
 
 // add an entry to the list
-void FGAirportList::add( const string &id, const double longitude,
-                         const double latitude, const double elevation,
-                         const string &name, bool has_metar, bool is_airport,
-                         bool is_seaport, bool is_heliport )
+FGAirport* FGAirportList::add( const string &id, const SGGeod& location, const SGGeod& tower_location,
+                         const string &name, bool has_metar, FGPositioned::Type aType)
 {
-    FGRunwayPreference rwyPrefs;
-    FGAirport* a = new FGAirport(id, longitude, latitude, elevation, name,
-            has_metar, is_airport, is_seaport, is_heliport);
-
+    FGAirport* a = new FGAirport(id, location, tower_location, name, has_metar, aType);
     airports_by_id[a->getId()] = a;
     // try and read in an auxilary file
 
     airports_array.push_back( a );
-    SG_LOG( SG_GENERAL, SG_BULK, "Adding " << id << " pos = " << longitude
-            << ", " << latitude << " elev = " << elevation );
+    SG_LOG( SG_GENERAL, SG_BULK, "Adding " << id << " pos = " << location.getLongitudeDeg()
+            << ", " << location.getLatitudeDeg() << " elev = " << location.getElevationFt() );
+            
+    return a;
 }
 
 
@@ -203,38 +293,68 @@ FGAirport* FGAirportList::search( const string& id)
     return (itr == airports_by_id.end() ? NULL : itr->second);
 }
 
-
-// search for first subsequent alphabetically to supplied id
-const FGAirport* FGAirportList::findFirstById( const string& id, bool exact )
+// wrap an FGIdentOrdering in an STL-compatible functor. not the most
+// efficent / pretty thing in the world, but avoids template nastiness in the 
+// headers, and we're only doing O(log(N)) comparisoms per search
+class orderingFunctor
 {
-    airport_map_iterator itr;
-    if (exact) {
-        itr = airports_by_id.find(id);
-    } else {
-        itr = airports_by_id.lower_bound(id);
-    }
-    if (itr == airports_by_id.end()) {
-        return (NULL);
-    } else {
-        return (itr->second);
-    }
+public:
+  orderingFunctor(FGIdentOrdering* aOrder) :
+    mOrdering(aOrder)
+  { assert(aOrder); }
+  
+  bool operator()(const airport_map::value_type& aA, const std::string& aB) const
+  {
+    return mOrdering->compare(aA.first,aB);
+  }
+
+  bool operator()(const std::string& aA, const airport_map::value_type& aB) const
+  {
+    return mOrdering->compare(aA, aB.first);
+  }
+
+  bool operator()(const airport_map::value_type& aA, const airport_map::value_type& aB) const
+  {
+    return mOrdering->compare(aA.first, aB.first);
+  }
+  
+private:
+  FGIdentOrdering* mOrdering;
+};
+
+const FGAirport* FGAirportList::findFirstById(const std::string& aIdent, FGIdentOrdering* aOrder)
+{
+  airport_map_iterator itr;
+  if (aOrder) {
+    orderingFunctor func(aOrder);
+    itr = std::lower_bound(airports_by_id.begin(),airports_by_id.end(), aIdent, func);
+  } else {
+    itr = airports_by_id.lower_bound(aIdent);
+  }
+  
+  if (itr == airports_by_id.end()) {
+    return NULL;
+  }
+  
+  return itr->second;
 }
 
-
 // search for the airport nearest the specified position
-FGAirport* FGAirportList::search(double lon_deg, double lat_deg)
+FGAirport* FGAirportList::search(double lon_deg, double lat_deg, double max_range)
 {
     static FGAirportSearchFilter accept_any;
-    return search(lon_deg, lat_deg, accept_any);
+    return search(lon_deg, lat_deg, max_range, accept_any);
 }
 
 
 // search for the airport nearest the specified position and
 // passing the filter
 FGAirport* FGAirportList::search(double lon_deg, double lat_deg,
+        double max_range,
         FGAirportSearchFilter& filter)
 {
-    double min_dist = 360.0;
+    double min_dist = max_range;
+
     airport_list_iterator it = airports_array.begin();
     airport_list_iterator end = airports_array.end();
     airport_list_iterator closest = end;
