@@ -18,21 +18,15 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: cockpit.cxx,v 1.22.2.1 2007-10-13 14:47:34 durk Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#ifdef HAVE_WINDOWS_H
-#  include <windows.h>
-#endif
-
 #include <simgear/compiler.h>
 
-#include SG_GLU_H
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -59,10 +53,6 @@
 #include "hud.hxx"
 
 
-// This is a structure that contains all data related to
-// cockpit/panel/hud system
-
-static pCockpit ac_cockpit;
 // The following routines obtain information concerntin the aircraft's
 // current state and return it to calling instrument display routines.
 // They should eventually be member functions of the aircraft.
@@ -99,11 +89,9 @@ get_formated_gmt_time( void )
 {
     static char buf[32];
     const struct tm *p = globals->get_time_params()->getGmt();
-    if (snprintf( buf, 32, "%d/%d/%4d %d:%02d:%02d",
+    sprintf( buf, "%d/%d/%4d %d:%02d:%02d",
          p->tm_mon+1, p->tm_mday, 1900 + p->tm_year,
-         p->tm_hour, p->tm_min, p->tm_sec) >= 32) {
-        SG_LOG(SG_GENERAL, SG_ALERT, "Caught overflow in " << SG_ORIGIN);
-    }
+         p->tm_hour, p->tm_min, p->tm_sec);
 
     return buf;
 }
@@ -543,20 +531,11 @@ bool fgCockpitInit( fgAIRCRAFT *cur_aircraft )
     // current aircraft.
 
     fgHUDInit( cur_aircraft );
-    ac_cockpit = new fg_Cockpit();
-
-    SG_LOG( SG_COCKPIT, SG_INFO,
-        "  Code " << ac_cockpit->code() << " Status "
-        << ac_cockpit->status() );
 
     return true;
 }
 
-void fgCockpitUpdate( void ) {
-
-    SG_LOG( SG_COCKPIT, SG_DEBUG,
-            "Cockpit: code " << ac_cockpit->code() << " status "
-            << ac_cockpit->status() );
+void fgCockpitUpdate( osg::State* state ) {
 
     static const SGPropertyNode * xsize_node = fgGetNode("/sim/startup/xsize");
     static const SGPropertyNode * ysize_node = fgGetNode("/sim/startup/ysize");
@@ -570,7 +549,7 @@ void fgCockpitUpdate( void ) {
     if ( hud_visibility_node->getBoolValue() ) {
         // This will check the global hud linked list pointer.
         // If there is anything to draw it will.
-        fgUpdateHUD();
+        fgUpdateHUD( state );
     }
 
     glViewport( 0, 0, iwidth, iheight );
@@ -581,7 +560,7 @@ void fgCockpitUpdate( void ) {
 
 
 struct FuncTable {
-    char *name;
+    const char *name;
     FLTFNPTR func;
 } fn_table[] = {
     { "agl", get_agl },
