@@ -1,4 +1,3 @@
-
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -459,7 +458,7 @@ void HudLadder::draw(void)
                 y = (((float)(i - pitch_value) * factor) + .5f);
 
                 if (!(i % div_units)) {           //  At integral multiple of div
-                    snprintf(TextLadder, 8, "%d", i);
+                    sprintf(TextLadder, "%d", i);
                     font->getBBox(TextLadder, pointsize, italic,
                             &left, &right, &bot, &top);
                     label_length = right - left;
@@ -506,7 +505,7 @@ void HudLadder::draw(void)
                     y = (((float)(i - actslope) * factor) + .5);
 
                 if (!(i % div_units)) {  //  At integral multiple of div
-                    snprintf(TextLadder, 8, "%d", i);
+                    sprintf(TextLadder, "%d", i);
                     font->getBBox(TextLadder, pointsize, italic,
                             &left, &right, &bot, &top);
                     label_length = right - left;
