@@ -32,8 +32,9 @@
 #include <Environment/environment_mgr.hxx>
 #include <simgear/math/sg_geodesy.hxx>
 #include <simgear/math/polar3d.hxx>
-#include SG_GLU_H
-#include <ATC/ATCutils.hxx>
+#include <ATCDCL/ATCutils.hxx>
+
+#include <osg/GLU>
 
 
 // int x, int y, int width, int height, float scale_data, bool working)
@@ -81,7 +82,7 @@ runway_instr::runway_instr(const SGPropertyNode *node) :
 
 void runway_instr::draw()
 {
-    if (!is_broken() && get_active_runway(runway)) {
+    if (!is_broken() && (runway = get_active_runway())) {
         glPushAttrib(GL_LINE_STIPPLE | GL_LINE_STIPPLE_PATTERN | GL_LINE_WIDTH);
         float modelView[4][4], projMat[4][4];
         bool anyLines;
@@ -105,13 +106,16 @@ void runway_instr::draw()
             globals->get_viewmgr()->copyToCurrent();
         }
         //Set the camera to the cockpit view to get the view of the runway from the cockpit
-        ssgSetCamera((sgVec4 *)cockpit_view->get_VIEW());
+        // OSGFIXME
+//         ssgSetCamera((sgVec4 *)cockpit_view->get_VIEW());
         get_rwy_points(points3d);
         //Get the current project matrix
-        ssgGetProjectionMatrix(projMat);
+        // OSGFIXME
+//         ssgGetProjectionMatrix(projMat);
 //        const sgVec4 *viewMat = globals->get_current_view()->get_VIEW();
         //Get the current model view matrix (cockpit view)
-        ssgGetModelviewMatrix(modelView);
+        // OSGFIXME
+//         ssgGetModelviewMatrix(modelView);
         //Create a rotation matrix to correct for any offsets (other than default offsets) to the model view matrix
         sgMat4 xy; //rotation about the Rxy, negate the sin's on Ry
         xy[0][0] = cYaw;         xy[1][0] = 0.0f;   xy[2][0] = -sYaw;        xy[3][0] = 0.0f;
@@ -164,53 +168,46 @@ void runway_instr::draw()
             curr_view->setGoalPitchOffset_deg(gpo);
         }
         //Set the camera back to the current view
-        ssgSetCamera((sgVec4 *)curr_view);
+        // OSGFIXME
+//         ssgSetCamera((sgVec4 *)curr_view);
         glPopAttrib();
     }//if not broken
 }
 
 
-bool runway_instr::get_active_runway(FGRunway& runway)
+FGRunway* runway_instr::get_active_runway()
 {
-    FGEnvironment stationweather =
-            ((FGEnvironmentMgr *)globals->get_subsystem("environment"))->getEnvironment();
-    double hdg = stationweather.get_wind_from_heading_deg();
-    return globals->get_runways()->search(fgGetString("/sim/presets/airport-id"), int(hdg), &runway);
+  const FGAirport* apt = fgFindAirportID(fgGetString("/sim/presets/airport-id"));
+  if (!apt) return NULL;
+  
+  return apt->getActiveRunwayForUsage();
 }
 
 
 void runway_instr::get_rwy_points(sgdVec3 *points3d)
 {
     double alt = current_aircraft.fdm_state->get_Runway_altitude() * SG_FEET_TO_METER;
-    double length = (runway._length / 2.0) * SG_FEET_TO_METER;
-    double width = (runway._width / 2.0) * SG_FEET_TO_METER;
+    double length = runway->lengthM() * 0.5;
+    double width = runway->widthM() * 0.5;
     double frontLat, frontLon, backLat, backLon,az, tempLat, tempLon;
 
-    geo_direct_wgs_84(alt, runway._lat, runway._lon, runway._heading, length, &backLat, &backLon, &az);
+    geo_direct_wgs_84(alt, runway->_lat, runway->_lon, runway->_heading, length, &backLat, &backLon, &az);
     sgGeodToCart(backLat * SG_DEGREES_TO_RADIANS, backLon * SG_DEGREES_TO_RADIANS, alt, points3d[4]);
 
-    geo_direct_wgs_84(alt, runway._lat, runway._lon, runway._heading + 180, length, &frontLat, &frontLon, &az);
+    geo_direct_wgs_84(alt, runway->_lat, runway->_lon, runway->_heading + 180, length, &frontLat, &frontLon, &az);
     sgGeodToCart(frontLat * SG_DEGREES_TO_RADIANS, frontLon * SG_DEGREES_TO_RADIANS, alt, points3d[5]);
 
-    geo_direct_wgs_84(alt, backLat, backLon, runway._heading + 90, width, &tempLat, &tempLon, &az);
+    geo_direct_wgs_84(alt, backLat, backLon, runway->_heading + 90, width, &tempLat, &tempLon, &az);
     sgGeodToCart(tempLat * SG_DEGREES_TO_RADIANS, tempLon * SG_DEGREES_TO_RADIANS, alt, points3d[0]);
 
-    geo_direct_wgs_84(alt, backLat, backLon, runway._heading - 90, width, &tempLat, &tempLon, &az);
+    geo_direct_wgs_84(alt, backLat, backLon, runway->_heading - 90, width, &tempLat, &tempLon, &az);
     sgGeodToCart(tempLat * SG_DEGREES_TO_RADIANS, tempLon * SG_DEGREES_TO_RADIANS, alt, points3d[1]);
 
-    geo_direct_wgs_84(alt, frontLat, frontLon, runway._heading - 90, width, &tempLat, &tempLon, &az);
+    geo_direct_wgs_84(alt, frontLat, frontLon, runway->_heading - 90, width, &tempLat, &tempLon, &az);
     sgGeodToCart(tempLat * SG_DEGREES_TO_RADIANS, tempLon * SG_DEGREES_TO_RADIANS, alt, points3d[2]);
 
-    geo_direct_wgs_84(alt, frontLat, frontLon, runway._heading + 90, width, &tempLat, &tempLon, &az);
+    geo_direct_wgs_84(alt, frontLat, frontLon, runway->_heading + 90, width, &tempLat, &tempLon, &az);
     sgGeodToCart(tempLat * SG_DEGREES_TO_RADIANS, tempLon * SG_DEGREES_TO_RADIANS, alt, points3d[3]);
-
-    //Get the current tile center
-    SGVec3d tileCenter = globals->get_scenery()->get_center();
-    for (int i = 0; i < 6; i++) {
-        points3d[i][0] -= tileCenter.x();
-        points3d[i][1] -= tileCenter.y();
-        points3d[i][2] -= tileCenter.z();
-    }
 }
 
 
@@ -389,8 +386,8 @@ void runway_instr::drawArrow()
     Point3D ac(0.0), rwy(0.0);
     ac.setlat(current_aircraft.fdm_state->get_Latitude_deg());
     ac.setlon(current_aircraft.fdm_state->get_Longitude_deg());
-    rwy.setlat(runway._lat);
-    rwy.setlon(runway._lon);
+    rwy.setlat(runway->_lat);
+    rwy.setlon(runway->_lon);
     float theta = GetHeadingFromTo(ac, rwy);
     theta -= fgGetDouble("/orientation/heading-deg");
     theta = -theta;
@@ -420,8 +417,8 @@ void runway_instr::setLineWidth()
 {
     //Calculate the distance from the runway, A
     double course, distance;
-    calc_gc_course_dist(Point3D(runway._lon * SGD_DEGREES_TO_RADIANS,
-            runway._lat * SGD_DEGREES_TO_RADIANS, 0.0),
+    calc_gc_course_dist(Point3D(runway->longitude() * SGD_DEGREES_TO_RADIANS,
+            runway->latitude() * SGD_DEGREES_TO_RADIANS, 0.0),
             Point3D(current_aircraft.fdm_state->get_Longitude(),
             current_aircraft.fdm_state->get_Latitude(), 0.0 ),
             &course, &distance);
@@ -435,7 +432,7 @@ void runway_instr::setLineWidth()
     else
         alt_nm *= SG_METER_TO_NM;
 
-    //Calculate distance away from runway, C = v(A+B)
+    //Calculate distance away from runway, C = v(A≤+B≤)
     distance = sqrt(alt_nm * alt_nm + distance*distance);
     if (distance < scaleDist)
         glLineWidth(1.0 + ((lnScale - 1) * ((scaleDist - distance) / scaleDist)));
