@@ -16,16 +16,12 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-//  $Id: panel_io.cxx,v 1.26 2006-08-10 11:12:39 mfranz Exp $
+//  $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#ifdef HAVE_WINDOWS_H
-#  include <windows.h>
-#endif
-
 #include <string.h>		// for strcmp()
 
 #include <simgear/compiler.h>
@@ -34,9 +30,9 @@
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
 
-#include STL_IOSTREAM
-#include STL_FSTREAM
-#include STL_STRING
+#include <istream>
+#include <fstream>
+#include <string>
 
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
@@ -50,9 +46,9 @@
 //built-in layers
 #include "built_in/FGMagRibbon.hxx"
 
-SG_USING_STD(istream);
-SG_USING_STD(ifstream);
-SG_USING_STD(string);
+using std::istream;
+using std::ifstream;
+using std::string;
 
 
 
@@ -178,7 +174,7 @@ readAction (const SGPropertyNode * node, float w_scale, float h_scale)
   // button-less actions are fired initially
   if (!node->hasValue("w") || !node->hasValue("h")) {
     for (i = 0; i < bindings.size(); i++) {
-      FGBinding b(bindings[i]);
+      SGBinding b(bindings[i], globals->get_props());
       b.fire();
     }
     return 0;
@@ -207,7 +203,7 @@ readAction (const SGPropertyNode * node, float w_scale, float h_scale)
 
     binding = dest->getChild("binding", j, true);
     copyProperties(bindings[i], binding);
-    action->addBinding(new FGBinding(binding), 0);
+    action->addBinding(new SGBinding(binding, globals->get_props()), 0);
   }
 
   if (node->hasChild("mod-up")) {
@@ -219,7 +215,7 @@ readAction (const SGPropertyNode * node, float w_scale, float h_scale)
 
       binding = dest->getChild("binding", j, true);
       copyProperties(bindings[i], binding);
-      action->addBinding(new FGBinding(binding), 1);
+      action->addBinding(new SGBinding(binding, globals->get_props()), 1);
     }
   }
 
@@ -288,20 +284,7 @@ readTransformation (const SGPropertyNode * node, float w_scale, float h_scale)
   if (trans_table != 0) {
     SG_LOG( SG_COCKPIT, SG_INFO, "Found interpolation table with "
             << trans_table->nChildren() << " children" );
-    t->table = new SGInterpTable();
-    for (int i = 0; i < trans_table->nChildren(); i++) {
-      const SGPropertyNode * node = trans_table->getChild(i);
-      if (!strcmp(node->getName(), "entry")) {
-	double ind = node->getDoubleValue("ind", 0.0);
-	double dep = node->getDoubleValue("dep", 0.0);
-	SG_LOG( SG_COCKPIT, SG_INFO, "Adding interpolation entry "
-                << ind << "==>" << dep );
-	t->table->addEntry(ind, dep);
-      } else {
-	SG_LOG( SG_COCKPIT, SG_INFO, "Skipping " << node->getName()
-                << " in interpolation" );
-      }
-    }
+    t->table = new SGInterpTable(trans_table);
   } else {
     t->table = 0;
   }
@@ -693,6 +676,8 @@ readPanel (const SGPropertyNode * root)
   if (!fgHasNode("/sim/panel/y-offset"))
     fgSetInt("/sim/panel/y-offset", root->getIntValue("y-offset", 0));
 
+  panel->setAutohide(root->getBoolValue("autohide", true));
+
   //
   // Assign the background texture, if any, or a bogus chequerboard.
   //
