@@ -18,17 +18,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: environment.cxx,v 1.19 2006-07-29 09:22:24 fredb Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#ifdef HAVE_WINDOWS_H
-#  include <windows.h>                     
-#endif
-
 #include <math.h>
 
 #include <plib/sg.h>
@@ -49,43 +45,50 @@
 // Atmosphere model.
 ////////////////////////////////////////////////////////////////////////
 
-// Copied from YASim Atmosphere.cxx, with m converted to ft, degK
-// converted to degC, Pa converted to inHG, and kg/m^3 converted to
-// slug/ft^3; they were then converted to deltas from the sea-level
-// defaults (approx. 15degC, 29.92inHG, and 0.00237slugs/ft^3).
+// Calculated based on the ISA standard day, as found at e.g.
+// http://www.av8n.com/physics/altimetry.htm
 
-// Original comment from YASim:
-
-// Copied from McCormick, who got it from "The ARDC Model Atmosphere"
-// Note that there's an error in the text in the first entry,
-// McCormick lists 299.16/101325/1.22500, but those don't agree with
-// R=287.  I chose to correct the temperature to 288.20, since 79F is
-// pretty hot for a "standard" atmosphere.
-
-// Elevation (ft), temperature factor (degK), pressure factor (inHG)
+// Each line of data has 3 elements:
+//   Elevation (ft), 
+//   temperature factor (dimensionless ratio of absolute temp), 
+//   pressure factor (dimensionless ratio)
 static double atmosphere_data[][3] = {
- { 0.00, 1.00, 1.000 },
- { 2952.76, 0.98, 0.898 },
- { 5905.51, 0.96, 0.804 },
- { 8858.27, 0.94, 0.719 },
- { 11811.02, 0.92, 0.641 },
- { 14763.78, 0.90, 0.570 },
- { 17716.54, 0.88, 0.506 },
- { 20669.29, 0.86, 0.447 },
- { 23622.05, 0.84, 0.394 },
- { 26574.80, 0.82, 0.347 },
- { 29527.56, 0.80, 0.304 },
- { 32480.31, 0.78, 0.266 },
- { 35433.07, 0.76, 0.231 },
- { 38385.83, 0.75, 0.201 },
- { 41338.58, 0.75, 0.174 },
- { 44291.34, 0.75, 0.151 },
- { 47244.09, 0.75, 0.131 },
- { 50196.85, 0.75, 0.114 },
- { 53149.61, 0.75, 0.099 },
- { 56102.36, 0.75, 0.086 },
- { 59055.12, 0.75, 0.075 },
- { 62007.87, 0.75, 0.065 },
+ {  -3000.00,   1.021,  1.1133 },
+ {      0.00,   1.000,  1.0000 },
+ {   2952.76,   0.980,  0.8978 },
+ {   5905.51,   0.959,  0.8042 },
+ {   8858.27,   0.939,  0.7187 },
+ {  11811.02,   0.919,  0.6407 },
+ {  14763.78,   0.898,  0.5697 },
+ {  17716.54,   0.878,  0.5052 },
+ {  20669.29,   0.858,  0.4468 },
+ {  23622.05,   0.838,  0.3940 },
+ {  26574.80,   0.817,  0.3463 },
+ {  29527.56,   0.797,  0.3034 },
+ {  32480.31,   0.777,  0.2649 },
+ {  35433.07,   0.756,  0.2305 },
+ {  38385.83,   0.752,  0.2000 },
+ {  41338.58,   0.752,  0.1736 },
+ {  44291.34,   0.752,  0.1506 },
+ {  47244.09,   0.752,  0.1307 },
+ {  50196.85,   0.752,  0.1134 },
+ {  53149.61,   0.752,  0.0984 },
+ {  56102.36,   0.752,  0.0854 },
+ {  59055.12,   0.752,  0.0741 },
+ {  62007.87,   0.752,  0.0643 },
+ {  65000.00,   0.752,  0.0557 },
+ {  68000.00,   0.754,  0.0482 },
+ {  71000.00,   0.758,  0.0418 },
+ {  74000.00,   0.761,  0.0362 },
+ {  77000.00,   0.764,  0.0314 },
+ {  80000.00,   0.767,  0.0273 },
+ {  83000.00,   0.770,  0.0237 },
+ {  86000.00,   0.773,  0.0206 },
+ {  89000.00,   0.777,  0.0179 },
+ {  92000.00,   0.780,  0.0156 },
+ {  95000.00,   0.783,  0.0135 },
+ {  98000.00,   0.786,  0.0118 },
+ { 101000.00,   0.789,  0.0103 },
  { -1, -1, -1 }
 };
 
@@ -115,33 +118,39 @@ _setup_tables ()
 // Implementation of FGEnvironment.
 ////////////////////////////////////////////////////////////////////////
 
-FGEnvironment::FGEnvironment()
-  : elevation_ft(0),
-    visibility_m(32000),
-    temperature_sea_level_degc(15),
-    temperature_degc(15),
-    dewpoint_sea_level_degc(5), // guess
-    dewpoint_degc(5),
-    pressure_sea_level_inhg(29.92),
-    pressure_inhg(29.92),
-    turbulence_magnitude_norm(0),
-    turbulence_rate_hz(1),
-    wind_from_heading_deg(0),
-    wind_speed_kt(0),
-    wind_from_north_fps(0),
-    wind_from_east_fps(0),
-    wind_from_down_fps(0),
-    altitude_half_to_sun_m(1000),
-    altitude_tropo_top_m(10000)
+void FGEnvironment::_init()
 {
-  _setup_tables();
-  _recalc_density();
-  _recalc_relative_humidity();
+    elevation_ft = 0;
+    visibility_m = 32000;
+    temperature_sea_level_degc = 15;
+    temperature_degc = 15;
+    dewpoint_sea_level_degc = 5; // guess
+    dewpoint_degc = 5;
+    pressure_sea_level_inhg = 29.92;
+    pressure_inhg = 29.92;
+    turbulence_magnitude_norm = 0;
+    turbulence_rate_hz = 1;
+    wind_from_heading_deg = 0;
+    wind_speed_kt = 0;
+    wind_from_north_fps = 0;
+    wind_from_east_fps = 0;
+    wind_from_down_fps = 0;
+    altitude_half_to_sun_m = 1000;
+    altitude_tropo_top_m = 10000;
+    _setup_tables();
+    _recalc_density();
+    _recalc_relative_humidity();
+    
+}
+
+FGEnvironment::FGEnvironment()
+{
+    _init();
 }
 
 FGEnvironment::FGEnvironment (const FGEnvironment &env)
 {
-    FGEnvironment();
+    _init();
     copy(env);
 }
 
