@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: environment_ctrl.cxx,v 1.46.2.6 2007-10-13 14:47:34 durk Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -39,7 +39,7 @@
 #include "environment_mgr.hxx"
 #include "environment_ctrl.hxx"
 
-SG_USING_STD(sort);
+using std::sort;
 
 class metar_filter : public FGAirportSearchFilter {
     virtual bool pass(FGAirport *a) { return a->getMetar(); }
@@ -348,6 +348,7 @@ FGMetarEnvironmentCtrl::FGMetarEnvironmentCtrl ()
       MaxCloudAltitudeChangeFtSec( 20.0 ),
       MaxCloudThicknessChangeFtSec( 50.0 ),
       MaxCloudInterpolationHeightFt( 5000.0 ),
+      MaxCloudInterpolationDeltaFt( 4000.0 ),
 
       _error_count( 0 ),
       _stale_count( 0 ),
@@ -356,7 +357,8 @@ FGMetarEnvironmentCtrl::FGMetarEnvironmentCtrl ()
 {
 #if defined(ENABLE_THREADS)
     thread = new MetarThread(this);
-    thread->start( 1 );
+    thread->setProcessorAffinity(1);
+    thread->start();
 #endif // ENABLE_THREADS
 }
 
@@ -403,6 +405,9 @@ FGMetarEnvironmentCtrl::update_env_config ()
     double pressure;
     double temp;
     double dewpoint;
+    
+    // If we aren't in the METAR scenario, don't attempt to interpolate.
+    if (strcmp(fgGetString("/environment/weather-scenario", "METAR"), "METAR")) return;
 
     if (metar_loaded) {
         // Generate interpolated values between the METAR and the current
@@ -410,7 +415,6 @@ FGMetarEnvironmentCtrl::update_env_config ()
 
         // Pick up the METAR wind values and convert them into a vector.
         double metar[2];
-
         double metar_speed = fgGetDouble("/environment/metar/base-wind-speed-kt");
         double metar_heading = fgGetDouble("/environment/metar/base-wind-range-from");
 
@@ -424,7 +428,6 @@ FGMetarEnvironmentCtrl::update_env_config ()
         double current_heading = fgGetDouble(
                 "/environment/config/boundary/entry/wind-from-heading-deg");
 
-
         current[0] = current_speed * sin((current_heading / 180.0) * M_PI);
         current[1] = current_speed * cos((current_heading / 180.0) * M_PI);
 
@@ -433,15 +436,19 @@ FGMetarEnvironmentCtrl::update_env_config ()
         // factor by the maximum wind change.
         double x = fabs(current[0] - metar[0]);
         double y = fabs(current[1] - metar[1]);
-        double dx = x / (x + y);
-        double dy = 1 - dx;
 
-        double maxdx = dx * MaxWindChangeKtsSec;
-        double maxdy = dy * MaxWindChangeKtsSec;
+        // only interpolate if we have a difference
+        if (x + y > 0) {
+            double dx = x / (x + y);
+            double dy = 1 - dx;
 
-        // Interpolate each component separately.
-        current[0] = interpolate_val(current[0], metar[0], maxdx);
-        current[1] = interpolate_val(current[1], metar[1], maxdy);
+            double maxdx = dx * MaxWindChangeKtsSec;
+            double maxdy = dy * MaxWindChangeKtsSec;
+
+            // Interpolate each component separately.
+            current[0] = interpolate_val(current[0], metar[0], maxdx);
+            current[1] = interpolate_val(current[1], metar[1], maxdy);
+        }
 
         // Now convert back to polar coordinates.
         if ((current[0] == 0.0) && (current[1] == 0.0)) {
@@ -522,12 +529,15 @@ FGMetarEnvironmentCtrl::update_env_config ()
             double current_alt = fgGetDouble(s);
             double required_alt = (*layer)->getDoubleValue("elevation-ft");
 
-            if (current_alt < -9000 || required_alt < -9000
-                    || fabs(aircraft_alt - required_alt) > MaxCloudInterpolationHeightFt) {
-                // We don't interpolate any values that are too high above us,
-                // or too far below us to be visible. Nor do we interpolate
-                // values to or from -9999, which is used as a placeholder
-                // when there isn't actually a cloud layer present.
+            if (current_alt < -9000 || required_alt < -9000 ||
+                fabs(aircraft_alt - required_alt) > MaxCloudInterpolationHeightFt ||
+                fabs(current_alt - required_alt) > MaxCloudInterpolationDeltaFt) {
+                // We don't interpolate any layers that are
+                //  - too far above us to be visible
+                //  - too far below us to be visible
+                //  - with too large a difference to make interpolation sensible
+                //  - to or from -9999 (used as a placeholder)
+                //  - any values that are too high above us,
                 snprintf(s, 128, cl, i);
                 strncat(s, "/elevation-ft", 128);
                 if (current_alt != required_alt)
@@ -560,6 +570,7 @@ FGMetarEnvironmentCtrl::update_env_config ()
                 }
             }
         }
+
     } else {
         // We haven't already loaded a METAR, so apply it immediately.
         dir_from = fgGetDouble("/environment/metar/base-wind-range-from");
@@ -572,7 +583,7 @@ FGMetarEnvironmentCtrl::update_env_config ()
         dewpoint = fgGetDouble("/environment/metar/dewpoint-degc");
 
         // Set the cloud layers by copying over the METAR versions.
-        SGPropertyNode * clouds = fgGetNode("/environment/metar/clouds");
+        SGPropertyNode * clouds = fgGetNode("/environment/metar/clouds", true);
 
         vector<SGPropertyNode_ptr> layers = clouds->getChildren("layer");
         vector<SGPropertyNode_ptr>::const_iterator layer;
@@ -599,21 +610,19 @@ FGMetarEnvironmentCtrl::update_env_config ()
             strncat(s, "/span-m", 128);
             fgSetDouble(s, 40000.0);
         }
+
+        // Force an update of the 3D clouds
+        fgSetDouble("/environment/rebuild-layers", 1.0);
     }
 
     fgSetupWind(dir_from, dir_to, speed, gust);
     fgDefaultWeatherValue("visibility-m", vis);
-stamp("update_env_config_14b");
     set_temp_at_altitude(temp, station_elevation_ft);
-stamp("update_env_config_14c");
     set_dewpoint_at_altitude(dewpoint, station_elevation_ft);
-stamp("update_env_config_14d");
     fgDefaultWeatherValue("pressure-sea-level-inhg", pressure);
-stamp("update_env_config_14e");
 
     // We've now successfully loaded a METAR into the configuration
     metar_loaded = true;
-stamp("update_env_config_15");
 }
 
 double FGMetarEnvironmentCtrl::interpolate_prop(const char * currentname,
@@ -657,6 +666,7 @@ FGMetarEnvironmentCtrl::init ()
         const FGAirport* a = globals->get_airports()
                    ->search( longitude->getDoubleValue(),
                              latitude->getDoubleValue(),
+                             360.0,
                              metar_only );
         if ( a ) {  
             FGMetarResult result = fetch_data( a->getId() );
@@ -688,10 +698,7 @@ FGMetarEnvironmentCtrl::reinit ()
 {
     _error_count = 0;
     _error_dt = 0.0;
-
-#if 0
-    update_env_config();
-#endif
+    metar_loaded = false;
 
     env->reinit();
 }
@@ -699,31 +706,28 @@ FGMetarEnvironmentCtrl::reinit ()
 void
 FGMetarEnvironmentCtrl::update(double delta_time_sec)
 {
-    timingInfo.clear();
-    stamp("begin"); 
+
     _dt += delta_time_sec;
     if (_error_count >= 3)
        return;
-    //stamp("env_start"); 
+
     FGMetarResult result;
-    //stamp("envir_01"); 
 
     static const SGPropertyNode *longitude
         = fgGetNode( "/position/longitude-deg", true );
     static const SGPropertyNode *latitude
         = fgGetNode( "/position/latitude-deg", true );
-    //stamp("envir_02");
     search_elapsed += delta_time_sec;
     fetch_elapsed += delta_time_sec;
     interpolate_elapsed += delta_time_sec;
 
     // if time for a new search request, push it onto the request
     // queue
-    //stamp("point_03");
     if ( search_elapsed > search_interval_sec ) {
         const FGAirport* a = globals->get_airports()
                    ->search( longitude->getDoubleValue(),
                              latitude->getDoubleValue(),
+                             360.0,
                              metar_only );
         if ( a ) {
             if ( !last_apt || last_apt->getId() != a->getId()
@@ -747,17 +751,13 @@ FGMetarEnvironmentCtrl::update(double delta_time_sec)
         }
     } else if ( interpolate_elapsed > EnvironmentUpdatePeriodSec ) {
         // Interpolate the current configuration closer to the actual METAR
-        //stamp("point_03a");
         update_env_config();
-        stamp("envir_03b");
         env->reinit();
-        stamp("envir_03c");
         interpolate_elapsed = 0.0;
     }
 
 #if !defined(ENABLE_THREADS)
     // No loader thread running so manually fetch the data
-    stamp("point_04");
     string id = "";
     while ( !request_queue.empty() ) {
         id = request_queue.front();
@@ -770,7 +770,7 @@ FGMetarEnvironmentCtrl::update(double delta_time_sec)
         result_queue.push( result );
     }
 #endif // ENABLE_THREADS
-    stamp("point_05");
+
     // process any results from the loader.
     while ( !result_queue.empty() ) {
         result = result_queue.front();
@@ -789,9 +789,8 @@ FGMetarEnvironmentCtrl::update(double delta_time_sec)
             search_elapsed = 9999.0;
         }
     }
-    stamp("point_06");
+
     env->update(delta_time_sec);
-    stamp("point_07");
 }
 
 
