@@ -18,24 +18,28 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: environment_mgr.cxx,v 1.24.2.1 2006-11-25 12:21:28 mfranz Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
+#include <simgear/constants.h>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/scene/sky/sky.hxx>
 #include <simgear/environment/visual_enviro.hxx>
+#include <simgear/scene/model/particles.hxx>
 
 #include <Main/main.hxx>
 #include <Main/fg_props.hxx>
 #include <Aircraft/aircraft.hxx>
+#include <FDM/flight.hxx>
 
 #include "environment.hxx"
 #include "environment_ctrl.hxx"
 #include "environment_mgr.hxx"
 #include "fgclouds.hxx"
+#include "precipitation_mgr.hxx"
 
 class SGSky;
 extern SGSky *thesky;
@@ -54,14 +58,18 @@ FGEnvironmentMgr::FGEnvironmentMgr ()
   _controller->setEnvironment(_environment);
   set_subsystem("controller", _controller, 0.5);
   fgClouds = new FGClouds( _controller );
+  _precipitationManager = new FGPrecipitationMgr;
+  set_subsystem("precipitation", _precipitationManager);
 }
 
 FGEnvironmentMgr::~FGEnvironmentMgr ()
 {
   delete _environment;
   remove_subsystem("controller");
+  remove_subsystem("precipitation");
   delete _controller;
   delete fgClouds;
+  delete _precipitationManager;
 }
 
 void
@@ -174,27 +182,25 @@ FGEnvironmentMgr::bind ()
 	  &FGEnvironmentMgr::set_cloud_layer_coverage);
     fgSetArchivable(buf);
   }
-  fgTie("/sim/rendering/clouds3d-enable", &sgEnviro,
-	  &SGEnviro::get_clouds_enable_state,
-	  &SGEnviro::set_clouds_enable_state);
-  fgTie("/sim/rendering/clouds3d-vis-range", &sgEnviro,
-	  &SGEnviro::get_clouds_visibility,
-	  &SGEnviro::set_clouds_visibility);
-  fgTie("/sim/rendering/clouds3d-density", &sgEnviro,
-	  &SGEnviro::get_clouds_density,
-	  &SGEnviro::set_clouds_density);
-  fgTie("/sim/rendering/clouds3d-cache-size", &sgEnviro,
-	  &SGEnviro::get_clouds_CacheSize,
-	  &SGEnviro::set_clouds_CacheSize);
-  fgTie("/sim/rendering/clouds3d-cache-resolution", &sgEnviro,
-	  &SGEnviro::get_CacheResolution,
-	  &SGEnviro::set_CacheResolution);
+  fgTie("/sim/rendering/clouds3d-enable", fgClouds,
+	  &FGClouds::get_3dClouds,
+	  &FGClouds::set_3dClouds);
+  fgTie("/sim/rendering/clouds3d-density", thesky,
+	  &SGSky::get_3dCloudDensity,
+	  &SGSky::set_3dCloudDensity);
+  fgTie("/sim/rendering/clouds3d-vis-range", thesky,
+        &SGSky::get_3dCloudVisRange,
+        &SGSky::set_3dCloudVisRange);
+  
   fgTie("/sim/rendering/precipitation-enable", &sgEnviro,
-	  &SGEnviro::get_precipitation_enable_state,
+	  &SGEnviro::get_precipitation_enable_state, 
 	  &SGEnviro::set_precipitation_enable_state);
   fgTie("/environment/rebuild-layers", fgClouds,
       &FGClouds::get_update_event,
       &FGClouds::set_update_event);
+  fgTie("/environment/weather-scenario", fgClouds,
+        &FGClouds::get_scenario,
+        &FGClouds::set_scenario);
   fgTie("/sim/rendering/lightning-enable", &sgEnviro,
       &SGEnviro::get_lightning_enable_state,
       &SGEnviro::set_lightning_enable_state);
@@ -239,10 +245,9 @@ FGEnvironmentMgr::unbind ()
   fgUntie("/sim/rendering/clouds3d-enable");
   fgUntie("/sim/rendering/clouds3d-vis-range");
   fgUntie("/sim/rendering/clouds3d-density");
-  fgUntie("/sim/rendering/clouds3d-cache-size");
-  fgUntie("/sim/rendering/clouds3d-cache-resolution");
   fgUntie("/sim/rendering/precipitation-enable");
   fgUntie("/environment/rebuild-layers");
+  fgUntie("/environment/weather-scenario");
   fgUntie("/sim/rendering/lightning-enable");
   fgUntie("/environment/turbulence/use-cloud-turbulence");
 }
@@ -258,6 +263,10 @@ FGEnvironmentMgr::update (double dt)
 				   _environment->get_wind_from_east_fps(),
 				   _environment->get_wind_from_down_fps());
   _environment->set_elevation_ft(fgGetDouble("/position/altitude-ft"));
+  osg::Vec3 windVec(-_environment->get_wind_from_north_fps(),
+                    -_environment->get_wind_from_east_fps(),
+                    _environment->get_wind_from_down_fps());
+  simgear::Particles::setWindVector(windVec * SG_FEET_TO_METER);
 
   _update_fdm();
 }
@@ -279,6 +288,18 @@ FGEnvironmentMgr::getEnvironment (double lat, double lon, double alt) const
   return env;
 }
 
+FGEnvironment
+FGEnvironmentMgr::getEnvironment(const SGGeod& aPos) const
+{
+  // Always returns the same environment
+  // for now; we'll make it interesting
+  // later.
+  FGEnvironment env = *_environment;
+  env.set_elevation_ft(aPos.getElevationFt());
+  return env;
+
+}
+
 void
 FGEnvironmentMgr::_update_fdm () const
 {
