@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: ExternalPipe.cxx,v 1.24 2006-02-21 01:16:11 mfranz Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -32,6 +32,7 @@
 #endif
 
 #include <stdio.h>              // FILE*, fopen(), fread(), fwrite(), et. al.
+#include <iostream>             // for cout, endl
 
 #include <simgear/debug/logstream.hxx>
 #include <simgear/io/lowlevel.hxx> // endian tests
@@ -44,6 +45,8 @@
 
 #include "ExternalPipe.hxx"
 
+using std::cout;
+using std::endl;
 
 static const int MAX_BUF = 32768;
 
@@ -522,6 +525,7 @@ void FGExternalPipe::update_property( double dt ) {
             cout << "Error reading data" << endl;
         } else {
             // cout << "  read " << strlen(cmd) << " bytes" << endl;
+            // cout << cmd << endl;
         }
 
         // chop trailing newline
