@@ -37,8 +37,7 @@ INCLUDES
 #endif
 
 #include <string>
-#include <stdlib.h>
-
+#include <cstdlib>
 #include <FGFDMExec.h>
 #include <models/FGAtmosphere.h>
 #include "FGInitialCondition.h"
@@ -47,10 +46,9 @@ INCLUDES
 #include <models/FGPropulsion.h>
 #include <models/FGAerodynamics.h>
 
-
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGTrimAxis.cpp,v 1.2.2.1 2007-01-06 10:49:23 ehofman Exp $";
+static const char *IdSrc = "$Id$";
 static const char *IdHdr = ID_TRIMAXIS;
 
 /*****************************************************************************/
