@@ -45,7 +45,7 @@ INCLUDES
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_GROUNDCALLBACK "$Id: FGGroundCallback.h,v 1.1.2.1 2007-01-06 10:49:23 ehofman Exp $"
+#define ID_GROUNDCALLBACK "$Id$"
 
 namespace JSBSim {
 
@@ -59,7 +59,7 @@ CLASS DOCUMENTATION
     ball formed earth.
 
     @author Mathias Froehlich
-    @version $Id: FGGroundCallback.h,v 1.1.2.1 2007-01-06 10:49:23 ehofman Exp $
+    @version $Id$
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -69,7 +69,16 @@ CLASS DECLARATION
 class FGGroundCallback : public FGJSBBase
 {
 public:
+  /** Default constructor.
+  Within this constructor, the reference radius is set to the WGS84 equatorial
+  radius. This constructor should really not be called, instead relying on the 
+  constructor that takes reference radius as an argument. */
   FGGroundCallback();
+
+  /** Constructor
+  This constructor accepts the reference radius in feet. This is the preferred  
+  constructor. */
+  FGGroundCallback(double ReferenceRadius);
   virtual ~FGGroundCallback();
 
   /** Compute the altitude above sealevel. */
@@ -77,6 +86,8 @@ public:
   /** Compute the altitude above ground. Defaults to sealevel altitude. */
   virtual double GetAGLevel(double t, const FGLocation& l, FGLocation& cont,
                             FGColumnVector3& n, FGColumnVector3& v) const;
+  virtual void SetTerrainGeoCentRadius(double radius) {mReferenceRadius = radius;}
+  virtual double GetTerrainGeoCentRadius(void) const {return mReferenceRadius;}
 private:
   /// Reference radius.
   double mReferenceRadius;
