@@ -49,7 +49,7 @@ SENTRY
   DEFINITIONS
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_QUATERNION "$Id: FGQuaternion.h,v 1.1.2.1 2007-01-06 10:49:24 ehofman Exp $"
+#define ID_QUATERNION "$Id$"
 
 namespace JSBSim {
 
@@ -146,11 +146,13 @@ public:
   /// Destructor.
   ~FGQuaternion() {}
 
-  /** Quaternion 'velocity' for given angular rates.
+  /** Quaternion derivative for given angular rates.
       Computes the quaternion derivative which results from the given
       angular velocities
       @param PQR a constant reference to the body rate vector
-      @return the quaternion derivative */
+      @return the quaternion derivative
+      @see Stevens and Lewis, "Aircraft Control and Simulation", Second Edition,
+           Equation 1.3-36. */
   FGQuaternion GetQDot(const FGColumnVector3& PQR) const;
 
   /** Transformation matrix.
@@ -166,7 +168,7 @@ public:
   /** Retrieves the Euler angles.
       @return a reference to the triad of euler angles corresponding
       to this quaternion rotation.
-      @units radians  */
+      units radians  */
   const FGColumnVector3& GetEuler(void) const {
     ComputeDerived();
     return mEulerAngles;
@@ -174,9 +176,10 @@ public:
 
   /** Retrieves the Euler angles.
       @param i the euler angle index.
+      units radians.
       @return a reference to the i-th euler angles corresponding
       to this quaternion rotation.
-      @units radians */
+   */
   double GetEuler(int i) const {
     ComputeDerived();
     return mEulerAngles(i);
@@ -186,7 +189,7 @@ public:
       @param i the euler angle index.
       @return a reference to the i-th euler angles corresponding
       to this quaternion rotation.
-      @units degrees */
+      units degrees */
   double GetEulerDeg(int i) const {
     ComputeDerived();
     return radtodeg*mEulerAngles(i);
@@ -237,8 +240,8 @@ public:
       Return the value of the matrix entry at the given index.
       Indices are counted starting with 1.
 
-      This function is just a shortcut for the @ref double
-      operator()(unsigned int idx) const function. It is
+      This function is just a shortcut for the <tt>double
+      operator()(unsigned int idx) const</tt> function. It is
       used internally to access the elements in a more convenient way.
 
       Note that the index given in the argument is unchecked.
@@ -252,8 +255,8 @@ public:
       Return a reference to the vector entry at the given index.
       Indices are counted starting with 1.
 
-      This function is just a shortcut for the @ref double&
-      operator()(unsigned int idx) function. It is
+      This function is just a shortcut for the <tt>double&
+      operator()(unsigned int idx)</tt> function. It is
       used internally to access the elements in a more convenient way.
 
       Note that the index given in the argument is unchecked.
@@ -481,7 +484,7 @@ private:
 /** Scalar multiplication.
 
     @param scalar scalar value to multiply with.
-    @param p Vector to multiply.
+    @param q Vector to multiply.
 
     Multiply the Vector with a scalar value.
 */
