@@ -37,18 +37,13 @@ INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #include "FGTable.h"
-
-#if defined ( sgi ) && !defined( __GNUC__ ) && (_COMPILER_VERSION < 740)
-# include <iomanip.h>
-#else
-# include <iomanip>
-#endif
+#include <iomanip>
 
 using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGTable.cpp,v 1.5.2.2 2007-05-27 09:33:09 ehofman Exp $";
+static const char *IdSrc = "$Id$";
 static const char *IdHdr = ID_TABLE;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -300,11 +295,6 @@ double** FGTable::Allocate(void)
 
 FGTable::~FGTable()
 {
-  if (!Name.empty() && !internal) {
-    string tmp = PropertyManager->mkPropertyName(Name, false); // Allow upper case
-    PropertyManager->Untie(tmp);
-  }
-
   if (nTables > 0) {
     for (unsigned int i=0; i<nTables; i++) delete Tables[i];
     Tables.clear();
