@@ -38,16 +38,7 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#ifdef FGFS
-#  include <simgear/compiler.h>
-#  ifdef SG_HAVE_STD_INCLUDES
-#    include <vector>
-#  else
-#    include <vector.h>
-#  endif
-#else
-#  include <vector>
-#endif
+#include <vector>
 
 #include <string>
 #include <models/flight_control/FGFCSComponent.h>
@@ -59,7 +50,7 @@ INCLUDES
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_FCS "$Id: FGFCS.h,v 1.2.2.2 2007-05-27 09:33:10 ehofman Exp $"
+#define ID_FCS "$Id$"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -74,12 +65,13 @@ CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 /** Encapsulates the Flight Control System (FCS) functionality.
-    This class owns and contains the list of FGFCSComponents
-    that define the control system for this aircraft. The config file for the
-    aircraft contains a description of the control path that starts at an input
-    or command and ends at an effector, e.g. an aerosurface. The FCS components
-    which comprise the control laws for an axis are defined sequentially in
-    the configuration file. For instance, for the X-15:
+    This class also encapsulates the identical "system" and "autopilot" capability.
+    FGFCS owns and contains the list of FGFCSComponents
+    that define a system or systems for the modeled aircraft. The config file
+    for the aircraft contains a description of the control path that starts at
+    an input or command and ends at an effector, e.g. an aerosurface. The FCS
+    components which comprise the control laws for an axis are defined
+    sequentially in the configuration file. For instance, for the X-15:
 
     @code
     <flight_control name="X-15 SAS">
@@ -119,14 +111,14 @@ CLASS DOCUMENTATION
     addition to using the "NAME" attribute in,
 
     @code
-    \<flight_control name="X-15 SAS">
+    <flight_control name="X-15 SAS">
     @endcode
 
     one can also supply a filename:
 
     @code
-    \<flight_control name="X-15 SAS" file="X15.xml">
-    \</flight_control>
+    <flight_control name="X-15 SAS" file="X15.xml">
+    </flight_control>
     @endcode
 
     In this case, the FCS would be read in from another file.
@@ -170,21 +162,24 @@ CLASS DOCUMENTATION
     @property fcs/spoiler-pos-deg
     @property fcs/spoiler-pos-norm
     @property fcs/mag-spoiler-pos-rad
+    @property fcs/wing-fold-pos-norm
     @property gear/gear-pos-norm
+    @property gear/tailhook-pos-norm
 
     @author Jon S. Berndt
-    @version $Revision: 1.2.2.2 $
-    @see FGFCSComponent
-    @see FGXMLElement
+    @version $Revision$
+    @see FGActuator
+    @see FGDeadBand
+    @see FGFCSFunction
+    @see FGFilter
     @see FGGain
+    @see FGKinemat
+    @see FGPID
+    @see FGSensor
     @see FGSummer
     @see FGSwitch
-    @see FGFCSFunction
-    @see FGCondition
-    @see FGGradient
-    @see FGFilter
-    @see FGDeadBand
-    @see FGKinemat
+    @see FGFCSComponent
+    @see Element
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -201,6 +196,8 @@ public:
   /// Destructor
   ~FGFCS();
 
+  bool InitModel(void);
+
   /** Runs the Flight Controls model; called by the Executive
       @return false if no error */
   bool Run(void);
@@ -330,6 +327,14 @@ public:
       @return gear position (0 up, 1 down) */
   inline double GetGearPos(void) const { return GearPos; }
 
+  /** Gets the tailhook position (0 up, 1 down)
+      @return tailhook position (0 up, 1 down) */
+  inline double GetTailhookPos(void) const { return TailhookPos; }
+
+  /** Gets the wing fold position (0 unfolded, 1 folded)
+      @return wing fold position (0 unfolded, 1 folded) */
+  inline double GetWingFoldPos(void) const { return WingFoldPos; }
+
   /** Gets the prop pitch position.
       @param engine engine ID number
       @return prop pitch position for the given engine in range from 0 - 1.0 */
@@ -472,6 +477,13 @@ public:
       @param gear position 0 up, 1 down       */
    void SetGearPos(double gearpos) { GearPos = gearpos; }
 
+  /** Set the tailhook position
+      @param tailhook position 0 up, 1 down       */
+   void SetTailhookPos(double hookpos) { TailhookPos = hookpos; }
+
+  /** Set the wing fold position
+      @param wing fold position 0 unfolded, 1 folded  */
+   void SetWingFoldPos(double foldpos) { WingFoldPos = foldpos; }
 
   /** Sets the actual prop pitch setting for the specified engine
       @param engine engine ID number
@@ -516,11 +528,17 @@ public:
   double GetCBrake(void) const {return CenterBrake;}
   //@}
 
+  enum SystemType { stFCS, stSystem, stAutoPilot }; 
+
   /** Loads the Flight Control System.
       Load() is called from FGFDMExec.
       @param el pointer to the Element instance
+      @param systype type of system (FCS, Autopilot, System) 
       @return true if succesful */
-  bool Load(Element* el);
+  bool Load(Element* el, SystemType systype);
+
+  ifstream* FindSystemFile(string system_filename);
+  string FindSystemFullPathname(string system_filename);
 
   void AddThrottle(void);
   void AddGear(void);
@@ -544,15 +562,17 @@ private:
   vector <double> SteerPosDeg;
   double LeftBrake, RightBrake, CenterBrake; // Brake settings
   double GearCmd,GearPos;
+  double TailhookPos, WingFoldPos;
 
-  vector <FGFCSComponent*> FCSComponents;
-  vector <FGFCSComponent*> APComponents;
+  typedef vector <FGFCSComponent*> FCSCompVec;
+  FCSCompVec Systems;
+  FCSCompVec FCSComponents;
+  FCSCompVec APComponents;
+  FCSCompVec sensors;
   vector <double*> interface_properties;
-  vector <FGFCSComponent*> sensors;
   void bind(void);
   void bindModel(void);
   void bindThrottle(unsigned int);
-  void unbind(FGPropertyManager *node);
   void Debug(int from);
 };
 }
