@@ -38,23 +38,14 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#ifdef FGFS
-#  include <simgear/compiler.h>
-#  ifdef SG_HAVE_STD_INCLUDES
-#    include <vector>
-#  else
-#    include <vector.h>
-#  endif
-#else
-#  include <vector>
-#endif
+#include <vector>
 
 #include "FGModel.h"
 #include "FGLGear.h"
 #include <math/FGColumnVector3.h>
 #include <input_output/FGXMLElement.h>
 
-#define ID_GROUNDREACTIONS "$Id: FGGroundReactions.h,v 1.1.2.1 2007-01-06 10:49:25 ehofman Exp $"
+#define ID_GROUNDREACTIONS "$Id$"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -66,7 +57,21 @@ namespace JSBSim {
 CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-/** Manages ground reactions modeling.
+/** Manages ground reactions modeling. Maintains a list of landing gear and
+    ground contact points, all instances of FGLGear.  Sums their forces and
+    moments so that these may be provided to FGPropagate.  Parses the 
+    \<ground_reactions> section of the aircraft configuration file.
+ <h3>Configuration File Format of \<ground_reactions> Section:</h3>
+@code
+    <ground_reactions>
+        <contact>
+           ... {see FGLGear for specifics of this format}
+        </contact>
+        ... {more contacts}
+    </ground_reactions>
+@endcode   
+
+
   */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -79,6 +84,7 @@ public:
   FGGroundReactions(FGFDMExec*);
   ~FGGroundReactions(void);
 
+  bool InitModel(void);
   bool Run(void);
   bool Load(Element* el);
   FGColumnVector3& GetForces(void) {return vForces;}
@@ -89,21 +95,19 @@ public:
   string GetGroundReactionValues(string delimeter);
   bool GetWOW(void);
 
-  int GetNumGearUnits(void) const { return lGear.size(); }
+  int GetNumGearUnits(void) const { return (int)lGear.size(); }
 
   /** Gets a gear instance
       @param gear index of gear instance
       @return a pointer to the FGLGear instance of the gear unit requested */
-  inline FGLGear* GetGearUnit(int gear) { return &(lGear[gear]); }
-
-  void bind(void);
-  void unbind(void);
+  inline FGLGear* GetGearUnit(int gear) { return lGear[gear]; }
 
 private:
-  vector <FGLGear> lGear;
+  vector <FGLGear*> lGear;
   FGColumnVector3 vForces;
   FGColumnVector3 vMoments;
 
+  void bind(void);
   void Debug(int from);
 };
 }
