@@ -48,7 +48,11 @@ INCLUDES
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_MASSBALANCE "$Id: FGMassBalance.h,v 1.1.2.1 2007-01-06 10:49:25 ehofman Exp $"
+#define ID_MASSBALANCE "$Id$"
+
+#if defined(WIN32) && !defined(__CYGWIN__)
+#define snprintf _snprintf
+#endif
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONSS
@@ -60,7 +64,36 @@ namespace JSBSim {
 CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-/** Models weight and balance information.
+/** Models weight, balance and moment of inertia information.  Maintains a vector
+    of point masses. Sums the contribution of all, and provides this to FGPropagate.
+    Loads the \<mass_balance> section of the aircraft configuration file.
+
+    <h3>Configuration File Format:</h3>
+@code
+    <mass_balance>
+        <ixx unit="{SLUG*FT2 | KG*M2}"> {number} </ixx>
+        <iyy unit="{SLUG*FT2 | KG*M2}"> {number} </iyy>
+        <izz unit="{SLUG*FT2 | KG*M2}"> {number} </izz>
+        <ixy unit="{SLUG*FT2 | KG*M2}"> {number} </ixy>
+        <ixz unit="{SLUG*FT2 | KG*M2}"> {number} </ixz>
+        <iyz unit="{SLUG*FT2 | KG*M2}"> {number} </iyz>
+        <emptywt unit="{LBS | KG"> {number} </emptywt>
+        <location name="CG" unit="{IN | M}">
+            <x> {number} </x>
+            <y> {number} </y>
+            <z> {number} </z>
+        </location>
+        <pointmass name="{string}">
+            <weight unit="{LBS | KG}"> {number} </weight>
+            <location name="POINTMASS" unit="{IN | M}">
+                <x> {number} </x>
+                <y> {number} </y>
+                <z> {number} </z>
+            </location>
+        </pointmass>
+        ... other point masses ...
+    </mass_balance>
+@endcode
   */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -75,7 +108,7 @@ public:
   ~FGMassBalance();
 
   bool Load(Element* el);
-
+  bool InitModel(void);
   bool Run(void);
 
   inline double GetMass(void) const {return Mass;}
@@ -121,15 +154,13 @@ public:
   inline void SetBaseCG(const FGColumnVector3& CG) {vbaseXYZcg = vXYZcg = CG;}
 
   void AddPointMass(Element* el);
-  double GetPointMassWeight(void);
+  double GetTotalPointMassWeight(void);
+
   FGColumnVector3& GetPointMassMoment(void);
   FGMatrix33& GetJ(void) {return mJ;}
   FGMatrix33& GetJinv(void) {return mJinv;}
   void SetAircraftBaseInertias(FGMatrix33 BaseJ) {baseJ = BaseJ;}
-
-  void bind(void);
-  void unbind(void);
-
+  
 private:
   double Weight;
   double EmptyWeight;
@@ -146,16 +177,38 @@ private:
   FGMatrix33& CalculatePMInertias(void);
 
   struct PointMass {
+    char tmp[80];
     PointMass(double w, FGColumnVector3& vXYZ) {
       Weight = w;
       Location = vXYZ;
     }
     FGColumnVector3 Location;
     double Weight;
+    double GetPointMassLocation(int axis) const {return Location(axis);}
+    void SetPointMassLocation(int axis, double value) {Location(axis) = value;}
+    void SetPointMassWeight(double wt) {Weight = wt;}
+    double GetPointMassWeight(void) const {return Weight;}
+
+    void bind(FGPropertyManager* PropertyManager, int num) {
+      snprintf(tmp, 80, "inertia/pointmass-weight-lbs[%u]", num);
+      PropertyManager->Tie( tmp, this, &PointMass::GetPointMassWeight,
+                                       &PointMass::SetPointMassWeight);
+
+      snprintf(tmp, 80, "inertia/pointmass-location-X-inches[%u]", num);
+      PropertyManager->Tie( tmp, this, eX, &PointMass::GetPointMassLocation,
+                                           &PointMass::SetPointMassLocation);
+      snprintf(tmp, 80, "inertia/pointmass-location-Y-inches[%u]", num);
+      PropertyManager->Tie( tmp, this, eY, &PointMass::GetPointMassLocation,
+                                           &PointMass::SetPointMassLocation);
+      snprintf(tmp, 80, "inertia/pointmass-location-Z-inches[%u]", num);
+      PropertyManager->Tie( tmp, this, eZ, &PointMass::GetPointMassLocation,
+                                           &PointMass::SetPointMassLocation);
+    }
   };
 
-  vector <struct PointMass> PointMasses;
+  vector <struct PointMass*> PointMasses;
 
+  void bind(void);
   void Debug(int from);
 };
 }
