@@ -26,6 +26,7 @@
 HISTORY
 --------------------------------------------------------------------------------
 12/02/98   JSB   Created
+11/09/07   HDW   Added FlightGear Socket Interface
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 SENTRY
@@ -40,28 +41,19 @@ INCLUDES
 
 #include "FGModel.h"
 
-#ifdef FGFS
-#  include <simgear/compiler.h>
-#  include STL_IOSTREAM
-#  include STL_FSTREAM
-#else
-#  if defined(sgi) && !defined(__GNUC__) && (_COMPILER_VERSION < 740)
-#    include <iostream.h>
-#    include <fstream.h>
-#  else
-#    include <iostream>
-#    include <fstream>
-#  endif
-#endif
+#include <iostream>
+#include <fstream>
+
+#include "input_output/FGfdmSocket.h"
+#include "input_output/FGXMLFileRead.h"
+#include "input_output/net_fdm.hxx"
 
-#include <input_output/FGfdmSocket.h>
-#include <input_output/FGXMLFileRead.h>
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_OUTPUT "$Id: FGOutput.h,v 1.1.2.2 2007-05-27 09:33:10 ehofman Exp $"
+#define ID_OUTPUT "$Id$"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -77,23 +69,39 @@ CLASS DOCUMENTATION
     OUTPUT section definition
 
     The following specifies the way that JSBSim writes out data.
-
+<pre>
     NAME is the filename you want the output to go to
 
     TYPE can be:
-      CSV       Comma separated data. If a filename is supplied then the data
-                goes to that file. If COUT or cout is specified, the data goes
-                to stdout. If the filename is a null filename the data goes to
-                stdout, as well.
-      SOCKET    Will eventually send data to a socket output, where NAME
-                would then be the IP address of the machine the data should be
-                sent to. DON'T USE THIS YET!
-      TABULAR   Columnar data. NOT IMPLEMENTED YET!
-      TERMINAL  Output to terminal. NOT IMPLEMENTED YET!
-      NONE      Specifies to do nothing. THis setting makes it easy to turn on and
-                off the data output without having to mess with anything else.
+      CSV         Comma separated data. If a filename is supplied then the
+                  data goes to that file. If "COUT" or "cout" is specified, the
+                  data goes to stdout. If the filename is a null filename the
+                  data goes to stdout, as well.
+      SOCKET      Will eventually send data to a socket output, where NAME
+                  would then be the IP address of the machine the data should
+                  be sent to. DON'T USE THIS YET!
+      FLIGHTGEAR  A socket is created for sending binary data packets to
+                  an external instance of FlightGear for visuals.  Parameters
+                  defining the socket are given on the \<output> line.
+      TABULAR     Columnar data. NOT IMPLEMENTED YET!
+      TERMINAL    Output to terminal. NOT IMPLEMENTED YET!
+      NONE        Specifies to do nothing. This setting makes it easy to turn on and
+                  off the data output without having to mess with anything else.
 
-    The arguments that can be supplied, currently, are
+      Examples:
+</pre>
+@code
+	<output name="localhost" type="FLIGHTGEAR" port="5500" protocol="tcp" rate="10"></output>
+@endcode
+@code
+	<output name="B737_datalog.csv" type="CSV" rate="20">
+	   <property> velocities/vc-kts </property>
+	   <velocities> ON </velocities>
+	</output>
+@endcode
+<br>
+<pre>
+    The arguments that can be supplied, currently, are:
 
     RATE_IN_HZ  An integer rate in times-per-second that the data is output. This
                 value may not be *exactly* what you want, due to the dependence
@@ -101,22 +109,22 @@ CLASS DOCUMENTATION
 
     The following parameters tell which subsystems of data to output:
 
-    SIMULATION       ON|OFF
-    ATMOSPHERE       ON|OFF
-    MASSPROPS        ON|OFF
-    AEROSURFACES     ON|OFF
-    RATES            ON|OFF
-    VELOCITIES       ON|OFF
-    FORCES           ON|OFF
-    MOMENTS          ON|OFF
-    POSITION         ON|OFF
-    COEFFICIENTS     ON|OFF
-    GROUND_REACTIONS ON|OFF
-    FCS              ON|OFF
-    PROPULSION       ON|OFF
-
+    simulation       ON|OFF
+    atmosphere       ON|OFF
+    massprops        ON|OFF
+    aerosurfaces     ON|OFF
+    rates            ON|OFF
+    velocities       ON|OFF
+    forces           ON|OFF
+    moments          ON|OFF
+    position         ON|OFF
+    coefficients     ON|OFF
+    ground_reactions ON|OFF
+    fcs              ON|OFF
+    propulsion       ON|OFF
+</pre>
     NOTE that Time is always output with the data.
-    @version $Id: FGOutput.h,v 1.1.2.2 2007-05-27 09:33:10 ehofman Exp $
+    @version $Id$
  */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -129,11 +137,16 @@ public:
   FGOutput(FGFDMExec*);
   ~FGOutput();
 
+  bool InitModel(void);
   bool Run(void);
 
   void DelimitedOutput(string);
   void SocketOutput(void);
+  void FlightGearSocketOutput(void);
   void SocketStatusOutput(string);
+  void SocketDataFill(FGNetFDM* net);
+
+
   void SetType(string);
   void SetSubsystems(int tt) {SubSystems = tt;}
   inline void Enable(void) { enabled = true; }
@@ -161,13 +174,17 @@ public:
     /** Subsystem: Propulsion (= 4096)       */ ssPropulsion      = 4096
   } subsystems;
 
+
+  FGNetFDM fgSockBuf;
+
 private:
-  enum {otNone, otCSV, otTab, otSocket, otTerminal, otUnknown} Type;
+  enum {otNone, otCSV, otTab, otSocket, otTerminal, otFlightGear, otUnknown} Type;
   bool sFirstPass, dFirstPass, enabled;
   int SubSystems;
   string output_file_name, delimeter, Filename, DirectivesFile;
   ofstream datafile;
   FGfdmSocket* socket;
+  FGfdmSocket* flightGearSocket;
   vector <FGPropertyManager*> OutputProperties;
 
   void Debug(int from);
