@@ -41,7 +41,7 @@ INCLUDES
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGFCSComponent.cpp,v 1.2.2.2 2007-05-27 09:33:11 ehofman Exp $";
+static const char *IdSrc = "$Id$";
 static const char *IdHdr = ID_FCSCOMPONENT;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -113,7 +113,7 @@ FGFCSComponent::FGFCSComponent(FGFCS* _fcs, Element* element) : fcs(_fcs)
       InputNodes.push_back( tmp );
     } else {
       cerr << fgred << "  In component: " << Name << " unknown property "
-           << input << " referenced. Aborting" << endl;
+           << input << " referenced. Aborting" << reset << endl;
       exit(-1);
     }
     input_element = element->FindNextElement("input");
@@ -121,7 +121,7 @@ FGFCSComponent::FGFCSComponent(FGFCS* _fcs, Element* element) : fcs(_fcs)
 
   if (element->FindElement("output")) {
     IsOutput = true;
-    OutputNode = PropertyManager->GetNode( element->FindElementValue("output") );
+    OutputNode = PropertyManager->GetNode( element->FindElementValue("output"), true );
     if (!OutputNode) {
       cerr << endl << "  Unable to process property: " << element->FindElementValue("output") << endl;
       throw(string("Invalid output property name in flight control definition"));
@@ -132,14 +132,20 @@ FGFCSComponent::FGFCSComponent(FGFCS* _fcs, Element* element) : fcs(_fcs)
   if (clip_el) {
     clip_string = clip_el->FindElementValue("min");
     if (clip_string.find_first_not_of("+-.0123456789") != string::npos) { // it's a property
-      if (clip_string[0] == '-') clipMinSign = -1.0;
+      if (clip_string[0] == '-') {
+        clipMinSign = -1.0;
+        clip_string.erase(0,1);
+      }
       ClipMinPropertyNode = PropertyManager->GetNode( clip_string );
     } else {
       clipmin = clip_el->FindElementValueAsNumber("min");
     }
     clip_string = clip_el->FindElementValue("max");
     if (clip_string.find_first_not_of("+-.0123456789") != string::npos) { // it's a property
-      if (clip_string[0] == '-') clipMaxSign = -1.0;
+      if (clip_string[0] == '-') {
+        clipMaxSign = -1.0;
+        clip_string.erase(0,1);
+      }
       ClipMaxPropertyNode = PropertyManager->GetNode( clip_string );
     } else {
       clipmax = clip_el->FindElementValueAsNumber("max");
@@ -223,12 +229,31 @@ void FGFCSComponent::bind(void)
 
 void FGFCSComponent::Debug(int from)
 {
+  string propsign="";
+
   if (debug_lvl <= 0) return;
 
   if (debug_lvl & 1) { // Standard console startup message output
     if (from == 0) {
       cout << endl << "    Loading Component \"" << Name
                    << "\" of type: " << Type << endl;
+
+      if (clip) {
+        if (ClipMinPropertyNode != 0L) {
+          if (clipMinSign < 0.0) propsign="-";
+          cout << "      Minimum limit: " << propsign << ClipMinPropertyNode->GetName() << endl;
+          propsign="";
+        } else {
+          cout << "      Minimum limit: " << clipmin << endl;
+        }
+        if (ClipMaxPropertyNode != 0L) {
+          if (clipMaxSign < 0.0) propsign="-";
+          cout << "      Maximum limit: " << propsign << ClipMaxPropertyNode->GetName() << endl;
+          propsign="";
+        } else {
+          cout << "      Maximum limit: " << clipmax << endl;
+        }
+      }  
     }
   }
   if (debug_lvl & 2 ) { // Instantiation/Destruction notification
