@@ -37,23 +37,20 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#ifdef FGFS
-#  include <simgear/compiler.h>
-#endif
-
-#include <string>
-#include <vector>
 #include <FGJSBBase.h>
 #include <input_output/FGPropertyManager.h>
 #include <input_output/FGXMLElement.h>
+#include <string>
+#include <vector>
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_FCSCOMPONENT "$Id: FGFCSComponent.h,v 1.2.2.1 2007-01-06 10:49:25 ehofman Exp $"
+#define ID_FCSCOMPONENT "$Id$"
 
 using std::string;
+using std::vector;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -83,7 +80,7 @@ CLASS DOCUMENTATION
     - FGPID
 
     @author Jon S. Berndt
-    @version $Id: FGFCSComponent.h,v 1.2.2.1 2007-01-06 10:49:25 ehofman Exp $
+    @version $Id$
     @see Documentation for the FGFCS class, and for the configuration file class
 */
 
