@@ -37,26 +37,19 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#ifdef FGFS
-#  include <simgear/compiler.h>
-#  ifdef SG_HAVE_STD_INCLUDES
-#    include <vector>
-#  else
-#    include <vector.h>
-#  endif
-#else
-#  include <vector>
-#endif
-
-#include <string>
 #include "FGFCSComponent.h"
 #include <input_output/FGXMLElement.h>
+#include <vector>
+#include <string>
+
+using std::vector;
+using std::string;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_FLAPS "$Id: FGKinemat.h,v 1.1.2.1 2007-01-06 10:49:25 ehofman Exp $"
+#define ID_FLAPS "$Id$"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -81,7 +74,7 @@ vehicle control or configuration. The form of the component specification is:
       <position> number </position>
       <time> number </time>
     </setting>
-    
+    ...
   </traverse>
   [<clipto>
     <min> {[-]property name | value} </min>
@@ -123,12 +116,8 @@ CLASS DECLARATION
 class FGKinemat  : public FGFCSComponent {
 public:
   /** Constructor.
-      @param fcs A reference to the ccurrent flightcontrolsystem.
-      @param AC_cfg reference to the current aircraft configuration file.
-      Initializes the FGKinemat object from the given configuration
-      file. The Configuration file is expected to be at the stream
-      position where the kinematic object starts. Also it is expected to
-      be past the end of the current kinematic configuration on exit.
+      @param fcs A reference to the current flight control system.
+      @param element reference to the current configuration file node.
    */
   FGKinemat(FGFCS* fcs, Element* element);
 
