@@ -37,31 +37,17 @@ HISTORY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#ifdef FGFS
-#  include <simgear/compiler.h>
-#  ifdef SG_HAVE_STD_INCLUDES
-#    include <fstream>
-#  else
-#    include <fstream.h>
-#  endif
-#else
-#  if defined(sgi) && !defined(__GNUC__) && (_COMPILER_VERSION < 740)
-#    include <fstream.h>
-#  else
-#    include <fstream>
-#  endif
-#endif
-
 #include "FGEngine.h"
 #include "FGTank.h"
 #include "FGPropeller.h"
 #include "FGNozzle.h"
 #include <input_output/FGXMLParse.h>
 #include <math/FGColumnVector3.h>
+#include <fstream>
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGEngine.cpp,v 1.2.2.2 2007-05-27 09:33:11 ehofman Exp $";
+static const char *IdSrc = "$Id$";
 static const char *IdHdr = ID_ENGINE;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -78,20 +64,11 @@ FGEngine::FGEngine(FGFDMExec* exec, Element* engine_element, int engine_number)
   Type = etUnknown;
   X = Y = Z = 0.0;
   EnginePitch = EngineYaw = 0.0;
-  SLFuelFlowMax = SLOxiFlowMax = 0.0;
+  SLFuelFlowMax = 0.0;
   MaxThrottle = 1.0;
   MinThrottle = 0.0;
-  Thrust = 0.0;
-  Throttle = 0.0;
-  Mixture = 1.0;
-  Starter = false;
-  FuelNeed = OxidizerNeed = 0.0;
-  Starved = Running = Cranking = false;
-  PctPower = 0.0;
-  TrimMode = false;
-  FuelFlow_gph = 0.0;
-  FuelFlow_pph = 0.0;
-  FuelFreeze = false;
+
+  ResetToIC(); // initialize dynamic terms
 
   FDMExec = exec;
   State = FDMExec->GetState();
@@ -113,15 +90,16 @@ FGEngine::FGEngine(FGFDMExec* exec, Element* engine_element, int engine_number)
   else      cerr << "No engine location found for this engine." << endl;
 
   local_element = engine_element->GetParent()->FindElement("orient");
-  if (local_element)  orientation = local_element->FindElementTripletConvertTo("DEG");
-  else          cerr << "No engine orientation found for this engine." << endl;
+  if (local_element)  orientation = local_element->FindElementTripletConvertTo("RAD");
+//  else          cerr << "No engine orientation found for this engine." << endl;
+// Jon: The engine orientation has a default and is not normally used.
 
   SetPlacement(location, orientation);
 
   // Load thruster
   local_element = engine_element->GetParent()->FindElement("thruster");
   if (local_element) {
-    LoadThruster(local_element);
+    if (!LoadThruster(local_element)) exit(-1);
   } else {
     cerr << "No thruster definition supplied with engine definition." << endl;
   }
@@ -137,6 +115,14 @@ FGEngine::FGEngine(FGFDMExec* exec, Element* engine_element, int engine_number)
     cerr << "No feed tank specified in engine definition." << endl;
   }
 
+  char property_name[80];
+  snprintf(property_name, 80, "propulsion/engine[%d]/set-running", EngineNumber);
+  PropertyManager->Tie( property_name, this, &FGEngine::GetRunning, &FGEngine::SetRunning );
+  snprintf(property_name, 80, "propulsion/engine[%u]/thrust-lbs", EngineNumber);
+  PropertyManager->Tie( property_name, this, &FGEngine::GetThrust);
+  snprintf(property_name, 80, "propulsion/engine[%u]/fuel-flow-rate-pps", EngineNumber);
+  PropertyManager->Tie( property_name, this, &FGEngine::GetFuelFlowRate);
+
   Debug(0);
 }
 
@@ -148,11 +134,29 @@ FGEngine::~FGEngine()
   Debug(1);
 }
 
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+void FGEngine::ResetToIC(void)
+{
+  Thrust = 0.0;
+  Throttle = 0.0;
+  Mixture = 1.0;
+  Starter = false;
+  FuelExpended = 0.0;
+  Starved = Running = Cranking = false;
+  PctPower = 0.0;
+  TrimMode = false;
+  FuelFlow_gph = 0.0;
+  FuelFlow_pph = 0.0;
+  FuelFreeze = false;
+}
+
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 // This base class function should be called from within the
 // derived class' Calculate() function before any other calculations are done.
 // This base class method removes fuel from the fuel tanks as appropriate,
 // and sets the starved flag if necessary.
+// This version of the fuel consumption code should never see an oxidizer tank.
 
 void FGEngine::ConsumeFuel(void)
 {
@@ -160,22 +164,20 @@ void FGEngine::ConsumeFuel(void)
   if (TrimMode) return;
 
   unsigned int i;
-  double Fshortage, Oshortage, TanksWithFuel, TanksWithOxidizer;
+  double Fshortage, TanksWithFuel;
   FGTank* Tank;
-  bool haveOxTanks = false;
-  Fshortage = Oshortage = TanksWithFuel = TanksWithOxidizer = 0.0;
+  Fshortage = TanksWithFuel = 0.0;
 
   // count how many assigned tanks have fuel
   for (i=0; i<SourceTanks.size(); i++) {
     Tank = Propulsion->GetTank(SourceTanks[i]);
     if (Tank->GetType() == FGTank::ttFUEL){
       if (Tank->GetContents() > 0.0) ++TanksWithFuel;
-    } else if (Tank->GetType() == FGTank::ttOXIDIZER) {
-      haveOxTanks = true;
-      if (Tank->GetContents() > 0.0) ++TanksWithOxidizer;
+    } else {
+       cerr << "No oxidizer tanks should be used for this engine type." << endl;
     }
   }
-  if (TanksWithFuel==0 || (haveOxTanks && TanksWithOxidizer==0)) {
+  if (TanksWithFuel==0) {
     Starved = true;
     return;
   }
@@ -184,12 +186,12 @@ void FGEngine::ConsumeFuel(void)
     Tank = Propulsion->GetTank(SourceTanks[i]);
     if (Tank->GetType() == FGTank::ttFUEL) {
        Fshortage += Tank->Drain(CalcFuelNeed()/TanksWithFuel);
-    } else if (Tank->GetType() == FGTank::ttOXIDIZER) {
-       Oshortage += Tank->Drain(CalcOxidizerNeed()/TanksWithOxidizer);
+    } else {
+       cerr << "No oxidizer tanks should be used for this engine type." << endl;
     }
   }
 
-  if (Fshortage < 0.00 || Oshortage < 0.00) Starved = true;
+  if (Fshortage < 0.00) Starved = true;
   else Starved = false;
 }
 
@@ -197,16 +199,10 @@ void FGEngine::ConsumeFuel(void)
 
 double FGEngine::CalcFuelNeed(void)
 {
-  FuelNeed = SLFuelFlowMax*PctPower*State->Getdt()*Propulsion->GetRate();
-  return FuelNeed;
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
-double FGEngine::CalcOxidizerNeed(void)
-{
-  OxidizerNeed = SLOxiFlowMax*PctPower*State->Getdt()*Propulsion->GetRate();
-  return OxidizerNeed;
+  double dT = State->Getdt()*Propulsion->GetRate();
+  FuelFlowRate = SLFuelFlowMax*PctPower;
+  FuelExpended = FuelFlowRate*dT;
+  return FuelExpended;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -254,10 +250,6 @@ bool FGEngine::LoadThruster(Element *thruster_element)
   FGColumnVector3 location, orientation;
   string separator = "/";
 
-# ifdef macintosh
-    separator = ";";
-# endif
-
   fullpath = enginePath + separator;
   localpath = aircraftPath + separator + "Engines" + separator;
 
