@@ -43,30 +43,18 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#ifdef FGFS
-#  include <simgear/compiler.h>
-#  include STL_STRING
-   SG_USING_STD(string);
-#  ifdef SG_HAVE_STD_INCLUDES
-#    include <vector>
-#  else
-#    include <vector.h>
-#  endif
-#else
-#  include <vector>
-#  include <string>
-#endif
-
 #include <FGJSBBase.h>
 #include "FGThruster.h"
 #include <input_output/FGPropertyManager.h>
 #include <input_output/FGXMLFileRead.h>
+#include <vector>
+#include <string>
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_ENGINE "$Id: FGEngine.h,v 1.1.2.2 2007-05-27 09:33:11 ehofman Exp $"
+#define ID_ENGINE "$Id$"
 
 using std::string;
 using std::vector;
@@ -94,9 +82,47 @@ CLASS DOCUMENTATION
 
 /** Base class for all engines.
     This base class contains methods and members common to all engines, such as
-    logic to drain fuel from the appropriate tank, etc.
+    logic to drain fuel from the appropriate tank, etc. 
+    <br>
+    <h3>Configuration File Format:</h3>
+@code
+        <engine file="{string}">
+            <location unit="{IN | M}">
+                <x> {number} </x>
+                <y> {number} </y>
+                <z> {number} </z>
+            </location>
+            <!-- optional orientation definition -->
+            <orient unit="{RAD | DEG}">
+                <roll>  {number} </roll>
+                <pitch> {number} </pitch>
+                <yaw> {number} </yaw>
+            </orient>
+            <feed> {integer} </feed>
+            ... optional more feed tank index numbers ... 
+            <thruster file="{string}">
+                <location unit="{IN | M}">
+                    <x> {number} </x>
+                    <y> {number} </y>
+                    <z> {number} </z>
+                </location>
+                <orient unit="{RAD | DEG}">
+                    <roll> {number} </roll>
+                    <pitch> {number} </pitch>
+                    <yaw> {number} </yaw>
+                </orient>
+            </thruster>
+        </engine>
+@endcode
+<pre>
+    NOTES:
+	Engines feed from all tanks equally.
+
+	Not all thruster types can be matched with a given engine type.  See the class
+	documentation for engine and thruster classes.
+</pre>     
     @author Jon S. Berndt
-    @version $Id: FGEngine.h,v 1.1.2.2 2007-05-27 09:33:11 ehofman Exp $
+    @version $Id$
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -123,9 +149,10 @@ public:
 
   virtual double getFuelFlow_gph () const {return FuelFlow_gph;}
   virtual double getFuelFlow_pph () const {return FuelFlow_pph;}
-  virtual double GetThrust(void) { return Thrust; }
+  virtual double GetFuelFlowRate(void) const {return FuelFlowRate;}
+  virtual double GetThrust(void) const { return Thrust; }
   virtual bool   GetStarved(void) { return Starved; }
-  virtual bool   GetRunning(void) { return Running; }
+  virtual bool   GetRunning(void) const { return Running; }
   virtual bool   GetCranking(void) { return Cranking; }
 
   virtual void SetStarved(bool tt) { Starved = tt; }
@@ -138,29 +165,15 @@ public:
 
   virtual void SetStarter(bool s) { Starter = s; }
 
+  virtual int InitRunning(void){ return 1; }
+
+  /** Resets the Engine parameters to the initial conditions */
+  void ResetToIC(void);
+
   /** Calculates the thrust of the engine, and other engine functions.
       @return Thrust in pounds */
   virtual double Calculate(void) {return 0.0;}
 
-  /** Reduces the fuel in the active tanks by the amount required.
-      This function should be called from within the
-      derived class' Calculate() function before any other calculations are
-      done. This base class method removes fuel from the fuel tanks as
-      appropriate, and sets the starved flag if necessary. */
-  virtual void ConsumeFuel(void);
-
-  /** The fuel need is calculated based on power levels and flow rate for that
-      power level. It is also turned from a rate into an actual amount (pounds)
-      by multiplying it by the delta T and the rate.
-      @return Total fuel requirement for this engine in pounds. */
-  virtual double CalcFuelNeed(void);
-
-  /** The oxidizer need is calculated based on power levels and flow rate for that
-      power level. It is also turned from a rate into an actual amount (pounds)
-      by multiplying it by the delta T and the rate.
-      @return Total oxidizer requirement for this engine in pounds. */
-  virtual double CalcOxidizerNeed(void);
-
   /// Sets engine placement information
   virtual void SetPlacement(FGColumnVector3& location, FGColumnVector3& orientation);
 
@@ -179,6 +192,19 @@ public:
   virtual string GetEngineValues(string delimeter) = 0;
 
 protected:
+  /** Reduces the fuel in the active tanks by the amount required.
+      This function should be called from within the
+      derived class' Calculate() function before any other calculations are
+      done. This base class method removes fuel from the fuel tanks as
+      appropriate, and sets the starved flag if necessary. */
+  virtual void ConsumeFuel(void);
+
+  /** The fuel need is calculated based on power levels and flow rate for that
+      power level. It is also turned from a rate into an actual amount (pounds)
+      by multiplying it by the delta T and the rate.
+      @return Total fuel requirement for this engine in pounds. */
+  virtual double CalcFuelNeed(void);
+
   FGPropertyManager* PropertyManager;
   string Name;
   const int   EngineNumber;
@@ -187,15 +213,14 @@ protected:
   double EnginePitch;
   double EngineYaw;
   double SLFuelFlowMax;
-  double SLOxiFlowMax;
   double MaxThrottle;
   double MinThrottle;
 
   double Thrust;
   double Throttle;
   double Mixture;
-  double FuelNeed;
-  double OxidizerNeed;
+  double FuelExpended;
+  double FuelFlowRate;
   double PctPower;
   bool  Starter;
   bool  Starved;
