@@ -44,7 +44,7 @@ INCLUDES
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_NOZZLE "$Id: FGNozzle.h,v 1.1.2.1 2007-01-06 10:49:25 ehofman Exp $";
+#define ID_NOZZLE "$Id$";
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -56,9 +56,26 @@ namespace JSBSim {
 CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-/** Models a rocket nozzle.
+/** Models a rocket nozzle. 
+
+<h3>Configuration File Format:</h3>
+
+@code
+<nozzle name="{string}">
+  <pe unit="{PSF}"> {number}  </pe>
+  <area unit="{FT2 | M2 | IN2}"> {number}  </area>
+</nozzle>
+@endcode
+
+<h3>Configuration parameters are:</h3>
+<pre>
+    <b>pe</b> -      Nozzle design exit pressure.
+    <b>area</b> -    Nozzle area at the exit plane.
+</pre>
+
+    All parameters MUST be specified.  
     @author Jon S. Berndt
-    @version $Id: FGNozzle.h,v 1.1.2.1 2007-01-06 10:49:25 ehofman Exp $
+    @version $Id$
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -73,18 +90,13 @@ public:
   /// Destructor
   ~FGNozzle();
 
-  double Calculate(double CfPc);
-  double GetPowerRequired(void);
+  double Calculate(double vacThrust);
   string GetThrusterLabels(int id, string delimeter);
   string GetThrusterValues(int id, string delimeter);
 
 private:
   double PE;
-  double ExpR;
-  double nzlEff;
-  double Diameter;
-  double AreaT;
-  double Area2;
+  double Area;
   void Debug(int from);
 };
 }
