@@ -27,7 +27,7 @@ HISTORY
 --------------------------------------------------------------------------------
 09/12/2000  JSB  Created
 10/01/2001  DPM  Modified to use equations from Dave Luff's piston model.
-
+11/01/2008  RKJ  Modified piston engine model for more general use.
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 SENTRY
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -47,7 +47,7 @@ INCLUDES
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_PISTON "$Id: FGPiston.h,v 1.1.2.1 2007-01-06 10:49:26 ehofman Exp $";
+#define ID_PISTON "$Id$";
 #define FG_MAX_BOOST_SPEEDS 3
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -61,6 +61,42 @@ CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 /** Models Dave Luff's Turbo/Supercharged Piston engine model.
+
+<h3>Configuration File Format:</h3>
+
+@code
+<piston_engine name="{string}">
+  <minmp unit="{INHG | PA | ATM}"> {number} </minmp> <!-- Depricated -->
+  <maxmp unit="{INHG | PA | ATM}"> {number} </maxmp> <!-- Depricated -->
+  <displacement unit="{IN3 | LTR | CC}"> {number} </displacement>
+  <sparkfaildrop> {number} </sparkfaildrop>
+  <maxhp unit="{HP | WATTS}"> {number} </maxhp>
+  <cycles> {number} </cycles>
+  <idlerpm> {number} </idlerpm>
+  <maxrpm> {number} </maxrpm>
+  <maxthrottle> {number} </maxthrottle>
+  <minthrottle> {number} </minthrottle>
+  <numboostspeeds> {number} </numboostspeeds>
+  <bsfc unit="{LBS/HP*HR | "KG/KW*HR"}"> {number} </bsft>
+  <volumetric_efficiency> {number} </volumetric_efficiency>
+  <boostoverride> {0 | 1} </boostoverride>
+  <ratedboost1 unit="{INHG | PA | ATM}"> {number} </ratedboost1>
+  <ratedpower1 unit="{HP | WATTS}"> {number} </ratedpower1>
+  <ratedrpm1> {number} </ratedrpm1>
+  <ratedaltitude1 unit="{FT | M}"> {number} </ratedaltitude1>
+  <ratedboost2 unit="{INHG | PA | ATM}"> {number} </ratedboost2>
+  <ratedpower2 unit="{HP | WATTS}"> {number} </ratedpower2>
+  <ratedrpm2> {number} </ratedrpm2>
+  <ratedaltitude2 unit="{FT | M}"> {number} </ratedaltitude2>
+  <ratedboost3 unit="{INHG | PA | ATM}"> {number} </ratedboost3>
+  <ratedpower3 unit="{HP | WATTS}"> {number} </ratedpower3>
+  <ratedrpm3> {number} </ratedrpm3>
+  <ratedaltitude3 unit="{FT | M}"> {number} </ratedaltitude3>
+  <takeoffboost unit="{INHG | PA | ATM}"> {number} </takeoffboost>
+</piston_engine>
+@endcode
+
+<pre>
     Additional elements are required for a supercharged engine.  These can be
     left off a non-supercharged engine, ie. the changes are all backward
     compatible at present.
@@ -127,11 +163,13 @@ CLASS DOCUMENTATION
     Note that MAXMP is still the non-boosted max manifold pressure even for
     boosted engines - effectively this is simply a measure of the pressure drop
     through the fully open throttle.
+</pre>
 
     @author Jon S. Berndt (Engine framework code and framework-related mods)
     @author Dave Luff (engine operational code)
     @author David Megginson (initial porting and additional code)
-    @version $Id: FGPiston.h,v 1.1.2.1 2007-01-06 10:49:26 ehofman Exp $
+    @author Ron Jensen (additional engine code)
+    @version $Id$
   */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -153,6 +191,7 @@ public:
   double GetPowerAvailable(void) {return PowerAvailable;}
   double CalcFuelNeed(void);
 
+  void ResetToIC(void);
   void SetMagnetos(int magnetos) {Magnetos = magnetos;}
 
   double  GetEGT(void) { return EGT_degC; }
@@ -165,6 +204,10 @@ public:
   double getOilTemp_degF (void) {return KelvinToFahrenheit(OilTemp_degK);}
   double getRPM(void) {return RPM;}
 
+protected:
+  double ThrottlePos;
+
+
 private:
   int crank_counter;
 
@@ -188,6 +231,8 @@ private:
   void doOilPressure(void);
   void doOilTemperature(void);
 
+  int InitRunning(void);
+
   //
   // constants
   //
@@ -200,6 +245,7 @@ private:
 
   FGTable *Lookup_Combustion_Efficiency;
   FGTable *Power_Mixture_Correlation;
+  FGTable *Mixture_Efficiency_Correlation;
 
   //
   // Configuration
@@ -208,9 +254,11 @@ private:
   double MaxManifoldPressure_inHg; // Inches Hg
   double Displacement;             // cubic inches
   double MaxHP;                    // horsepower
+  double SparkFailDrop;            // drop of power due to spark failure
   double Cycles;                   // cycles/power stroke
   double IdleRPM;                  // revolutions per minute
-  double StarterHP;                // initial horsepower of starter motor 
+  double MaxRPM;                   // revolutions per minute
+  double StarterHP;                // initial horsepower of starter motor
   int BoostSpeeds;	// Number of super/turbocharger boost speeds - zero implies no turbo/supercharging.
   int BoostSpeed;	// The current boost-speed (zero-based).
   bool Boosted;		// Set true for boosted engine.
@@ -236,6 +284,7 @@ private:
   double minMAP;  // Pa
   double maxMAP;  // Pa
   double MAP;     // Pa
+  double BSFC;    // brake specific fuel consumption [lbs/horsepower*hour
 
   //
   // Inputs (in addition to those in FGEngine).
@@ -249,11 +298,13 @@ private:
   bool Magneto_Right;
   int Magnetos;
 
+
   //
   // Outputs (in addition to those in FGEngine).
   //
   double rho_air;
   double volumetric_efficiency;
+  double suction_loss;
   double m_dot_air;
   double equivalence_ratio;
   double m_dot_fuel;
