@@ -44,7 +44,7 @@ INCLUDES
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGRocket.cpp,v 1.1.2.2 2007-05-27 09:33:11 ehofman Exp $";
+static const char *IdSrc = "$Id$";
 static const char *IdHdr = ID_ROCKET;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -57,18 +57,20 @@ FGRocket::FGRocket(FGFDMExec* exec, Element *el, int engine_number)
   Element* thrust_table_element = 0;
   ThrustTable = 0L;
   BurnTime = 0.0;
+  previousFuelNeedPerTank = 0.0;
+  previousOxiNeedPerTank = 0.0;
+  PropellantFlowRate = 0.0;
+  FuelFlowRate = 0.0;
+  OxidizerFlowRate = 0.0;
+  SLOxiFlowMax = 0.0;
+  It = 0.0;
 
   // Defaults
-   Variance = 0.0;
    MinThrottle = 0.0;
    MaxThrottle = 1.0;
 
-  if (el->FindElement("shr"))
-    SHR = el->FindElementValueAsNumber("shr");
-  if (el->FindElement("max_pc"))
-    maxPC = el->FindElementValueAsNumberConvertTo("max_pc", "PSF");
-  if (el->FindElement("prop_eff"))
-    propEff = el->FindElementValueAsNumber("prop_eff");
+  if (el->FindElement("isp"))
+    Isp = el->FindElementValueAsNumber("isp");
   if (el->FindElement("maxthrottle"))
     MaxThrottle = el->FindElementValueAsNumber("maxthrottle");
   if (el->FindElement("minthrottle"))
@@ -77,21 +79,19 @@ FGRocket::FGRocket(FGFDMExec* exec, Element *el, int engine_number)
     SLFuelFlowMax = el->FindElementValueAsNumberConvertTo("slfuelflowmax", "LBS/SEC");
   if (el->FindElement("sloxiflowmax"))
     SLOxiFlowMax = el->FindElementValueAsNumberConvertTo("sloxiflowmax", "LBS/SEC");
-  if (el->FindElement("variance"))
-    Variance = el->FindElementValueAsNumber("variance");
 
   thrust_table_element = el->FindElement("thrust_table");
   if (thrust_table_element) {
     ThrustTable = new FGTable(PropertyManager, thrust_table_element);
   }
 
+  bindmodel();
+
   Debug(0);
 
   Type = etRocket;
   Flameout = false;
 
-  PC = 0.0;
-  kFactor = (2.0*SHR*SHR/(SHR-1.0))*pow(2.0/(SHR+1), (SHR+1)/(SHR-1));
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -105,41 +105,143 @@ FGRocket::~FGRocket(void)
 
 double FGRocket::Calculate(void)
 {
-  double Cf=0;
+  double dT = State->Getdt()*Propulsion->GetRate();
 
   if (!Flameout && !Starved) ConsumeFuel();
 
+  PropellantFlowRate = (FuelExpended + OxidizerExpended)/dT;
   Throttle = FCS->GetThrottlePos(EngineNumber);
 
-  // If there is a thrust table, it is a function of elapsed burn time. The engine
-  // is started when the throttle is advanced to 1.0. After that, it burns
-  // without regard to throttle setting. The table returns a value between zero
-  // and one, representing the percentage of maximum vacuum thrust being applied.
+  // If there is a thrust table, it is a function of propellant remaining. The
+  // engine is started when the throttle is advanced to 1.0. After that, it
+  // burns without regard to throttle setting. The table returns a value between
+  // zero and one, representing the percentage of maximum vacuum thrust being
+  // applied.
 
-  if (ThrustTable != 0L) {
-    if (Throttle == 1 || BurnTime > 0.0) {
+  if (ThrustTable != 0L) { // Thrust table given -> Solid fuel used
+
+    if ((Throttle == 1 || BurnTime > 0.0 ) && !Starved) {
       BurnTime += State->Getdt();
+      double TotalEngineFuelAvailable=0.0;
+      for (int i=0; i<(int)SourceTanks.size(); i++)
+        TotalEngineFuelAvailable += Propulsion->GetTank(SourceTanks[i])->GetContents();
+
+      VacThrust = ThrustTable->GetValue(TotalEngineFuelAvailable);
+    } else {
+      VacThrust = 0.0;
+    }
+
+  } else { // liquid fueled rocket assumed
+
+    if (Throttle < MinThrottle || Starved) { // Combustion not supported
+
+      PctPower = Thrust = 0.0; // desired thrust
+      Flameout = true;
+      VacThrust = 0.0;
+
+    } else { // Calculate thrust
+
+      PctPower = Throttle / MaxThrottle; // Min and MaxThrottle range from 0.0 to 1.0, normally.
+      Flameout = false;
+      VacThrust = Isp * PropellantFlowRate;
+
+    }
+
+  } // End thrust calculations
+
+  Thrust = Thruster->Calculate(VacThrust);
+  It += Thrust * dT;
+
+  return Thrust;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+// This overrides the base class ConsumeFuel() function, for special rocket
+// engine processing.
+
+void FGRocket::ConsumeFuel(void)
+{
+  unsigned int i;
+  FGTank* Tank;
+  bool haveOxTanks = false;
+  double Fshortage=0, Oshortage=0, TanksWithFuel=0, TanksWithOxidizer=0;
+
+  if (FuelFreeze) return;
+  if (TrimMode) return;
+
+  // Count how many assigned tanks have fuel for this engine at this time.
+  // If there is/are fuel tanks but no oxidizer tanks, this indicates
+  // a solid rocket is being modeled.
+
+  for (i=0; i<SourceTanks.size(); i++) {
+    Tank = Propulsion->GetTank(SourceTanks[i]);
+    switch(Tank->GetType()) {
+      case FGTank::ttFUEL:
+        if (Tank->GetContents() > 0.0 && Tank->GetSelected()) ++TanksWithFuel;
+        break;
+      case FGTank::ttOXIDIZER:
+        haveOxTanks = true;
+        if (Tank->GetContents() > 0.0 && Tank->GetSelected()) ++TanksWithOxidizer;
+        break;
     }
-    Throttle = ThrustTable->GetValue(BurnTime);
   }
 
-  if (Throttle < MinThrottle || Starved) {
-    PctPower = Thrust = 0.0; // desired thrust
-    Flameout = true;
-    PC = 0.0;
+  // If this engine has burned out, it is starved.
+
+  if (TanksWithFuel==0 || (haveOxTanks && TanksWithOxidizer==0)) {
+    Starved = true;
+    return;
+  }
+
+  // Expend fuel from the engine's tanks if the tank is selected as a source
+  // for this engine.
+
+  double fuelNeedPerTank = CalcFuelNeed()/TanksWithFuel;
+  double oxiNeedPerTank = CalcOxidizerNeed()/TanksWithOxidizer;
+
+  for (i=0; i<SourceTanks.size(); i++) {
+    Tank = Propulsion->GetTank(SourceTanks[i]);
+    if ( ! Tank->GetSelected()) continue; // If this tank is not selected as a source, skip it.
+    switch(Tank->GetType()) {
+      case FGTank::ttFUEL:
+        Fshortage += Tank->Drain(2.0*fuelNeedPerTank - previousFuelNeedPerTank);
+        previousFuelNeedPerTank = fuelNeedPerTank;
+        break;
+      case FGTank::ttOXIDIZER:
+        Oshortage += Tank->Drain(2.0*oxiNeedPerTank - previousOxiNeedPerTank);
+        previousOxiNeedPerTank = oxiNeedPerTank;
+        break;
+    }
+  }
+
+  if (Fshortage < 0.00 || (haveOxTanks && Oshortage < 0.00)) Starved = true;
+  else Starved = false;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+double FGRocket::CalcFuelNeed(void)
+{
+  double dT = State->Getdt()*Propulsion->GetRate();
+
+  if (ThrustTable != 0L) {          // Thrust table given - infers solid fuel
+    FuelFlowRate = VacThrust/Isp;   // This calculates wdot (weight flow rate in lbs/sec)
   } else {
-    PctPower = Throttle / MaxThrottle;
-    //todo: remove Variance?
-    PC = maxPC*PctPower * (1.0 + Variance * ((double)rand()/(double)RAND_MAX - 0.5));
-    // The Cf (below) is CF from Eqn. 3-30, "Rocket Propulsion Elements", Fifth Edition,
-    // George P. Sutton. Note that the thruster function GetPowerRequired() might
-    // be better called GetResistance() or something; this function returns the
-    // nozzle exit pressure.
-    Cf = sqrt(kFactor*(1 - pow(Thruster->GetPowerRequired()/(PC), (SHR-1)/SHR)));
-    Flameout = false;
+    FuelFlowRate = SLFuelFlowMax*PctPower;
   }
 
-  return Thruster->Calculate(Cf*maxPC*PctPower*propEff);
+  FuelExpended = FuelFlowRate*dT; // For this time step ...
+  return FuelExpended;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+double FGRocket::CalcOxidizerNeed(void)
+{
+  double dT = State->Getdt()*Propulsion->GetRate();
+  OxidizerFlowRate = SLOxiFlowMax*PctPower;
+  OxidizerExpended = OxidizerFlowRate*dT;
+  return OxidizerExpended;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -148,7 +250,7 @@ string FGRocket::GetEngineLabels(string delimeter)
 {
   std::ostringstream buf;
 
-  buf << Name << " Chamber Pressure (engine " << EngineNumber << " in psf)" << delimeter
+  buf << Name << " Total Impulse (engine " << EngineNumber << " in psf)" << delimeter
       << Thruster->GetThrusterLabels(EngineNumber, delimeter);
 
   return buf.str();
@@ -160,11 +262,27 @@ string FGRocket::GetEngineValues(string delimeter)
 {
   std::ostringstream buf;
 
-  buf << PC << delimeter << Thruster->GetThrusterValues(EngineNumber, delimeter);
+  buf << It << delimeter << Thruster->GetThrusterValues(EngineNumber, delimeter);
 
   return buf.str();
 }
 
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+// This funciton should tie properties to rocket engine specific properties
+// that are not bound in the base class (FGEngine) code.
+//
+void FGRocket::bindmodel()
+{
+  char property_name[80];
+
+  snprintf(property_name, 80, "propulsion/engine[%u]/total-impulse", EngineNumber);
+  PropertyManager->Tie( property_name, this, &FGRocket::GetTotalImpulse);
+  snprintf(property_name, 80, "propulsion/engine[%u]/oxi-flow-rate-pps", EngineNumber);
+  PropertyManager->Tie( property_name, this, &FGRocket::GetOxiFlowRate);
+  snprintf(property_name, 80, "propulsion/engine[%u]/vacuum-thrust_lbs", EngineNumber);
+  PropertyManager->Tie( property_name, this, &FGRocket::GetVacThrust);
+}
+
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 //    The bitmasked value choices are as follows:
 //    unset: In this case (the default) JSBSim would only print
@@ -191,14 +309,12 @@ void FGRocket::Debug(int from)
   if (debug_lvl & 1) { // Standard console startup message output
     if (from == 0) { // Constructor
       cout << "      Engine Name: " << Name << endl;
-      cout << "      Specific Heat Ratio = " << SHR << endl;
-      cout << "      Maximum Chamber Pressure = " << maxPC << endl;
-      cout << "      Propulsive Efficiency = " << propEff << endl;
-      cout << "      MaxThrottle = " << MaxThrottle << endl;
-      cout << "      MinThrottle = " << MinThrottle << endl;
-      cout << "      FuelFlowMax = " << SLFuelFlowMax << endl;
-      cout << "      OxiFlowMax = " << SLOxiFlowMax << endl;
-      cout << "      Variance = " << Variance << endl;
+      cout << "      Vacuum Isp = " << Isp << endl;
+      cout << "      Maximum Throttle = " << MaxThrottle << endl;
+      cout << "      Minimum Throttle = " << MinThrottle << endl;
+      cout << "      Fuel Flow (max) = " << SLFuelFlowMax << endl;
+      cout << "      Oxidizer Flow (max) = " << SLOxiFlowMax << endl;
+      cout << "      Mixture ratio = " << SLOxiFlowMax/SLFuelFlowMax << endl;
     }
   }
   if (debug_lvl & 2 ) { // Instantiation/Destruction notification
