@@ -36,33 +36,35 @@ HISTORY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <FGFDMExec.h>
-#include <models/FGAuxiliary.h>
 #include "FGTank.h"
 
-#if !defined ( sgi ) || defined( __GNUC__ ) && (_COMPILER_VERSION < 740)
 using std::cerr;
 using std::endl;
 using std::cout;
-#endif
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGTank.cpp,v 1.1.2.1 2007-01-06 10:49:26 ehofman Exp $";
+static const char *IdSrc = "$Id$";
 static const char *IdHdr = ID_TANK;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-FGTank::FGTank(FGFDMExec* exec, Element* el)
+FGTank::FGTank(FGFDMExec* exec, Element* el, int tank_number)
+                  : TankNumber(tank_number)
 {
   string token;
   Element* element;
+  Element* element_Grain;
   Area = 1.0;
   Temperature = -9999.0;
+  Ixx = Iyy = Izz = 0.0;
   Auxiliary = exec->GetAuxiliary();
-  Radius = Capacity = Contents = 0.0;
+  Radius = Capacity = Contents = Standpipe = Length = InnerRadius = 0.0;
+  PropertyManager = exec->GetPropertyManager();
+  vXYZ.InitMatrix();
+  vXYZ_drain.InitMatrix();
 
   type = el->GetAttributeValue("type");
   if      (type == "FUEL")     Type = ttFUEL;
@@ -73,14 +75,23 @@ FGTank::FGTank(FGFDMExec* exec, Element* el)
   if (element)  vXYZ = element->FindElementTripletConvertTo("IN");
   else          cerr << "No location found for this tank." << endl;
 
+  vXYZ_drain = vXYZ; // Set initial drain location to initial tank CG
+
+  element = el->FindElement("drain_location");
+  if (element)  {
+    vXYZ_drain = element->FindElementTripletConvertTo("IN");
+  }
+
   if (el->FindElement("radius"))
     Radius = el->FindElementValueAsNumberConvertTo("radius", "IN");
   if (el->FindElement("capacity"))
     Capacity = el->FindElementValueAsNumberConvertTo("capacity", "LBS");
   if (el->FindElement("contents"))
-    Contents = el->FindElementValueAsNumberConvertTo("contents", "LBS");
+    InitialContents = Contents = el->FindElementValueAsNumberConvertTo("contents", "LBS");
   if (el->FindElement("temperature"))
-    Temperature = el->FindElementValueAsNumber("temperature");
+    InitialTemperature = Temperature = el->FindElementValueAsNumber("temperature");
+  if (el->FindElement("standpipe"))
+    InitialStandpipe = Standpipe = el->FindElementValueAsNumberConvertTo("standpipe", "LBS");
 
   Selected = true;
 
@@ -91,7 +102,50 @@ FGTank::FGTank(FGFDMExec* exec, Element* el)
     PctFull  = 0;
   }
 
-  if (Temperature != -9999.0)  Temperature = FahrenheitToCelsius(Temperature);
+  // Check whether this is a solid propellant "tank". Initialize it if true.
+
+  grainType = gtUNKNOWN; // This is the default
+  
+  element_Grain = el->FindElement("grain_config");
+  if (element_Grain) {
+
+    strGType = element_Grain->GetAttributeValue("type");
+    if (strGType == "CYLINDRICAL")     grainType = gtCYLINDRICAL;
+    else if (strGType == "ENDBURNING") grainType = gtENDBURNING;
+    else                               cerr << "Unknown propellant grain type specified" << endl;
+
+    if (element_Grain->FindElement("length"))
+      Length = element_Grain->FindElementValueAsNumberConvertTo("length", "IN");
+    if (element_Grain->FindElement("bore_diameter"))
+      InnerRadius = element_Grain->FindElementValueAsNumberConvertTo("bore_diameter", "IN")/2.0;
+
+    // Initialize solid propellant values for debug and runtime use.
+
+    switch (grainType) {
+      case gtCYLINDRICAL:
+        if (Radius <= InnerRadius) {
+          cerr << "The bore diameter should be smaller than the total grain diameter!" << endl;
+          exit(-1);
+        }
+        Volume = M_PI * Length * (Radius*Radius - InnerRadius*InnerRadius); // cubic inches
+        break;
+      case gtENDBURNING:
+        Volume = M_PI * Length * Radius * Radius; // cubic inches
+        break;
+      case gtUNKNOWN:
+        cerr << "Unknown grain type found in this rocket engine definition." << endl;
+        exit(-1);
+    }
+    Density = (Contents*lbtoslug)/Volume; // slugs/in^3
+
+  }
+
+  char property_name[80];
+  snprintf(property_name, 80, "propulsion/tank[%d]/contents-lbs", TankNumber);
+  PropertyManager->Tie( property_name, (FGTank*)this, &FGTank::GetContents,
+                                       &FGTank::SetContents );
+
+  if (Temperature != -9999.0)  InitialTemperature = Temperature = FahrenheitToCelsius(Temperature);
   Area = 40.0 * pow(Capacity/1975, 0.666666667);
 
   Debug(0);
@@ -106,19 +160,50 @@ FGTank::~FGTank()
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+void FGTank::ResetToIC(void)
+{
+  Temperature = InitialTemperature;
+  Standpipe = InitialStandpipe;
+  Contents = InitialContents;
+  PctFull = 100.0*Contents/Capacity;
+  Selected = true;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+const FGColumnVector3 FGTank::GetXYZ(void)
+{
+  return vXYZ_drain + (Contents/Capacity)*(vXYZ - vXYZ_drain);
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+const double FGTank::GetXYZ(int idx)
+{
+  return vXYZ_drain(idx) + (Contents/Capacity)*(vXYZ(idx)-vXYZ_drain(idx));
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 double FGTank::Drain(double used)
 {
-  double shortage = Contents - used;
+  double remaining = Contents - used;
+
+  if (remaining >= 0) { // Reduce contents by amount used.
 
-  if (shortage >= 0) {
     Contents -= used;
     PctFull = 100.0*Contents/Capacity;
-  } else {
+
+  } else { // This tank must be empty.
+
     Contents = 0.0;
     PctFull = 0.0;
     Selected = false;
   }
-  return shortage;
+
+  if (grainType != gtUNKNOWN) CalculateInertias();
+
+  return remaining;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -161,11 +246,40 @@ double FGTank::Calculate(double dt)
   double TempFlowFactor = 1.115;      // Watts/sqft/C
   double TAT = Auxiliary->GetTAT_C();
   double Tdiff = TAT - Temperature;
-  double dT = 0.0;                    // Temp change due to one surface
+  double dTemp = 0.0;                 // Temp change due to one surface
   if (fabs(Tdiff) > 0.1) {
-    dT = (TempFlowFactor * Area * Tdiff * dt) / (Contents * HeatCapacity);
+    dTemp = (TempFlowFactor * Area * Tdiff * dt) / (Contents * HeatCapacity);
   }
-  return Temperature += (dT + dT);    // For now, assume upper/lower the same
+  return Temperature += (dTemp + dTemp);    // For now, assume upper/lower the same
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+//  This function calculates the moments of inertia for a solid propellant
+//  grain - either an end burning cylindrical grain or a bored cylindrical
+//  grain.
+
+void FGTank::CalculateInertias(void)
+{
+  double Mass = Contents*lbtoslug;
+  double RadSumSqr;
+  double Rad2 = Radius*Radius;
+  Volume = (Contents*lbtoslug)/Density; // in^3
+
+  switch (grainType) {
+    case gtCYLINDRICAL:
+      InnerRadius = sqrt(Rad2 - Volume/(M_PI * Length));
+      RadSumSqr = (Rad2 + InnerRadius*InnerRadius)/144.0;
+      Ixx = 0.5*Mass*RadSumSqr;
+      Iyy = Mass*(3.0*RadSumSqr + Length*Length/144.0)/12.0;
+      break;
+    case gtENDBURNING:
+      Length = Volume/(M_PI*Rad2);
+      Ixx = 0.5*Mass*Rad2/144.0;
+      Iyy = Mass*(3.0*Rad2 + Length*Length)/(144.0*12.0);
+      break;
+  }
+  Izz  = Iyy;
+
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
