@@ -72,24 +72,19 @@
 
 #include <simgear/compiler.h>
 
-#if defined( __MWERKS__ )
-// -dw- optimizer chokes (big-time) trying to optimize humongous
-// loop/switch statements
-#pragma optimization_level 0
-#endif
 
 #include <cstdlib>
 #include <string>
-#include STL_IOSTREAM
+#include <iostream>
 
 #include "uiuc_menu_Cn.h"
 
-SG_USING_STD(cerr);
-SG_USING_STD(cout);
-SG_USING_STD(endl);
+using std::cerr;
+using std::cout;
+using std::endl;
 
 #ifndef _MSC_VER
-SG_USING_STD(exit);
+using std::exit;
 #endif
 
 void parse_Cn( const string& linetoken2, const string& linetoken3,
