@@ -13,12 +13,14 @@ public:
 		      ADVANCE, REHEAT, PROP,
 		      BRAKE, STEER, EXTEND, HEXTEND, LEXTEND,
 		      INCIDENCE, FLAP0, FLAP1, SLAT, SPOILER, VECTOR,
+              FLAP0EFFECTIVENESS, FLAP1EFFECTIVENESS,
                       BOOST, CASTERING, PROPPITCH, PROPFEATHER,
                       COLLECTIVE, CYCLICAIL, CYCLICELE, ROTORENGINEON,
                       TILTYAW, TILTPITCH, TILTROLL,
                       ROTORBRAKE, ROTORENGINEMAXRELTORQUE, ROTORRELTARGET,
                       ROTORBALANCE, REVERSE_THRUST, WASTEGATE,
-                      WINCHRELSPEED, HITCHOPEN, PLACEWINCH, FINDAITOW};
+              WINCHRELSPEED, HITCHOPEN, PLACEWINCH, FINDAITOW
+              };
 
     enum { OPT_SPLIT  = 0x01,
            OPT_INVERT = 0x02,
