@@ -1,6 +1,7 @@
 #ifndef _GROUND_HPP
 #define _GROUND_HPP
 
+class SGMaterial;
 namespace yasim {
 
 class Ground {
@@ -11,6 +12,10 @@ public:
     virtual void getGroundPlane(const double pos[3],
                                 double plane[4], float vel[3]);
 
+    virtual void getGroundPlane(const double pos[3],
+                              double plane[4], float vel[3],
+                              int *type, const SGMaterial **material);
+
     virtual bool caughtWire(const double pos[4][3]);
 
     virtual bool getWire(double end[2][3], float vel[2][3]);
