@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Id: gui_funcs.cxx,v 1.46.2.6 2007-07-03 15:35:57 mfranz Exp $
+ * $Id$
  **************************************************************************/
 
 
@@ -32,52 +32,18 @@
 
 #include <simgear/compiler.h>
 
-#ifdef SG_MATH_EXCEPTION_CLASH
-#  include <math.h>
-#endif
-
-#ifdef HAVE_WINDOWS_H
-#  include <windows.h>
-#endif
-
-#include SG_GL_H
-
-#if defined(FX) && defined(XMESA)
-#  include <GL/xmesa.h>
-#endif
-
-#include STL_FSTREAM
-#include STL_STRING
-
-#include <stdlib.h>
-#include <string.h>
-
-// for help call back
-#ifdef WIN32
-# include <shellapi.h>
-# ifdef __CYGWIN__
-#  include <sys/cygwin.h>
-# endif
-#endif
-
+#include <fstream>
+#include <string>
+#include <cstring>
 #include <sstream>
 
-#include <plib/ssg.h>
+#include <stdlib.h>
 
-#include <simgear/constants.h>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/screen/screen-dump.hxx>
 
-#include <Include/general.hxx>
-#include <Aircraft/aircraft.hxx>
-#include <Aircraft/controls.hxx>
-#include <Airports/simple.hxx>
 #include <Cockpit/panel.hxx>
-#include <FDM/flight.hxx>
-#include <Main/main.hxx>
-#include <Main/fg_init.hxx>
-#include <Main/fg_io.hxx>
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 #include <Main/renderer.hxx>
@@ -88,13 +54,12 @@
 #  include <simgear/screen/win32-printer.h>
 #  include <simgear/screen/GlBitmaps.h>
 #endif
-
+#ifdef __MINGW32__
+#include <shellapi.h>
+#endif
 #include "gui.h"
-#include "gui_local.hxx"
-#include "sgVec3Slider.hxx"
 
-SG_USING_STD(string);
-SG_USING_STD(cout);
+using std::string;
 
 
 #if defined( TR_HIRES_SNAP)
@@ -104,18 +69,7 @@ extern void fgUpdateHUD( GLfloat x_start, GLfloat y_start,
 #endif
 
 
-// TODO: remove after the last hardcoded dialog has died
-char *gui_msg_OK     = "OK";
-char *gui_msg_NO     = "NO";
-char *gui_msg_YES    = "YES";
-char *gui_msg_CANCEL = "CANCEL";
-char *gui_msg_RESET  = "RESET";
-
-
 const __fg_gui_fn_t __fg_gui_fn[] = {
-
-        // File
-        {"reInit", reInit},
 #ifdef TR_HIRES_SNAP
         {"dumpHiResSnapShot", fgHiResDumpWrapper},
 #endif
@@ -208,31 +162,22 @@ ____________________________________________________________________*/
 // on Sunday 3rd of December
 
 
-void helpCb (puObject *)
+void helpCb ()
 {
     string command;
 	
-#if defined(FX) && !defined(WIN32)
-#  if defined(XMESA_FX_FULLSCREEN) && defined(XMESA_FX_WINDOW)
-    if ( globals->get_fullscreen() ) {
-        globals->set_fullscreen(false);
-        XMesaSetFXmode( XMESA_FX_WINDOW );
-    }
-#  endif
-#endif
-	
     SGPath path( globals->get_fg_root() );
     path.append( "Docs/index.html" );
 	
 #if !defined(WIN32)
 
-    string help_app = fgGetString("/sim/startup/browser-app");
+    command = globals->get_browser();
+    string::size_type pos;
+    if ((pos = command.find("%u", 0)) != string::npos)
+        command.replace(pos, 2, path.str());
+    else
+        command += " " + path.str();
 
-    if ( system("xwininfo -name Netscape > /dev/null 2>&1") == 0 ) {
-        command = help_app + " -remote \"openURL(" + path.str() + ")\"";
-    } else {
-        command = help_app + " " + path.str();
-    }
     command += " &";
     system( command.c_str() );
 
@@ -262,7 +207,6 @@ void fgHiResDump()
 {
     FILE *f;
     string message;
-    bool show_pu_cursor = false;
     bool menu_status = fgGetBool("/sim/menubar/visibility");
     char *filename = new char [24];
     static int count = 1;
@@ -276,14 +220,11 @@ void fgHiResDump()
     }
 
     fgSetBool("/sim/menubar/visibility", false);
-    TurnCursorOff();
-    if ( !puCursorIsHidden() ) {
-	show_pu_cursor = true;
-	puHideCursor();
-    }
+    int mouse = fgGetMouseCursor();
+    fgSetMouseCursor(MOUSE_CURSOR_NONE);
 
     FGRenderer *renderer = globals->get_renderer();
-    renderer->init();
+//     renderer->init();
     renderer->resize( fgGetInt("/sim/startup/xsize"),
                       fgGetInt("/sim/startup/ysize") );
 
@@ -325,11 +266,12 @@ void fgHiResDump()
     trTileBuffer(tr, GL_RGB, GL_UNSIGNED_BYTE, tile);
     trImageSize(tr, imageWidth, imageHeight);
     trRowOrder(tr, TR_TOP_TO_BOTTOM);
-    sgFrustum *frustum = ssgGetFrustum();
-    trFrustum(tr,
-              frustum->getLeft(), frustum->getRight(),
-              frustum->getBot(),  frustum->getTop(), 
-              frustum->getNear(), frustum->getFar());
+    // OSGFIXME
+//     sgFrustum *frustum = ssgGetFrustum();
+//     trFrustum(tr,
+//               frustum->getLeft(), frustum->getRight(),
+//               frustum->getBot(),  frustum->getTop(), 
+//               frustum->getNear(), frustum->getFar());
 	
     /* Prepare ppm output file */
     while (count < 1000) {
@@ -380,13 +322,15 @@ void fgHiResDump()
         int curRow =  trGet(tr, TR_CURRENT_ROW);
 
         renderer->update( false );
-        if ( do_hud )
-            fgUpdateHUD( curColumn*hud_col_step,      curRow*hud_row_step,
-                         (curColumn+1)*hud_col_step, (curRow+1)*hud_row_step );
-        if (do_panel)
-            globals->get_current_panel()->update(
-                                   curColumn*panel_col_step, panel_col_step,
-                                   curRow*panel_row_step,    panel_row_step );
+        // OSGFIXME
+//         if ( do_hud )
+//             fgUpdateHUD( curColumn*hud_col_step,      curRow*hud_row_step,
+//                          (curColumn+1)*hud_col_step, (curRow+1)*hud_row_step );
+        // OSGFIXME
+//         if (do_panel)
+//             globals->get_current_panel()->update(
+//                                    curColumn*panel_col_step, panel_col_step,
+//                                    curRow*panel_row_step,    panel_row_step );
         more = trEndTile(tr);
 
         /* save tile into tile row buffer*/
@@ -444,11 +388,7 @@ void fgHiResDump()
 
     delete [] filename;
 
-    if ( show_pu_cursor ) {
-        puShowCursor();
-    }
-
-    TurnCursorOn();
+    fgSetMouseCursor(mouse);
     fgSetBool("/sim/menubar/visibility", menu_status);
 
     if ( !freeze ) {
@@ -472,7 +412,7 @@ GLubyte *hiResScreenCapture( int multiplier )
     float fov = oldfov / multiplier;
     FGViewer *v = globals->get_current_view();
     fgSetDouble("/sim/current-view/field-of-view", fov);
-    globals->get_renderer()->init();
+//     globals->get_renderer()->init();
     int cur_width = fgGetInt("/sim/startup/xsize");
     int cur_height = fgGetInt("/sim/startup/ysize");
     delete( b1 );
@@ -497,14 +437,9 @@ GLubyte *hiResScreenCapture( int multiplier )
 
 #if defined( WIN32 ) && !defined( __CYGWIN__) && !defined(__MINGW32__)
 // win32 print screen function
-void printScreen ( puObject *obj ) {
-    bool show_pu_cursor = false;
-    TurnCursorOff();
-    if ( !puCursorIsHidden() ) {
-	show_pu_cursor = true;
-	puHideCursor();
-    }
-    // BusyCursor( 0 );
+void printScreen () {
+    int mouse = fgGetMouseCursor();
+    fgSetMouseCursor(MOUSE_CURSOR_NONE);
 
     CGlPrinter p( CGlPrinter::PRINT_BITMAP );
     int cur_width = fgGetInt("/sim/startup/xsize");
@@ -512,30 +447,92 @@ void printScreen ( puObject *obj ) {
     p.Begin( "FlightGear", cur_width*3, cur_height*3 );
     p.End( hiResScreenCapture(3) );
 
-    // BusyCursor(1);
-    if ( show_pu_cursor ) {
-	puShowCursor();
-    }
-    TurnCursorOn();
+    fgSetMouseCursor(mouse);
 }
 #endif // #ifdef WIN32
 
 
-void fgDumpSnapShotWrapper ( puObject *obj ) {
+void fgDumpSnapShotWrapper () {
     fgDumpSnapShot();
 }
 
 
-void fgHiResDumpWrapper ( puObject *obj ) {
+void fgHiResDumpWrapper () {
     fgHiResDump();
 }
 
 
 // do a screen snap shot
-void fgDumpSnapShot () {
-    bool show_pu_cursor = false;
-    char *filename = new char [24];
+bool fgDumpSnapShot () {
+    static SGConstPropertyNode_ptr master_freeze = fgGetNode("/sim/freeze/master");
+
+    bool freeze = master_freeze->getBoolValue();
+    if ( !freeze ) {
+        fgSetBool("/sim/freeze/master", true);
+    }
+
+    int mouse = fgGetMouseCursor();
+    fgSetMouseCursor(MOUSE_CURSOR_NONE);
+
+    fgSetBool("/sim/signals/screenshot", true);
+
+    FGRenderer *renderer = globals->get_renderer();
+    renderer->resize( fgGetInt("/sim/startup/xsize"),
+                      fgGetInt("/sim/startup/ysize") );
+
+    // we need two render frames here to clear the menu and cursor
+    // ... not sure why but doing an extra fgRenderFrame() shouldn't
+    // hurt anything
+    renderer->update( true );
+    renderer->update( true );
+
+    string dir = fgGetString("/sim/paths/screenshot-dir");
+    if (dir.empty())
+        dir = fgGetString("/sim/fg-current");
+
+    SGPath path(dir + '/');
+    if (path.create_dir( 0755 )) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "Cannot create screenshot directory '"
+                << dir << "'. Trying home directory.");
+        dir = fgGetString("/sim/fg-home");
+    }
+
+    char filename[24];
     static int count = 1;
+    while (count < 1000) {
+        snprintf(filename, 24, "fgfs-screen-%03d.ppm", count++);
+
+        SGPath p(dir);
+        p.append(filename);
+        if (!p.exists()) {
+            path.set(p.str());
+            break;
+        }
+    }
+
+    int result = sg_glDumpWindow(path.c_str(),
+                                 fgGetInt("/sim/startup/xsize"),
+                                 fgGetInt("/sim/startup/ysize"));
+
+    fgSetString("/sim/paths/screenshot-last", path.c_str());
+    fgSetBool("/sim/signals/screenshot", false);
+
+    fgSetMouseCursor(mouse);
+
+    if ( !freeze ) {
+        fgSetBool("/sim/freeze/master", false);
+    }
+    return result != 0;
+}
+
+// do an entire scenegraph dump
+void fgDumpSceneGraph()
+{
+    char *filename = new char [24];
+    string message;
+    static int count = 1;
+
+    FGRenderer *renderer = globals->get_renderer();
 
     static const SGPropertyNode *master_freeze
 	= fgGetNode("/sim/freeze/master");
@@ -545,50 +542,77 @@ void fgDumpSnapShot () {
         fgSetBool("/sim/freeze/master", true);
     }
 
-    TurnCursorOff();
-    if ( !puCursorIsHidden() ) {
-	show_pu_cursor = true;
-	puHideCursor();
-    }
-    fgSetBool("/sim/signals/screenshot", true);
-
-    FGRenderer *renderer = globals->get_renderer();
-    renderer->init();
-    renderer->resize( fgGetInt("/sim/startup/xsize"),
-                        fgGetInt("/sim/startup/ysize") );
-
-    // we need two render frames here to clear the menu and cursor
-    // ... not sure why but doing an extra fgRenderFrame() shouldn't
-    // hurt anything
-    renderer->update( true );
-    renderer->update( true );
-
     while (count < 1000) {
         FILE *fp;
-        snprintf(filename, 24, "fgfs-screen-%03d.ppm", count++);
+        snprintf(filename, 24, "fgfs-graph-%03d.osg", count++);
         if ( (fp = fopen(filename, "r")) == NULL )
             break;
         fclose(fp);
     }
 
-    int result = sg_glDumpWindow(filename,
-                                 fgGetInt("/sim/startup/xsize"),
-                                 fgGetInt("/sim/startup/ysize"));
-    fgSetString("/sim/last-screenshot", result ? filename : "");
-
-    fgSetBool("/sim/signals/screenshot", false);
-    //mkDialog (message.c_str());
-
-    delete [] filename;
-
-    if ( show_pu_cursor ) {
-	puShowCursor();
+    if ( fgDumpSceneGraphToFile(filename)) {
+	message = "Entire scene graph saved to \"";
+	message += filename;
+	message += "\".";
+    } else {
+        message = "Failed to save to \"";
+	message += filename;
+	message += "\".";
     }
 
-    TurnCursorOn();
+    mkDialog (message.c_str());
+
+    delete [] filename;
 
     if ( !freeze ) {
         fgSetBool("/sim/freeze/master", false);
     }
 }
 
+    
+// do an terrain branch dump
+void fgDumpTerrainBranch()
+{
+    char *filename = new char [24];
+    string message;
+    static int count = 1;
+
+    FGRenderer *renderer = globals->get_renderer();
+
+    static const SGPropertyNode *master_freeze
+	= fgGetNode("/sim/freeze/master");
+
+    bool freeze = master_freeze->getBoolValue();
+    if ( !freeze ) {
+        fgSetBool("/sim/freeze/master", true);
+    }
+
+    while (count < 1000) {
+        FILE *fp;
+        snprintf(filename, 24, "fgfs-graph-%03d.osg", count++);
+        if ( (fp = fopen(filename, "r")) == NULL )
+            break;
+        fclose(fp);
+    }
+
+    if ( fgDumpTerrainBranchToFile(filename)) {
+	message = "Terrain graph saved to \"";
+	message += filename;
+	message += "\".";
+    } else {
+        message = "Failed to save to \"";
+	message += filename;
+	message += "\".";
+    }
+
+    mkDialog (message.c_str());
+
+    delete [] filename;
+
+    if ( !freeze ) {
+        fgSetBool("/sim/freeze/master", false);
+    }
+}
+
+    
+
