@@ -18,31 +18,29 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: input.cxx,v 1.87.2.10 2007-12-01 23:37:44 mfranz Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#ifdef HAVE_WINDOWS_H
-#  include <windows.h>
-#endif
-
 #include <simgear/compiler.h>
 
 #include <math.h>
 #include <ctype.h>
 #include <sstream>
 
-#include STL_FSTREAM
-#include STL_STRING
+#include <fstream>
+#include <string>
 #include <vector>
 
 #include <simgear/compiler.h>
 
 #include <simgear/constants.h>
 #include <simgear/debug/logstream.hxx>
+#include <simgear/math/SGMath.hxx>
 #include <simgear/props/props.hxx>
+#include <simgear/scene/util/SGSceneUserData.hxx>
 
 #include <Aircraft/aircraft.hxx>
 #include <Autopilot/xmlauto.hxx>
@@ -60,15 +58,13 @@
 
 #include <Scenery/scenery.hxx>
 #include <Main/renderer.hxx>
-#include <plib/ssg.h>
-#include <simgear/math/sg_geodesy.hxx>
 
-SG_USING_STD(ifstream);
-SG_USING_STD(ostringstream);
-SG_USING_STD(string);
-SG_USING_STD(vector);
+using std::ifstream;
+using std::ostringstream;
+using std::string;
+using std::vector;
 
-void mouseClickHandler(int button, int updown, int x, int y);
+void mouseClickHandler(int button, int updown, int x, int y, bool mainWindow, const osgGA::GUIEventAdapter*);
 void mouseMotionHandler(int x, int y);
 void keyHandler(int key, int keymod, int mousex, int mousey);
 
@@ -84,6 +80,12 @@ static FGInput * default_input = 0;
 // Local functions.
 ////////////////////////////////////////////////////////////////////////
 
+static int
+getModifiers ()
+{
+  return fgGetKeyModifiers() >> 1;
+}
+
 static bool
 getModShift ()
 {
@@ -114,88 +116,12 @@ getModSuper ()
   return bool(fgGetKeyModifiers() & KEYMOD_SUPER);
 }
 
-
-////////////////////////////////////////////////////////////////////////
-// Implementation of FGBinding.
-////////////////////////////////////////////////////////////////////////
-
-FGBinding::FGBinding ()
-  : _command(0),
-    _arg(new SGPropertyNode),
-    _setting(0)
+static bool
+getModHyper ()
 {
+  return bool(fgGetKeyModifiers() & KEYMOD_HYPER);
 }
 
-FGBinding::FGBinding (const SGPropertyNode * node)
-  : _command(0),
-    _arg(0),
-    _setting(0)
-{
-  read(node);
-}
-
-FGBinding::~FGBinding ()
-{
-  if (_arg && _arg->getParent())
-    _arg->getParent()->removeChild(_arg->getName(), _arg->getIndex(), false);
-}
-
-void
-FGBinding::read (const SGPropertyNode * node)
-{
-  const SGPropertyNode * conditionNode = node->getChild("condition");
-  if (conditionNode != 0)
-    setCondition(sgReadCondition(globals->get_props(), conditionNode));
-
-  _command_name = node->getStringValue("command", "");
-  if (_command_name.empty()) {
-    SG_LOG(SG_INPUT, SG_WARN, "No command supplied for binding.");
-    _command = 0;
-    return;
-  }
-
-  _arg = (SGPropertyNode *)node;
-  _setting = 0;
-}
-
-void
-FGBinding::fire () const
-{
-  if (test()) {
-    if (_command == 0)
-      _command = globals->get_commands()->getCommand(_command_name);
-    if (_command == 0) {
-      SG_LOG(SG_INPUT, SG_WARN, "No command attached to binding");
-    } else if (!(*_command)(_arg)) {
-      SG_LOG(SG_INPUT, SG_ALERT, "Failed to execute command "
-             << _command_name);
-    }
-  }
-}
-
-void
-FGBinding::fire (double offset, double max) const
-{
-  if (test()) {
-    _arg->setDoubleValue("offset", offset/max);
-    fire();
-  }
-}
-
-void
-FGBinding::fire (double setting) const
-{
-  if (test()) {
-                                // A value is automatically added to
-                                // the args
-    if (_setting == 0)          // save the setting node for efficiency
-      _setting = _arg->getChild("setting", 0, true);
-    _setting->setDoubleValue(setting);
-    fire();
-  }
-}
-
-
 
 ////////////////////////////////////////////////////////////////////////
 // Implementation of FGInput.
@@ -242,11 +168,13 @@ FGInput::postinit ()
 void
 FGInput::bind ()
 {
+  fgTie("/devices/status/keyboard", getModifiers);
   fgTie("/devices/status/keyboard/shift", getModShift);
   fgTie("/devices/status/keyboard/ctrl", getModCtrl);
   fgTie("/devices/status/keyboard/alt", getModAlt);
   fgTie("/devices/status/keyboard/meta", getModMeta);
   fgTie("/devices/status/keyboard/super", getModSuper);
+  fgTie("/devices/status/keyboard/hyper", getModHyper);
 
   _key_event->tie("key", SGRawValuePointer<int>(&_key_code));
   _key_event->tie("pressed", SGRawValuePointer<bool>(&_key_pressed));
@@ -256,16 +184,19 @@ FGInput::bind ()
   _key_event->tie("modifier/alt", SGRawValuePointer<bool>(&_key_alt));
   _key_event->tie("modifier/meta", SGRawValuePointer<bool>(&_key_meta));
   _key_event->tie("modifier/super", SGRawValuePointer<bool>(&_key_super));
+  _key_event->tie("modifier/hyper", SGRawValuePointer<bool>(&_key_hyper));
 }
 
 void
 FGInput::unbind ()
 {
+  fgUntie("/devices/status/keyboard");
   fgUntie("/devices/status/keyboard/shift");
   fgUntie("/devices/status/keyboard/ctrl");
   fgUntie("/devices/status/keyboard/alt");
   fgUntie("/devices/status/keyboard/meta");
   fgUntie("/devices/status/keyboard/super");
+  fgUntie("/devices/status/keyboard/hyper");
 
   _key_event->untie("key");
   _key_event->untie("pressed");
@@ -275,19 +206,15 @@ FGInput::unbind ()
   _key_event->untie("modifier/alt");
   _key_event->untie("modifier/meta");
   _key_event->untie("modifier/super");
+  _key_event->untie("modifier/hyper");
 }
 
 void 
 FGInput::update (double dt)
-{  
-  timingInfo.clear();
-  stamp("Input_begin");
+{
   _update_keyboard();
-  stamp("Input_keyboard");
   _update_joystick(dt);
-  stamp("Input_JS");
   _update_mouse(dt);
-  stamp("Input_Mouse");
 }
 
 void
@@ -327,19 +254,22 @@ FGInput::doKey (int k, int modifiers, int x, int y)
   }
 
   _key_code = k;
-  _key_modifiers = modifiers & ~KEYMOD_RELEASED;
+  _key_modifiers = modifiers >> 1;
   _key_pressed = !bool(modifiers & KEYMOD_RELEASED);
   _key_shift = bool(modifiers & KEYMOD_SHIFT);
   _key_ctrl = bool(modifiers & KEYMOD_CTRL);
   _key_alt = bool(modifiers & KEYMOD_ALT);
   _key_meta = bool(modifiers & KEYMOD_META);
   _key_super = bool(modifiers & KEYMOD_SUPER);
+  _key_hyper = bool(modifiers & KEYMOD_HYPER);
   _key_event->fireValueChanged();
-  if (!_key_code)
+  if (_key_code < 0)
     return;
 
   k = _key_code;
-  modifiers = _key_modifiers | (_key_pressed ? KEYMOD_NONE : KEYMOD_RELEASED);
+  modifiers = _key_modifiers << 1;
+  if (!_key_pressed)
+      modifiers |= KEYMOD_RELEASED;
   button &b = _key_bindings[k];
 
                                 // Key pressed.
@@ -368,7 +298,7 @@ FGInput::doKey (int k, int modifiers, int x, int y)
 }
 
 void
-FGInput::doMouseClick (int b, int updown, int x, int y)
+FGInput::doMouseClick (int b, int updown, int x, int y, bool mainWindow, const osgGA::GUIEventAdapter* ea)
 {
   int modifiers = fgGetKeyModifiers();
 
@@ -382,38 +312,50 @@ FGInput::doMouseClick (int b, int updown, int x, int y)
                                 // Pass on to PUI and the panel if
                                 // requested, and return if one of
                                 // them consumes the event.
+
+  if (updown != MOUSE_BUTTON_DOWN) {
+    // Execute the mouse up event in any case, may be we should
+    // stop processing here?
+    while (!_activePickCallbacks[b].empty()) {
+      _activePickCallbacks[b].front()->buttonReleased();
+      _activePickCallbacks[b].pop_front();
+    }
+  }
+
   if (mode.pass_through) {
-    if (puMouse(b, updown, x, y))
+    if (0 <= x && 0 <= y && puMouse(b, updown, x, y))
       return;
-    else if ((globals->get_current_panel() != 0) &&
+    else if (0 <= x && 0 <= y && (globals->get_current_panel() != 0) &&
              globals->get_current_panel()->getVisibility() &&
              globals->get_current_panel()->doMouseAction(b, updown, x, y))
       return;
-    else if (fgHandle3DPanelMouseEvent(b, updown, x, y))
+    else if (0 <= x && 0 <= y && fgHandle3DPanelMouseEvent(b, updown, x, y))
       return;
     else {
-      // pui and the panel didn't want the click event so compute a
-      // terrain intersection point corresponding to the mouse click
-      // and be happy.
-      FGScenery* scenery = globals->get_scenery();
-      SGVec3d start, dir, hit;
-      if (!b && updown == MOUSE_BUTTON_DOWN
-          && FGRenderer::getPickInfo(start, dir, x, y)
-          && scenery->get_cart_ground_intersection(start, dir, hit)) {
+      // pui didn't want the click event so compute a
+      // scenegraph intersection point corresponding to the mouse click
+      if (updown == MOUSE_BUTTON_DOWN) {
 
-        SGGeod geod = SGGeod::fromCart(hit);
-        SGPropertyNode *c = fgGetNode("/sim/input/click", true);
-        c->setDoubleValue("longitude-deg", geod.getLongitudeDeg());
-        c->setDoubleValue("latitude-deg", geod.getLatitudeDeg());
-        c->setDoubleValue("elevation-m", geod.getElevationM());
-        c->setDoubleValue("elevation-ft", geod.getElevationFt());
-
-        fgSetBool("/sim/signals/click", 1);
+        // Get the list of hit callbacks. Take the first callback that
+        // accepts the mouse button press and ignore the rest of them
+        // That is they get sorted by distance and by scenegraph depth.
+        // The nearest one is the first one and the deepest
+        // (the most specialized one in the scenegraph) is the first.
+        std::vector<SGSceneryPick> pickList;
+        if (FGRenderer::pick(pickList, ea)) {
+          std::vector<SGSceneryPick>::const_iterator i;
+          for (i = pickList.begin(); i != pickList.end(); ++i) {
+            if (i->callback->buttonPressed(b, i->info)) {
+              _activePickCallbacks[b].push_back(i->callback);
+              return;
+            }
+          }
+        }
       }
     }
   }
 
-                                // OK, PUI and the panel didn't want the click
+  // OK, PUI and the panel didn't want the click
   if (b >= MAX_MOUSE_BUTTONS) {
     SG_LOG(SG_INPUT, SG_ALERT, "Mouse button " << b
            << " where only " << MAX_MOUSE_BUTTONS << " expected");
@@ -879,47 +821,31 @@ FGInput::_update_keyboard ()
 void
 FGInput::_update_joystick (double dt)
 {
-  stamp("update_joystick_01");
-  int modifiers = KEYMOD_NONE;  // FIXME: any way to get the real ones?
-  int buttons;
-  // float js_val, diff;
   float axis_values[MAX_JOYSTICK_AXES];
+  int modifiers = fgGetKeyModifiers();
+  int buttons;
 
-  int i;
-  int j;
-
-  for ( i = 0; i < MAX_JOYSTICKS; i++) {
+  for (int i = 0; i < MAX_JOYSTICKS; i++) {
 
     jsJoystick * js = _joystick_bindings[i].js;
     if (js == 0 || js->notWorking())
       continue;
-    //stamp("update_joystick_02");
+
     js->read(&buttons, axis_values);
 
                                 // Fire bindings for the axes.
-    //stamp("update_joystick_03");
-    for ( j = 0; j < _joystick_bindings[i].naxes; j++) {
-      //stamp("update_joystick_04a");
+    for (int j = 0; j < _joystick_bindings[i].naxes; j++) {
       axis &a = _joystick_bindings[i].axes[j];
-      //stamp("update_joystick_04aX1");
+
                                 // Do nothing if the axis position
                                 // is unchanged; only a change in
                                 // position fires the bindings.
       if (fabs(axis_values[j] - a.last_value) > a.tolerance) {
-         //stamp("update_joystick_04aX2");
-//      SG_LOG(SG_INPUT, SG_DEBUG, "Axis " << j << " has moved");
         a.last_value = axis_values[j];
-	//stamp("update_joystick_04aX3");
-//      SG_LOG(SG_INPUT, SG_DEBUG, "There are "
-//             << a.bindings[modifiers].size() << " bindings");
-        for (unsigned int k = 0; k < a.bindings[modifiers].size(); k++) {
-          std::ostringstream desc;
-          stamp("update_joystick_fire_prep ");
-          a.bindings[modifiers][k]->fire(axis_values[j]);
-          stamp("update_joystick_fire_end");
-        }
-        stamp("update_joystick_04");
+        for (unsigned int k = 0; k < a.bindings[KEYMOD_NONE].size(); k++)
+          a.bindings[KEYMOD_NONE][k]->fire(axis_values[j]);
       }
+
                                 // do we have to emulate axis buttons?
       a.last_dt += dt;
       if(a.last_dt >= a.interval_sec) {
@@ -928,7 +854,7 @@ FGInput::_update_joystick (double dt)
                          modifiers,
                          axis_values[j] < a.low_threshold,
                          -1, -1);
-      
+
         if (a.high.bindings[modifiers].size())
           _update_button(_joystick_bindings[i].axes[j].high,
                          modifiers,
@@ -936,12 +862,10 @@ FGInput::_update_joystick (double dt)
                          -1, -1);
          a.last_dt -= a.interval_sec;
       }
-      //stamp("update_joystick_04");
     }
-    stamp("update_joystick_05");
 
                                 // Fire bindings for the buttons.
-    for (j = 0; j < _joystick_bindings[i].nbuttons; j++) {
+    for (int j = 0; j < _joystick_bindings[i].nbuttons; j++) {
       button &b = _joystick_bindings[i].buttons[j];
       b.last_dt += dt;
       if(b.last_dt >= b.interval_sec) {
@@ -952,9 +876,7 @@ FGInput::_update_joystick (double dt)
         b.last_dt -= b.interval_sec;
       }
     }
-    //stamp("update_joystick_06");
   }
-  stamp("update_joystick_07");
 }
 
 void
@@ -990,6 +912,16 @@ FGInput::_update_mouse ( double dt )
       m.save_x = m.x;
       m.save_y = m.y;
   }
+
+  // handle repeatable mouse press events
+  std::map<int, std::list<SGSharedPtr<SGPickCallback> > >::iterator mi;
+  for (mi = _activePickCallbacks.begin();
+       mi != _activePickCallbacks.end(); ++mi) {
+    std::list<SGSharedPtr<SGPickCallback> >::iterator li;
+    for (li = mi->second.begin(); li != mi->second.end(); ++li) {
+      (*li)->update(dt);
+    }
+  }
 }
 
 void
@@ -1000,8 +932,9 @@ FGInput::_update_button (button &b, int modifiers, bool pressed,
                                 // The press event may be repeated.
     if (!b.last_state || b.is_repeatable) {
       SG_LOG( SG_INPUT, SG_DEBUG, "Button has been pressed" );
-      for (unsigned int k = 0; k < b.bindings[modifiers].size(); k++)
+      for (unsigned int k = 0; k < b.bindings[modifiers].size(); k++) {
         b.bindings[modifiers][k]->fire(x, y);
+      }
     }
   } else {
                                 // The release event is never repeated.
@@ -1029,7 +962,7 @@ FGInput::_read_bindings (const SGPropertyNode * node,
 
     if (!strcmp(cmd, "nasal") && !_module.empty())
       bindings[i]->setStringValue("module", _module.c_str());
-    binding_list[modifiers].push_back(new FGBinding(bindings[i]));
+    binding_list[modifiers].push_back(new SGBinding(bindings[i], globals->get_props()));
   }
 
                                 // Read nested bindings for modifiers
@@ -1056,10 +989,14 @@ FGInput::_read_bindings (const SGPropertyNode * node,
   if (node->getChild("mod-super") != 0)
     _read_bindings(node->getChild("mod-super"), binding_list,
                    modifiers|KEYMOD_SUPER);
+
+  if (node->getChild("mod-hyper") != 0)
+    _read_bindings(node->getChild("mod-hyper"), binding_list,
+                   modifiers|KEYMOD_HYPER);
 }
 
 
-const vector<FGBinding *> &
+const FGInput::binding_list_t&
 FGInput::_find_key_bindings (unsigned int k, int modifiers)
 {
   unsigned char kc = (unsigned char)k;
@@ -1199,6 +1136,8 @@ FGInput::mouse_mode::~mouse_mode ()
 FGInput::mouse::mouse ()
   : x(-1),
     y(-1),
+    save_x(-1),
+    save_y(-1),
     nModes(1),
     current_mode(0),
     modes(0)
@@ -1224,10 +1163,10 @@ void keyHandler(int key, int keymod, int mousex, int mousey)
         default_input->doKey(key, keymod, mousex, mousey);
 }
 
-void mouseClickHandler(int button, int updown, int x, int y)
+void mouseClickHandler(int button, int updown, int x, int y, bool mainWindow, const osgGA::GUIEventAdapter* ea)
 {
     if(default_input)
-        default_input->doMouseClick(button, updown, x, y);
+      default_input->doMouseClick(button, updown, x, y, mainWindow, ea);
 }
 
 void mouseMotionHandler(int x, int y)
