@@ -23,7 +23,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: kln89.cxx,v 1.5 2006-02-21 01:19:03 mfranz Exp $
+// $Id$
 
 #include "kln89.hxx"
 #include "kln89_page.hxx"
@@ -43,11 +43,12 @@
 #include "kln89_symbols.hxx"
 #include <iostream>
 
-#include <ATC/ATCProjection.hxx>
+#include <ATCDCL/ATCProjection.hxx>
 #include <Main/fg_props.hxx>
 #include <simgear/math/point3d.hxx>
+#include "Airports/simple.hxx"
 
-SG_USING_STD(cout);
+using std::cout;
 
 KLN89::KLN89(RenderArea2D* instrument) 
 : DCLGPS(instrument) {
