@@ -47,7 +47,9 @@
 #include "tacan.hxx"
 #include "mk_viii.hxx"
 #include "mrg.hxx"
-
+#include "groundradar.hxx"
+#include "agradar.hxx"
+#include "rad_alt.hxx"
 
 FGInstrumentMgr::FGInstrumentMgr ()
 {
@@ -72,7 +74,7 @@ FGInstrumentMgr::FGInstrumentMgr ()
                         "Detected an internal inconsistency in the instrumentation\n"
                         "system specification file.  See earlier errors for details."));
             }
-        } catch (const sg_exception& exc) {
+        } catch (const sg_exception&) {
             SG_LOG( SG_ALL, SG_ALERT, "Failed to load instrumentation system model: "
                     << config.str() );
         }
@@ -168,7 +170,7 @@ bool FGInstrumentMgr::build ()
             set_subsystem( id, new VerticalSpeedIndicator( node ) );
 
         } else if ( name == "radar" ) {
-            set_subsystem( id, new wxRadarBg ( node ), 0.5 );
+            set_subsystem( id, new wxRadarBg ( node ), 1);
 
         } else if ( name == "inst-vertical-speed-indicator" ) {
             set_subsystem( id, new InstVerticalSpeedIndicator( node ) );
@@ -182,6 +184,15 @@ bool FGInstrumentMgr::build ()
         } else if ( name == "master-reference-gyro" ) {
             set_subsystem( id, new MasterReferenceGyro( node ) );
 
+        } else if ( name == "groundradar" ) {
+            set_subsystem( id, new GroundRadar( node ), 1 );
+
+        } else if ( name == "air-ground-radar" ) {
+            set_subsystem( id, new agRadar( node ),1);
+
+        } else if ( name == "radar-altimeter" ) {
+            set_subsystem( id, new radAlt( node ),1);
+
         } else {
             SG_LOG( SG_ALL, SG_ALERT, "Unknown top level section: "
                     << name );
