@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: kr_87.cxx,v 1.7 2006-06-15 19:16:22 frohlich Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
@@ -29,6 +29,8 @@
 
 #include <simgear/compiler.h>
 #include <simgear/math/sg_random.h>
+#include <simgear/math/sg_geodesy.hxx>
+#include <simgear/timing/sg_time.hxx>
 
 #include <Aircraft/aircraft.hxx>
 #include <Navaids/navlist.hxx>
@@ -36,7 +38,7 @@
 #include "kr_87.hxx"
 
 #include <string>
-SG_USING_STD(string);
+using std::string;
 
 static int play_count = 0;
 static time_t last_time = 0;
