@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: marker_beacon.cxx,v 1.11 2006-07-27 16:36:22 frohlich Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
@@ -36,7 +36,7 @@
 #include "marker_beacon.hxx"
 
 #include <string>
-SG_USING_STD(string);
+using std::string;
 
 
 // Constructor
@@ -156,14 +156,20 @@ FGMarkerBeacon::update(double dt)
 {
     need_update = false;
 
+    // On timeout, scan again, this needs to run every iteration no
+    // matter what the power or serviceable state.  If power is turned
+    // off or the unit becomes unserviceable while a beacon sound is
+    // playing, the search() routine still needs to be called so the
+    // sound effect can be properly disabled.
+
+    _time_before_search_sec -= dt;
+    if ( _time_before_search_sec < 0 ) {
+	search();
+    }
+
     if ( has_power() && serviceable->getBoolValue()
             && !sound_pause->getBoolValue()) {
 
-        // On timeout, scan again
-        _time_before_search_sec -= dt;
-        if ( _time_before_search_sec < 0 ) {
-            search();
-        }
         // marker beacon blinking
         bool light_on = ( outer_blink || middle_blink || inner_blink );
         SGTimeStamp current;
@@ -269,11 +275,11 @@ void FGMarkerBeacon::search()
     fgMkrBeacType beacon_type = NOBEACON;
     bool inrange = false;
     if ( b != NULL ) {
-        if ( b->get_type() == 7 ) {
+        if ( b->type() == FGPositioned::OM ) {
             beacon_type = OUTER;
-        } else if ( b->get_type() == 8 ) {
+        } else if ( b->type() == FGPositioned::MM ) {
             beacon_type = MIDDLE;
-        } else if ( b->get_type() == 9 ) {
+        } else if ( b->type() == FGPositioned::IM ) {
             beacon_type = INNER;
         }
         inrange = check_beacon_range( pos, b );
