@@ -2,6 +2,7 @@
 //
 // Written by Harald JOHNSEN, started May 2005.
 // With major amendments by Vivian MEAZZA May 2007
+// Ported to OSG by Tim Moore Jun 2007
 //
 //
 // Copyright (C) 2005  Harald JOHNSEN
@@ -26,15 +27,32 @@
 #  include "config.h"
 #endif
 
-#include <plib/sg.h>
+#include <osg/Array>
+#include <osg/Geometry>
+#include <osg/Matrixf>
+#include <osg/PrimitiveSet>
+#include <osg/StateSet>
+#include <osg/Version>
+#include <osgDB/ReaderWriter>
+#include <osgDB/WriteFile>
 
 #include <simgear/constants.h>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/environment/visual_enviro.hxx>
+#include <simgear/scene/model/model.hxx>
 #include <simgear/structure/exception.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/math/sg_geodesy.hxx>
 
+#include <sstream>
+#include <iomanip>
+using std::stringstream;
+using std::endl;
+using std::setprecision;
+using std::fixed;
+using std::setw;
+using std::setfill;
+
 #include <Main/fg_props.hxx>
 #include <Main/globals.hxx>
 #include <Cockpit/panel.hxx>
@@ -43,54 +61,70 @@
 #include <AIModel/AIManager.hxx>
 #include <AIModel/AIBallistic.hxx>
 
+#include <Include/general.hxx>
 #include "instrument_mgr.hxx"
 #include "od_gauge.hxx"
 #include "wxradar.hxx"
 
 
-typedef list <SGSharedPtr<FGAIBase> > radar_list_type;
+typedef list <osg::ref_ptr<FGAIBase> > radar_list_type;
 typedef radar_list_type::iterator radar_list_iterator;
 typedef radar_list_type::const_iterator radar_list_const_iterator;
 
 
 static const float UNIT = 1.0f / 8.0f;  // 8 symbols in a row/column in the texture
+static const char *DEFAULT_FONT = "typewriter.txf";
 
 
-wxRadarBg::wxRadarBg ( SGPropertyNode *node) :
+wxRadarBg::wxRadarBg(SGPropertyNode *node) :
     _name(node->getStringValue("name", "radar")),
     _num(node->getIntValue("number", 0)),
     _interval(node->getDoubleValue("update-interval-sec", 1.0)),
-    _time( 0.0 ),
-    _last_switchKnob( "off" ),
-    _sim_init_done ( false ),
-    _resultTexture( 0 ),
-    _wxEcho( 0 ),
-    _odg( 0 )
+    _time(0.0),
+    _sim_init_done(false),
+    _odg(0),
+    _last_switchKnob("off"),
+    _resultTexture(0),
+    _wxEcho(0)
 {
-    const char *tacan_source = node->getStringValue("tacan-source",
-            "/instrumentation/tacan");
+    string branch;
+    branch = "/instrumentation/" + _name;
+    _Instrument = fgGetNode(branch.c_str(), _num, true);
+
+    const char *tacan_source = node->getStringValue("tacan-source", "/instrumentation/tacan");
     _Tacan = fgGetNode(tacan_source, true);
+
+    _font_node = _Instrument->getNode("font", true);
+
+#define INITFONT(p, val, type) if (!_font_node->hasValue(p)) _font_node->set##type##Value(p, val)
+    INITFONT("name", DEFAULT_FONT, String);
+    INITFONT("size", 8, Float);
+    INITFONT("line-spacing", 0.25, Float);
+    INITFONT("color/red", 0, Float);
+    INITFONT("color/green", 0.8, Float);
+    INITFONT("color/blue", 0, Float);
+    INITFONT("color/alpha", 1, Float);
+#undef INITFONT
+
+    _font_node->addChangeListener(this, true);
 }
 
 
 wxRadarBg::~wxRadarBg ()
 {
+    _font_node->removeChangeListener(this);
 }
 
 
 void
 wxRadarBg::init ()
 {
-    string branch;
-    branch = "/instrumentation/" + _name;
-
-    _Instrument = fgGetNode(branch.c_str(), _num, true );
     _serviceable_node = _Instrument->getNode("serviceable", true);
 
     // texture name to use in 2D and 3D instruments
     _texture_path = _Instrument->getStringValue("radar-texture-path",
             "Aircraft/Instruments/Textures/od_wxradar.rgb");
-    _resultTexture = FGTextureManager::createTexture(_texture_path.c_str());
+    _resultTexture = FGTextureManager::createTexture(_texture_path.c_str(), false);
 
     SGPath tpath(globals->get_fg_root());
     string path = _Instrument->getStringValue("echo-texture-path",
@@ -98,11 +132,12 @@ wxRadarBg::init ()
     tpath.append(path);
 
     // no mipmap or else alpha will mix with pixels on the border of shapes, ruining the effect
-    _wxEcho = new ssgTexture( tpath.c_str(), false, false, false);
+    _wxEcho = SGLoadTexture2D(tpath, false, false);
+
 
     _Instrument->setFloatValue("trk", 0.0);
     _Instrument->setFloatValue("tilt", 0.0);
-    _Instrument->setStringValue("status","");
+    _Instrument->setStringValue("status", "");
     // those properties are used by a radar instrument of a MFD
     // input switch = OFF | TST | STBY | ON
     // input mode = WX | WXA | MAP
@@ -114,8 +149,9 @@ wxRadarBg::init ()
     // input range = n nm (20/40/80)
     // input display-mode = arc | rose | map | plan
 
-    FGInstrumentMgr *imgr = (FGInstrumentMgr *) globals->get_subsystem("instrumentation");
-    _odg = (FGODGauge *) imgr->get_subsystem("od_gauge");
+    FGInstrumentMgr *imgr = (FGInstrumentMgr *)globals->get_subsystem("instrumentation");
+    _odg = (FGODGauge *)imgr->get_subsystem("od_gauge");
+    _odg->setSize(512);
 
     _ai = (FGAIManager*)globals->get_subsystem("ai_model");
     _ai_enabled_node = fgGetNode("/sim/ai/enabled", true);
@@ -142,32 +178,114 @@ wxRadarBg::init ()
     _radar_weather_node     = n->getNode("WX", true);
     _radar_position_node    = n->getNode("pos", true);
     _radar_data_node        = n->getNode("data", true);
+    _radar_symbol_node      = n->getNode("symbol", true);
     _radar_centre_node      = n->getNode("centre", true);
+    _radar_rotate_node      = n->getNode("rotate", true);
 
     _radar_centre_node->setBoolValue(false);
-    if (_radar_coverage_node->getType() == SGPropertyNode::NONE)
+    if (!_radar_coverage_node->hasValue())
         _radar_coverage_node->setFloatValue(120);
-    if (_radar_ref_rng_node->getType() == SGPropertyNode::NONE)
+    if (!_radar_ref_rng_node->hasValue())
         _radar_ref_rng_node->setDoubleValue(35);
-    if (_radar_hdg_marker_node->getType() == SGPropertyNode::NONE)
+    if (!_radar_hdg_marker_node->hasValue())
         _radar_hdg_marker_node->setBoolValue(true);
 
     _x_offset = 0;
     _y_offset = 0;
+
+    // OSG geometry setup. The polygons for the radar returns will be
+    // stored in a single Geometry. The geometry will have several
+    // primitive sets so we can have different kinds of polys and
+    // choose a different overall color for each set.
+    _radarGeode = new osg::Geode;
+    osg::StateSet *stateSet = _radarGeode->getOrCreateStateSet();
+    stateSet->setTextureAttributeAndModes(0, _wxEcho.get());
+    _geom = new osg::Geometry;
+    _geom->setUseDisplayList(false);
+    // Initially allocate space for 128 quads
+    _vertices = new osg::Vec2Array;
+    _vertices->setDataVariance(osg::Object::DYNAMIC);
+    _vertices->reserve(128 * 4);
+    _geom->setVertexArray(_vertices);
+    _texCoords = new osg::Vec2Array;
+    _texCoords->setDataVariance(osg::Object::DYNAMIC);
+    _texCoords->reserve(128 * 4);
+    _geom->setTexCoordArray(0, _texCoords);
+    osg::Vec3Array *colors = new osg::Vec3Array;
+    colors->push_back(osg::Vec3(1.0f, 1.0f, 1.0f)); // color of echos
+    colors->push_back(osg::Vec3(1.0f, 0.0f, 0.0f)); // arc mask
+    colors->push_back(osg::Vec3(0.0f, 0.0f, 0.0f)); // rest of mask
+    _geom->setColorBinding(osg::Geometry::BIND_PER_PRIMITIVE_SET);
+    _geom->setColorArray(colors);
+    osg::PrimitiveSet *pset = new osg::DrawArrays(osg::PrimitiveSet::QUADS);
+    pset->setDataVariance(osg::Object::DYNAMIC);
+    _geom->addPrimitiveSet(pset);
+    pset = new osg::DrawArrays(osg::PrimitiveSet::QUADS);
+    pset->setDataVariance(osg::Object::DYNAMIC);
+    _geom->addPrimitiveSet(pset);
+    pset = new osg::DrawArrays(osg::PrimitiveSet::TRIANGLES);
+    pset->setDataVariance(osg::Object::DYNAMIC);
+    _geom->addPrimitiveSet(pset);
+    _geom->setInitialBound(osg::BoundingBox(osg::Vec3f(-256.0f, -256.0f, 0.0f),
+            osg::Vec3f(256.0f, 256.0f, 0.0f)));
+    _radarGeode->addDrawable(_geom);
+    _odg->allocRT();
+    // Texture in the 2D panel system
+    FGTextureManager::addTexture(_texture_path.c_str(), _odg->getTexture());
+
+    _textGeode = new osg::Geode;
+
+    osg::Camera *camera = _odg->getCamera();
+    camera->addChild(_radarGeode.get());
+    camera->addChild(_textGeode.get());
+}
+
+
+// Local coordinates for each echo
+const osg::Vec3f echoCoords[4] = {
+    osg::Vec3f(-.7f, -.7f, 0.0f), osg::Vec3f(.7f, -.7f, 0.0f),
+    osg::Vec3f(.7f, .7f, 0.0f), osg::Vec3f(-.7f, .7f, 0.0f)
+};
+
+
+const osg::Vec2f echoTexCoords[4] = {
+    osg::Vec2f(0.0f, 0.0f), osg::Vec2f(UNIT, 0.0f),
+    osg::Vec2f(UNIT, UNIT), osg::Vec2f(0.0f, UNIT)
+};
+
+
+// helper
+static void
+addQuad(osg::Vec2Array *vertices, osg::Vec2Array *texCoords,
+        const osg::Matrixf& transform, const osg::Vec2f& texBase)
+{
+    for (int i = 0; i < 4; i++) {
+        const osg::Vec3f coords = transform.preMult(echoCoords[i]);
+        texCoords->push_back(texBase + echoTexCoords[i]);
+        vertices->push_back(osg::Vec2f(coords.x(), coords.y()));
+    }
+}
+
+
+// Rotate by a heading value
+static inline
+osg::Matrixf wxRotate(float angle)
+{
+    return osg::Matrixf::rotate(angle, 0.0f, 0.0f, -1.0f);
 }
 
 
 void
 wxRadarBg::update (double delta_time_sec)
 {
-    if ( ! _sim_init_done ) {
-        if ( ! fgGetBool("sim/sceneryloaded", false) )
+    if (!_sim_init_done) {
+        if (!fgGetBool("sim/sceneryloaded", false))
             return;
 
         _sim_init_done = true;
     }
-    if ( !_odg || ! _serviceable_node->getBoolValue() ) {
-        _Instrument->setStringValue("status","");
+    if (!_odg || !_serviceable_node->getBoolValue()) {
+        _Instrument->setStringValue("status", "");
         return;
     }
     _time += delta_time_sec;
@@ -189,32 +307,22 @@ wxRadarBg::update (double delta_time_sec)
     }
 
     string switchKnob = _Instrument->getStringValue("switch", "on");
-    if ( _last_switchKnob != switchKnob ) {
+    if (_last_switchKnob != switchKnob) {
         // since 3D models don't share textures with the rest of the world
         // we must locate them and replace their handle by hand
         // only do that when the instrument is turned on
-        if ( _last_switchKnob == "off" )
-            _odg->set_texture(_texture_path.c_str(), _resultTexture->getHandle());
+        //if (_last_switchKnob == "off")
+        //_odg->set_texture(_texture_path.c_str(), _resultTexture->getHandle());
 
         _last_switchKnob = switchKnob;
     }
 
-    _odg->beginCapture(256);
-    _odg->Clear();
-
-    glMatrixMode(GL_MODELVIEW);
-    glLoadIdentity();
-    glPushMatrix();
-    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
-    glBindTexture(GL_TEXTURE_2D, 0);
-
-
-    if ( switchKnob == "off" ) {
-        _Instrument->setStringValue("status","");
-    } else if ( switchKnob == "stby" ) {
-        _Instrument->setStringValue("status","STBY");
-    } else if ( switchKnob == "tst" ) {
-        _Instrument->setStringValue("status","TST");
+    if (switchKnob == "off") {
+        _Instrument->setStringValue("status", "");
+    } else if (switchKnob == "stby") {
+        _Instrument->setStringValue("status", "STBY");
+    } else if (switchKnob == "tst") {
+        _Instrument->setStringValue("status", "TST");
         // find something interesting to do...
     } else {
         float r = _Instrument->getFloatValue("range", 40.0);
@@ -225,16 +333,17 @@ wxRadarBg::update (double delta_time_sec)
 
         _radar_ref_rng = _radar_ref_rng_node->getDoubleValue();
         _view_heading = get_heading() * SG_DEGREES_TO_RADIANS;
+        _centerTrans.makeTranslate(0.0f, 0.0f, 0.0f);
 
         _scale = 200.0 / _range_nm;
         _angle_offset = 0;
 
-        if ( _display_mode == ARC ) {
+        if (_display_mode == ARC) {
             _scale = 2*200.0f / _range_nm;
             _angle_offset = -_view_heading;
-            glTranslatef(0.0f, -200.0f, 0.0f);
+            _centerTrans.makeTranslate(0.0f, -200.0f, 0.0f);
 
-        } else if ( _display_mode == MAP ) {
+        } else if (_display_mode == MAP) {
             apply_map_offset();
 
             bool centre = _radar_centre_node->getBoolValue();
@@ -251,17 +360,29 @@ wxRadarBg::update (double delta_time_sec)
             //        << user_speed_north_fps * SG_FPS_TO_KT
             //        << " dt " << delta_time_sec);
 
-            glTranslatef(_x_offset, _y_offset, 0.0f);
+            _centerTrans.makeTranslate(_x_offset, _y_offset, 0.0f);
 
-        } else if ( _display_mode == PLAN ) {
-            // no sense I presume
+        } else if (_display_mode == PLAN) {
+            if (_radar_rotate_node->getBoolValue()) {
+                _angle_offset = -_view_heading;
+            }
         } else {
             // rose
         }
 
+        _vertices->clear();
+        _texCoords->clear();
+        _textGeode->removeDrawables(0, _textGeode->getNumDrawables());
+
+
         update_weather();
 
 
+        osg::DrawArrays *quadPSet
+                = static_cast<osg::DrawArrays*>(_geom->getPrimitiveSet(0));
+        quadPSet->set(osg::PrimitiveSet::QUADS, 0, _vertices->size());
+        quadPSet->dirty();
+
         // erase what is out of sight of antenna
         /*
             |\     /|
@@ -273,57 +394,72 @@ wxRadarBg::update (double delta_time_sec)
             ---------
         */
 
-        if ( _display_mode == ARC ) {
+        osg::DrawArrays *maskPSet
+                = static_cast<osg::DrawArrays*>(_geom->getPrimitiveSet(1));
+        osg::DrawArrays *trimaskPSet
+                = static_cast<osg::DrawArrays*>(_geom->getPrimitiveSet(2));
+
+        if (_display_mode == ARC) {
             float xOffset = 256.0f;
             float yOffset = 200.0f;
 
-            glDisable(GL_BLEND);
-            glColor4f(1.0f, 0.0f, 0.0f, 0.01f);
-            glBegin( GL_QUADS );
-            glTexCoord2f( 0.5f, 0.25f);
-            glVertex2f(-xOffset, 0.0 + yOffset);
-            glTexCoord2f( 1.0f, 0.25f);
-            glVertex2f(xOffset, 0.0 + yOffset);
-            glTexCoord2f( 1.0f, 0.5f);
-            glVertex2f(xOffset, 256.0 + yOffset);
-            glTexCoord2f( 0.5f, 0.5f);
-            glVertex2f(-xOffset, 256.0 + yOffset);
-            glEnd();
-
-            glColor4f(0.0f, 0.0f, 0.0f, 0.0f);
-            //glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
-            glDisable(GL_ALPHA_TEST);
-            glBindTexture(GL_TEXTURE_2D, 0);
+            int firstQuadVert = _vertices->size();
+            _texCoords->push_back(osg::Vec2f(0.5f, 0.25f));
+            _vertices->push_back(osg::Vec2f(-xOffset, 0.0 + yOffset));
+            _texCoords->push_back(osg::Vec2f(1.0f, 0.25f));
+            _vertices->push_back(osg::Vec2f(xOffset, 0.0 + yOffset));
+            _texCoords->push_back(osg::Vec2f(1.0f, 0.5f));
+            _vertices->push_back(osg::Vec2f(xOffset, 256.0 + yOffset));
+            _texCoords->push_back(osg::Vec2f(0.5f, 0.5f));
+            _vertices->push_back(osg::Vec2f(-xOffset, 256.0 + yOffset));
+            maskPSet->set(osg::PrimitiveSet::QUADS, firstQuadVert, 4);
 
+            // The triangles aren't supposed to be textured, but there's
+            // no need to set up a different Geometry, switch modes,
+            // etc. I happen to know that there's a white pixel in the
+            // texture at 1.0, 0.0 :)
             float centerY = tan(30 * SG_DEGREES_TO_RADIANS);
-            glBegin( GL_TRIANGLES );
-            glVertex2f(0.0, 0.0);
-            glVertex2f(-256.0, 0.0);
-            glVertex2f(-256.0, 256.0 * centerY);
+            _vertices->push_back(osg::Vec2f(0.0, 0.0));
+            _vertices->push_back(osg::Vec2f(-256.0, 0.0));
+            _vertices->push_back(osg::Vec2f(-256.0, 256.0 * centerY));
 
-            glVertex2f(0.0, 0.0);
-            glVertex2f(256.0, 0.0);
-            glVertex2f(256.0, 256.0 * centerY);
+            _vertices->push_back(osg::Vec2f(0.0, 0.0));
+            _vertices->push_back(osg::Vec2f(256.0, 0.0));
+            _vertices->push_back(osg::Vec2f(256.0, 256.0 * centerY));
 
-            glVertex2f(-256, 0.0);
-            glVertex2f(256.0, 0.0);
-            glVertex2f(-256.0, -256.0);
+            _vertices->push_back(osg::Vec2f(-256, 0.0));
+            _vertices->push_back(osg::Vec2f(256.0, 0.0));
+            _vertices->push_back(osg::Vec2f(-256.0, -256.0));
 
-            glVertex2f(256, 0.0);
-            glVertex2f(256.0, -256.0);
-            glVertex2f(-256.0, -256.0);
-            glEnd();
+            _vertices->push_back(osg::Vec2f(256, 0.0));
+            _vertices->push_back(osg::Vec2f(256.0, -256.0));
+            _vertices->push_back(osg::Vec2f(-256.0, -256.0));
 
-            glEnable(GL_BLEND);
-            glEnable(GL_ALPHA_TEST);
+            const osg::Vec2f whiteSpot(1.0f, 0.0f);
+            for (int i = 0; i < 3 * 4; i++)
+                _texCoords->push_back(whiteSpot);
+
+            trimaskPSet->set(osg::PrimitiveSet::TRIANGLES, firstQuadVert + 4, 3 * 4);
+
+        } else {
+            maskPSet->set(osg::PrimitiveSet::QUADS, 0, 0);
+            trimaskPSet->set(osg::PrimitiveSet::TRIANGLES, 0, 0);
         }
 
+        maskPSet->dirty();
+        trimaskPSet->dirty();
+
+        // draw without mask
+        _vertices->clear();
+        _texCoords->clear();
+
         update_aircraft();
         update_tacan();
         update_heading_marker();
+
+        quadPSet->set(osg::PrimitiveSet::QUADS, 0, _vertices->size());
+        quadPSet->dirty();
     }
-    glPopMatrix();
-    _odg->endCapture( _resultTexture->getHandle() );
 }
 
 
@@ -345,11 +481,9 @@ wxRadarBg::update_weather()
     // draw the cloud radar echo
     bool drawClouds = _radar_weather_node->getBoolValue();
     if (drawClouds) {
+
         // we do that in 3 passes, one for each color level
         // this is to 'merge' same colors together
-        glBindTexture(GL_TEXTURE_2D, _wxEcho->getHandle());
-        glColor3f(1.0f, 1.0f, 1.0f);
-        glBegin(GL_QUADS);
         for (int level = 0; level <= 2; level++) {
             float col = level * UNIT;
 
@@ -371,32 +505,20 @@ wxRadarBg::update_weather()
                 if (radius - size > 180)
                     continue;
 
-                // and apply a fov factor to simulate a greater scan angle
                 float angle = (iradarEcho->heading - _angle_offset) //* fovFactor
                         + 0.5 * SG_PI;
 
-                //angle *= fovFactor;
-                float x = cos(angle) * radius;
-                float y = sin(angle) * radius;
+                // Rotate echo into position, and rotate echo to have
+                // a constant orientation towards the
+                // airplane. Compass headings increase in clockwise
+                // direction, while graphics rotations follow
+                // right-hand (counter-clockwise) rule.
+                const osg::Vec2f texBase(col, (UNIT * (float) (4 + (cloudId & 3))));
 
-                // we will rotate the echo quads, this gives a better rendering
-                const float rot_x = cos (-_angle_offset);
-                const float rot_y = sin (-_angle_offset);
-
-                float size_x = rot_x * size;
-                float size_y = rot_y * size;
-
-                // use different shapes so the display is less boring
-                float row = UNIT * float(4 + (cloudId & 3));
-
-                glTexCoord2f(col, row);
-                glVertex2f(x - size_x, y - size_y);
-                glTexCoord2f(col+UNIT, row);
-                glVertex2f(x + size_y, y - size_x);
-                glTexCoord2f(col+UNIT, row+UNIT);
-                glVertex2f(x + size_x, y + size_y);
-                glTexCoord2f(col, row+UNIT);
-                glVertex2f(x - size_y, y + size_x);
+                osg::Matrixf m(osg::Matrixf::scale(size, size, 1.0f)
+                        * osg::Matrixf::translate(0.0f, radius, 0.0f)
+                        * wxRotate(angle) * _centerTrans);
+                addQuad(_vertices, _texCoords, m, texBase);
 
                 //SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: drawing clouds"
                 //        << " ID=" << cloudId
@@ -408,43 +530,62 @@ wxRadarBg::update_weather()
                 //        << " angle=" << angle * SG_RADIANS_TO_DEGREES);
             }
         }
-        glEnd(); // GL_QUADS
     }
 
     // draw lightning echos
     bool drawLightning = _Instrument->getBoolValue("lightning", true);
-    if ( drawLightning ) {
-        glBindTexture(GL_TEXTURE_2D, _wxEcho->getHandle());
-        glColor3f(1.0f, 1.0f, 1.0f);
-        glBegin(GL_QUADS);
-        float col = 3 * UNIT;
-        float row = 4 * UNIT;
+    if (drawLightning) {
+        const osg::Vec2f texBase(3 * UNIT, 4 * UNIT);
+
         for (iradarEcho = radarEcho->begin(); iradarEcho != end; ++iradarEcho) {
             if (!iradarEcho->lightning)
                 continue;
 
+            float size = UNIT * 0.5f;
             float radius = iradarEcho->dist * _scale;
-            float angle = iradarEcho->heading * SG_DEGREES_TO_RADIANS * fovFactor
+            float angle = iradarEcho->heading * SG_DEGREES_TO_RADIANS
                     - _angle_offset;
 
-            float x = cos( angle ) * radius;
-            float y = sin( angle ) * radius;
-            float size = UNIT * 0.5f;
-
-            glTexCoord2f(col, row);
-            glVertex2f(x - size, y - size);
-            glTexCoord2f(col + UNIT, row);
-            glVertex2f(x + size, y - size);
-            glTexCoord2f(col + UNIT, row + UNIT);
-            glVertex2f(x + size, y + size);
-            glTexCoord2f(col, row + UNIT);
-            glVertex2f(x - size, y + size);
+            osg::Matrixf m(osg::Matrixf::scale(size, size, 1.0f)
+                    * wxRotate(-angle)
+                    * osg::Matrixf::translate(0.0f, radius, 0.0f)
+                    * wxRotate(angle) * _centerTrans);
+            addQuad(_vertices, _texCoords, m, texBase);
         }
-        glEnd();
     }
 }
 
 
+void
+wxRadarBg::update_data(FGAIBase *ac, double radius, double bearing, bool selected)
+{
+    osgText::Text *callsign = new osgText::Text;
+    callsign->setFont(_font.get());
+    callsign->setFontResolution(12, 12);
+    callsign->setCharacterSize(_font_size);
+    callsign->setColor(selected ? osg::Vec4(1, 1, 1, 1) : _font_color);
+    osg::Matrixf m(wxRotate(-bearing)
+            * osg::Matrixf::translate(0.0f, radius, 0.0f)
+            * wxRotate(bearing) * _centerTrans);
+
+    osg::Vec3 pos = m.preMult(osg::Vec3(16, 16, 0));
+    // cast to int's, otherwise text comes out ugly
+    callsign->setPosition(osg::Vec3((int)pos.x(), (int)pos.y(), 0));
+    callsign->setAlignment(osgText::Text::LEFT_BOTTOM_BASE_LINE);
+    callsign->setLineSpacing(_font_spacing);
+
+    stringstream text;
+    text << ac->_getCallsign() << endl
+            << setprecision(0) << fixed
+            << setw(3) << setfill('0') << ac->_getHeading() << "\xB0 "
+            << setw(0) << ac->_getAltitude() << "ft" << endl
+            << ac->_getSpeed() << "kts";
+
+    callsign->setText(text.str());
+    _textGeode->addDrawable(callsign);
+}
+
+
 void
 wxRadarBg::update_aircraft()
 {
@@ -452,27 +593,39 @@ wxRadarBg::update_aircraft()
         return;
 
     bool draw_echoes = _radar_position_node->getBoolValue();
-    bool draw_symbols = _radar_data_node->getBoolValue();
-    if (!draw_echoes && !draw_symbols)
+    bool draw_symbols = _radar_symbol_node->getBoolValue();
+    bool draw_data = _radar_data_node->getBoolValue();
+    if (!draw_echoes && !draw_symbols && !draw_data)
         return;
 
     radar_list_type radar_list = _ai->get_ai_list();
-    SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: AI submodel list size" << radar_list.size());
+    //SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: AI submodel list size" << radar_list.size());
     if (radar_list.empty())
         return;
 
-    SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: Loading AI submodels ");
-    const double echo_radii[] = {0, 1, 1.5, 1.5, 0.001, 0.1, 1.5, 2, 1.5, 1.5};
+    //SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: Loading AI submodels ");
+    const double echo_radii[] = {0, 1, 1.5, 1.5, 0.001, 0.1, 1.5, 2, 1.5, 1.5, 1.5};
 
     double user_lat = _user_lat_node->getDoubleValue();
     double user_lon = _user_lon_node->getDoubleValue();
     double user_alt = _user_alt_node->getDoubleValue();
 
+    float limit = _radar_coverage_node->getFloatValue();
+    if (limit > 180)
+        limit = 180;
+    else if (limit < 0)
+        limit = 0;
+    limit *= SG_DEGREES_TO_RADIANS;
+
     radar_list_iterator it = radar_list.begin();
     radar_list_iterator end = radar_list.end();
+    FGAIBase *selected_ac = 0;
+    double selected_radius = 0;
+    double selected_bearing = 0;
+    int selected_id = fgGetInt("/instrumentation/radar/selected-id", -1);
 
     for (; it != end; ++it) {
-        FGAIBase *ac = *it;
+        FGAIBase *ac = (*it).get();
         int type       = ac->getType();
         double lat     = ac->_getLatitude();
         double lon     = ac->_getLongitude();
@@ -483,16 +636,16 @@ wxRadarBg::update_aircraft()
         calcRangeBearing(user_lat, user_lon, lat, lon, range, bearing);
 
         //SG_LOG(SG_GENERAL, SG_DEBUG,
-        //        "Radar: ID=" << ac->getID() << "(" << radar_list.size() << ")"
-        //        << " type=" << type
-        //        << " view_heading=" << _view_heading * SG_RADIANS_TO_DEGREES
-        //        << " alt=" << alt
-        //        << " heading=" << heading
-        //        << " range=" << range
-        //        << " bearing=" << bearing);
+        /*        "Radar: ID=" << ac->getID() << "(" << radar_list.size() << ")"
+                << " type=" << type
+                << " view_heading=" << _view_heading * SG_RADIANS_TO_DEGREES
+                << " alt=" << alt
+                << " heading=" << heading
+                << " range=" << range
+                << " bearing=" << bearing);*/
 
         bool isVisible = withinRadarHorizon(user_alt, alt, range);
-        SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: visible " << isVisible);
+        //SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: visible " << isVisible);
         if (!isVisible)
             continue;
 
@@ -505,12 +658,6 @@ wxRadarBg::update_aircraft()
         float radius = range * _scale;
         float angle = calcRelBearing(bearing, _view_heading);
 
-        float limit = _radar_coverage_node->getFloatValue();
-        if (limit > 180)
-            limit = 180;
-        else if (limit < 0)
-            limit = 0;
-        limit *= SG_DEGREES_TO_RADIANS;
         if (angle > limit || angle < -limit)
             continue;
 
@@ -519,72 +666,50 @@ wxRadarBg::update_aircraft()
 
         // pos mode
         if (draw_echoes) {
-            float col = 3 * UNIT;
-            float row = 3 * UNIT;
-
             float echo_radius = echo_radii[type] * 120;
             float size = echo_radius * UNIT;
-            float x = sin(bearing) * radius;
-            float y = cos(bearing) * radius;
-            float a_rot_x = sin(bearing + SGD_PI_4);
-            float a_rot_y = cos(bearing + SGD_PI_4);
-            float a_size_x = a_rot_x * size;
-            float a_size_y = a_rot_y * size;
 
-            glBindTexture(GL_TEXTURE_2D, _wxEcho->getHandle());
-            glColor3f(1.0f, 1.0f, 1.0f);
-            glBegin(GL_QUADS);
-
-            glTexCoord2f(col, row);
-            glVertex2f(x - a_size_x, y - a_size_y);
-            glTexCoord2f(col + UNIT, row);
-            glVertex2f(x + a_size_y, y - a_size_x);
-            glTexCoord2f(col + UNIT, row + UNIT);
-            glVertex2f(x + a_size_x, y + a_size_y);
-            glTexCoord2f(col, row + UNIT);
-            glVertex2f(x - a_size_y, y + a_size_x);
-
-            glEnd();
+            const osg::Vec2f texBase(3 * UNIT, 3 * UNIT);
+            osg::Matrixf m(osg::Matrixf::scale(size, size, 1.0f)
+                    * osg::Matrixf::translate(0.0f, radius, 0.0f)
+                    * wxRotate(bearing) * _centerTrans);
+            addQuad(_vertices, _texCoords, m, texBase);
 
             //SG_LOG(SG_GENERAL, SG_DEBUG, "Radar:    drawing AI"
-            //        << " x=" << x << " y=" << y
+                //<< " ID=" << ac->getID()
+                //<< " type=" << type
             //        << " radius=" << radius
             //        << " angle=" << angle * SG_RADIANS_TO_DEGREES);
         }
 
         // data mode
         if (draw_symbols) {
-            float col = 0 * UNIT;
-            float row = 3 * UNIT;
-
-            float size = 500 * UNIT;
-            float x = sin(bearing) * radius;
-            float y = cos(bearing) * radius;
-            float d_rot_x = sin(heading + SGD_PI_4);
-            float d_rot_y = cos(heading + SGD_PI_4);
-            float d_size_x = d_rot_x * size;
-            float d_size_y = d_rot_y * size;
-
-            glBindTexture(GL_TEXTURE_2D, _wxEcho->getHandle());
-            glColor3f(1.0f, 1.0f, 1.0f);
-            glBegin(GL_QUADS);
-
-            glTexCoord2f(col, row);
-            glVertex2f(x - d_size_x, y - d_size_y);
-            glTexCoord2f(col + UNIT, row);
-            glVertex2f(x + d_size_y, y - d_size_x);
-            glTexCoord2f(col + UNIT, row + UNIT);
-            glVertex2f(x + d_size_x, y + d_size_y);
-            glTexCoord2f(col, row + UNIT);
-            glVertex2f(x - d_size_y, y + d_size_x);
-
-            glEnd();
+            const osg::Vec2f texBase(0, 3 * UNIT);
+            float size = 600 * UNIT;
+            osg::Matrixf m(osg::Matrixf::scale(size, size, 1.0f)
+                    * wxRotate(heading - bearing)
+                    * osg::Matrixf::translate(0.0f, radius, 0.0f)
+                    * wxRotate(bearing) * _centerTrans);
+            addQuad(_vertices, _texCoords, m, texBase);
 
             //SG_LOG(SG_GENERAL, SG_DEBUG, "Radar:    drawing data"
             //        << " x=" << x <<" y="<< y
             //        << " bearing=" << angle * SG_RADIANS_TO_DEGREES
             //        << " radius=" << radius);
         }
+
+        if (draw_data) {
+            if (ac->getID() == selected_id) {
+                selected_ac = ac;
+                selected_radius = radius;
+                selected_bearing = bearing;
+            } else {
+                update_data(ac, radius, bearing, false);
+            }
+        }
+    }
+    if (selected_ac) {
+        update_data(selected_ac, selected_radius, selected_bearing, true);
     }
 }
 
@@ -599,31 +724,17 @@ wxRadarBg::update_tacan()
     if (mode != 1 || !inRange)
         return;
 
-    float col = 1 * UNIT;
-    float row = 3 * UNIT;
-
+    float size = 600 * UNIT;
     float radius = _tacan_distance_node->getFloatValue() * _scale;
     float angle = _tacan_bearing_node->getFloatValue() * SG_DEGREES_TO_RADIANS
             + _angle_offset;
 
-    float size = UNIT * 500;
-    float x = sin(angle) * radius;
-    float y = cos(angle) * radius;
-
-    glBindTexture(GL_TEXTURE_2D, _wxEcho->getHandle());
-    glColor3f(1.0f, 1.0f, 1.0f);
-    glBegin(GL_QUADS);
-
-    glTexCoord2f(col, row);
-    glVertex2f(x - size, y - size);
-    glTexCoord2f(col + UNIT, row);
-    glVertex2f(x + size, y - size);
-    glTexCoord2f(col + UNIT, row + UNIT);
-    glVertex2f(x + size, y + size);
-    glTexCoord2f(col, row + UNIT);
-    glVertex2f(x - size, y + size);
-
-    glEnd();
+    const osg::Vec2f texBase(1 * UNIT, 3 * UNIT);
+    osg::Matrixf m(osg::Matrixf::scale(size, size, 1.0f)
+            * wxRotate(-angle)
+            * osg::Matrixf::translate(0.0f, radius, 0.0f)
+            * wxRotate(angle) * _centerTrans);
+    addQuad(_vertices, _texCoords, m, texBase);
 
     //SG_LOG(SG_GENERAL, SG_DEBUG, "Radar:     drawing TACAN"
     //        << " dist=" << radius
@@ -640,33 +751,13 @@ wxRadarBg::update_heading_marker()
     if (!_radar_hdg_marker_node->getBoolValue())
         return;
 
-    float col = 2 * UNIT;
-    float row = 3 * UNIT;
-    float angle = _view_heading + _angle_offset;
+    const osg::Vec2f texBase(2 * UNIT, 3 * UNIT);
+    float size = 600 * UNIT;
+    osg::Matrixf m(osg::Matrixf::scale(size, size, 1.0f)
+            * wxRotate(_view_heading + _angle_offset));
 
-    float x = sin(angle);
-    float y = cos(angle);
-    float s_rot_x = sin(angle + SGD_PI_4);
-    float s_rot_y = cos(angle + SGD_PI_4);
-
-    float size = UNIT * 500;
-    float s_size_x = s_rot_x * size;
-    float s_size_y = s_rot_y * size;
-
-    glBindTexture(GL_TEXTURE_2D, _wxEcho->getHandle());
-    glColor3f(1.0f, 1.0f, 1.0f);
-    glBegin(GL_QUADS);
-
-    glTexCoord2f(col, row);
-    glVertex2f(x - s_size_x, y - s_size_y);
-    glTexCoord2f(col + UNIT, row);
-    glVertex2f(x + s_size_y, y - s_size_x);
-    glTexCoord2f(col + UNIT, row + UNIT);
-    glVertex2f(x + s_size_x, y + s_size_y);
-    glTexCoord2f(col, row + UNIT);
-    glVertex2f(x - s_size_y, y + s_size_x);
-
-    glEnd();
+    m *= _centerTrans;
+    addQuad(_vertices, _texCoords, m, texBase);
 
     //SG_LOG(SG_GENERAL, SG_DEBUG, "Radar:   drawing heading marker"
     //        << " x,y " << x <<","<< y
@@ -715,7 +806,7 @@ wxRadarBg::withinRadarHorizon(double user_alt, double alt, double range_nm)
         alt = 0;
 
     double radarhorizon = 1.23 * (sqrt(alt) + sqrt(user_alt));
-    SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: horizon " << radarhorizon);
+    //SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: horizon " << radarhorizon);
     return radarhorizon >= range_nm;
 }
 
@@ -737,21 +828,21 @@ wxRadarBg::inRadarRange(int type, double range_nm)
     //
     // TODO - make the maximum range adjustable at runtime
 
-    const double sigma[] = {0, 1, 100, 100, 0.001, 0.1, 100, 100, 1, 1};
+    const double sigma[] = {0, 1, 100, 100, 0.001, 0.1, 100, 100, 1, 1, 1};
     double constant = _radar_ref_rng;
 
     if (constant <= 0)
         constant = 35;
 
     double maxrange = constant * pow(sigma[type], 0.25);
-    SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: max range " << maxrange);
+    //SG_LOG(SG_GENERAL, SG_DEBUG, "Radar: max range " << maxrange);
     return maxrange >= range_nm;
 }
 
 
 void
 wxRadarBg::calcRangeBearing(double lat, double lon, double lat2, double lon2,
-        double &range, double &bearing ) const
+        double &range, double &bearing) const
 {
     // calculate the bearing and range of the second pos from the first
     double az2, distance;
@@ -774,3 +865,48 @@ wxRadarBg::calcRelBearing(float bearing, float heading)
     return angle;
 }
 
+
+void
+wxRadarBg::updateFont()
+{
+    float red = _font_node->getFloatValue("color/red");
+    float green = _font_node->getFloatValue("color/green");
+    float blue = _font_node->getFloatValue("color/blue");
+    float alpha = _font_node->getFloatValue("color/alpha");
+    _font_color.set(red, green, blue, alpha);
+
+    _font_size = _font_node->getFloatValue("size");
+    _font_spacing = _font_size * _font_node->getFloatValue("line-spacing");
+    string path = _font_node->getStringValue("name", DEFAULT_FONT);
+
+    SGPath tpath;
+    if (path[0] != '/') {
+        tpath = globals->get_fg_root();
+        tpath.append("Fonts");
+        tpath.append(path);
+    } else {
+        tpath = path;
+    }
+
+#if (FG_OSG_VERSION >= 21000)
+    osg::ref_ptr<osgDB::ReaderWriter::Options> fontOptions = new osgDB::ReaderWriter::Options("monochrome");
+    osg::ref_ptr<osgText::Font> font = osgText::readFontFile(tpath.c_str(), fontOptions.get());
+#else
+    osg::ref_ptr<osgText::Font> font = osgText::readFontFile(tpath.c_str());
+#endif
+
+    if (font != 0) {
+        _font = font;
+        _font->setMinFilterHint(osg::Texture::NEAREST);
+        _font->setMagFilterHint(osg::Texture::NEAREST);
+        _font->setGlyphImageMargin(0);
+        _font->setGlyphImageMarginRatio(0);
+    }
+}
+
+void
+wxRadarBg::valueChanged(SGPropertyNode*)
+{
+    updateFont();
+}
+
