@@ -8,8 +8,8 @@
 
 #include <simgear/compiler.h>
 
-#include STL_STRING
-#include STL_FSTREAM
+#include <string>
+#include <fstream>
 
 #include <simgear/sg_inlines.h>
 #include <simgear/debug/logstream.hxx>
@@ -18,6 +18,7 @@
 #include <simgear/structure/exception.hxx>
 #include <simgear/structure/commands.hxx>
 #include <simgear/props/props.hxx>
+#include <simgear/structure/event_mgr.hxx>
 
 #include <Cockpit/panel.hxx>
 #include <Cockpit/panel_io.hxx>
@@ -30,9 +31,6 @@
 #include <Aircraft/replay.hxx>
 #include <Scenery/tilemgr.hxx>
 #include <Scenery/scenery.hxx>
-#if defined(HAVE_PLIB_PSL)
-#  include <Scripting/scriptmgr.hxx>
-#endif
 #include <Scripting/NasalSys.hxx>
 #include <Sound/fg_fx.hxx>
 #include <Time/sunsolver.hxx>
@@ -48,9 +46,9 @@
 #include "viewmgr.hxx"
 #include "main.hxx"
 
-SG_USING_STD(string);
-SG_USING_STD(ifstream);
-SG_USING_STD(ofstream);
+using std::string;
+using std::ifstream;
+using std::ofstream;
 
 
 
@@ -166,20 +164,6 @@ do_null (const SGPropertyNode * arg)
   return true;
 }
 
-#if defined(HAVE_PLIB_PSL)
-/**
- * Built-in command: run a PSL script.
- *
- * script: the PSL script to execute
- */
-static bool
-do_script (const SGPropertyNode * arg)
-{
-    FGScriptMgr * mgr = (FGScriptMgr *)globals->get_subsystem("scripting");
-    return mgr->run(arg->getStringValue("script"));
-}
-#endif // HAVE_PLIB_PSL
-
 /**
  * Built-in command: run a Nasal script.
  */
@@ -228,6 +212,17 @@ do_exit (const SGPropertyNode * arg)
 }
 
 
+/**
+ * Reset FlightGear (Shift-Escape or Menu->File->Reset)
+ */
+static bool
+do_reset (const SGPropertyNode * arg)
+{
+    reInit();
+    return true;
+}
+
+
 /**
  * Built-in command: reinitialize one or more subsystems.
  *
@@ -326,16 +321,25 @@ do_resume (const SGPropertyNode * arg)
 static bool
 do_load (const SGPropertyNode * arg)
 {
-  const string &file = arg->getStringValue("file", "fgfs.sav");
-  ifstream input(file.c_str());
-  if (input.good() && fgLoadFlight(input)) {
-    input.close();
-    SG_LOG(SG_INPUT, SG_INFO, "Restored flight from " << file);
-    return true;
-  } else {
-    SG_LOG(SG_INPUT, SG_WARN, "Cannot load flight from " << file);
-    return false;
-  }
+    string file = arg->getStringValue("file", "fgfs.sav");
+    if (file.size() < 4 || file.substr(file.size() - 4) != ".sav")
+        file += ".sav";
+
+    if (!fgValidatePath(file.c_str(), false)) {
+        SG_LOG(SG_IO, SG_ALERT, "load: reading '" << file << "' denied "
+                "(unauthorized access)");
+        return false;
+    }
+
+    ifstream input(file.c_str());
+    if (input.good() && fgLoadFlight(input)) {
+        input.close();
+        SG_LOG(SG_INPUT, SG_INFO, "Restored flight from " << file);
+        return true;
+    } else {
+        SG_LOG(SG_INPUT, SG_WARN, "Cannot load flight from " << file);
+        return false;
+    }
 }
 
 
@@ -348,18 +352,27 @@ do_load (const SGPropertyNode * arg)
 static bool
 do_save (const SGPropertyNode * arg)
 {
-  const string &file = arg->getStringValue("file", "fgfs.sav");
-  bool write_all = arg->getBoolValue("write-all", false);
-  SG_LOG(SG_INPUT, SG_INFO, "Saving flight");
-  ofstream output(file.c_str());
-  if (output.good() && fgSaveFlight(output, write_all)) {
-    output.close();
-    SG_LOG(SG_INPUT, SG_INFO, "Saved flight to " << file);
-    return true;
-  } else {
-    SG_LOG(SG_INPUT, SG_ALERT, "Cannot save flight to " << file);
-    return false;
-  }
+    string file = arg->getStringValue("file", "fgfs.sav");
+    if (file.size() < 4 || file.substr(file.size() - 4) != ".sav")
+        file += ".sav";
+
+    if (!fgValidatePath(file.c_str(), false)) {
+        SG_LOG(SG_IO, SG_ALERT, "save: reading '" << file << "' denied "
+                "(unauthorized access)");
+        return false;
+    }
+
+    bool write_all = arg->getBoolValue("write-all", false);
+    SG_LOG(SG_INPUT, SG_INFO, "Saving flight");
+    ofstream output(file.c_str());
+    if (output.good() && fgSaveFlight(output, write_all)) {
+        output.close();
+        SG_LOG(SG_INPUT, SG_INFO, "Saved flight to " << file);
+        return true;
+    } else {
+        SG_LOG(SG_INPUT, SG_ALERT, "Cannot save flight to " << file);
+        return false;
+    }
 }
 
 
@@ -452,7 +465,6 @@ do_view_next( bool )
     globals->get_current_view()->setHeadingOffset_deg(0.0);
     globals->get_viewmgr()->next_view();
     fix_hud_visibility();
-    globals->get_tile_mgr()->refresh_view_timestamps();
 }
 
 static void
@@ -461,7 +473,6 @@ do_view_prev( bool )
     globals->get_current_view()->setHeadingOffset_deg(0.0);
     globals->get_viewmgr()->prev_view();
     fix_hud_visibility();
-    globals->get_tile_mgr()->refresh_view_timestamps();
 }
 
 /**
@@ -473,7 +484,6 @@ do_view_cycle (const SGPropertyNode * arg)
   globals->get_current_view()->setHeadingOffset_deg(0.0);
   globals->get_viewmgr()->next_view();
   fix_hud_visibility();
-  globals->get_tile_mgr()->refresh_view_timestamps();
   return true;
 }
 
@@ -483,10 +493,32 @@ do_view_cycle (const SGPropertyNode * arg)
 static bool
 do_screen_capture (const SGPropertyNode * arg)
 {
-  fgDumpSnapShot();
-  return true;
+  return fgDumpSnapShot();
 }
 
+static bool
+do_dump_scene_graph (const SGPropertyNode*)
+{
+    fgDumpSceneGraph();
+    return true;
+}
+
+static bool
+do_dump_terrain_branch (const SGPropertyNode*)
+{
+    fgDumpTerrainBranch();
+
+    double lon_deg = fgGetDouble("/position/longitude-deg");
+    double lat_deg = fgGetDouble("/position/latitude-deg");
+    SGGeod geodPos = SGGeod::fromDegFt(lon_deg, lat_deg, 0.0);
+    SGVec3d zero = SGVec3d::fromGeod(geodPos);
+
+    SG_LOG(SG_INPUT, SG_INFO, "Model parameters:");
+    SG_LOG(SG_INPUT, SG_INFO, "Center: " << zero.x() << ", " << zero.y() << ", " << zero.z() );
+    SG_LOG(SG_INPUT, SG_INFO, "Rotation: " << lat_deg << ", " << lon_deg );
+
+    return true;
+}
 
 /**
  * Built-in command: hires capture screen.
@@ -1147,6 +1179,59 @@ do_gui_redraw (const SGPropertyNode * arg)
     return true;
 }
 
+
+/**
+ * Adds model to the scenery. The path to the added branch (/models/model[*])
+ * is returned in property "property".
+ */
+static bool
+do_add_model (const SGPropertyNode * arg)
+{
+    SGPropertyNode * model = fgGetNode("models", true);
+    for (int i = 0;; i++) {
+        if (i < 0)
+            return false;
+        if (!model->getChild("model", i, false)) {
+            model = model->getChild("model", i, true);
+            break;
+        }
+    }
+    copyProperties(arg, model);
+    if (model->hasValue("elevation-m"))
+        model->setDoubleValue("elevation-ft", model->getDoubleValue("elevation-m")
+                * SG_METER_TO_FEET);
+    model->getNode("load", true);
+    model->removeChildren("load");
+    const_cast<SGPropertyNode *>(arg)->setStringValue("property", model->getPath());
+    return true;
+}
+
+
+/**
+ * Set mouse cursor coordinates and cursor shape.
+ */
+static bool
+do_set_cursor (const SGPropertyNode * arg)
+{
+    if (arg->hasValue("x") || arg->hasValue("y")) {
+        SGPropertyNode *mx = fgGetNode("/devices/status/mice/mouse/x", true);
+        SGPropertyNode *my = fgGetNode("/devices/status/mice/mouse/y", true);
+        int x = arg->getIntValue("x", mx->getIntValue());
+        int y = arg->getIntValue("y", my->getIntValue());
+        fgWarpMouse(x, y);
+        mx->setIntValue(x);
+        my->setIntValue(y);
+    }
+
+    SGPropertyNode *cursor = const_cast<SGPropertyNode *>(arg)->getNode("cursor", true);
+    if (cursor->getType() != SGPropertyNode::NONE)
+        fgSetMouseCursor(cursor->getIntValue());
+
+    cursor->setIntValue(fgGetMouseCursor());
+    return true;
+}
+
+
 /**
  * Built-in command: play an audio message (i.e. a wav file) This is
  * fire and forget.  Call this once per message and it will get dumped
@@ -1237,6 +1322,7 @@ do_replay (const SGPropertyNode * arg)
     return true;
 }
 
+
 static bool
 do_decrease_visibility (const SGPropertyNode * arg)
 {
@@ -1296,6 +1382,12 @@ do_load_xml_to_proptree(const SGPropertyNode * arg)
     if (file.extension() != "xml")
         file.concat(".xml");
 
+    if (!fgValidatePath(file.c_str(), false)) {
+        SG_LOG(SG_IO, SG_ALERT, "loadxml: reading '" << file.str() << "' denied "
+                "(unauthorized access)");
+        return false;
+    }
+
     SGPropertyNode *targetnode;
     if (arg->hasValue("targetnode"))
         targetnode = fgGetNode(arg->getStringValue("targetnode"), true);
@@ -1309,7 +1401,7 @@ do_load_xml_to_proptree(const SGPropertyNode * arg)
         return false;
     }
 
-    return  true;
+    return true;
 }
 
 
@@ -1336,6 +1428,12 @@ do_save_xml_from_proptree(const SGPropertyNode * arg)
     if (file.extension() != "xml")
         file.concat(".xml");
 
+    if (!fgValidatePath(file.c_str(), true)) {
+        SG_LOG(SG_IO, SG_ALERT, "savexml: writing to '" << file.str() << "' denied "
+                "(unauthorized access)");
+        return false;
+    }
+
     SGPropertyNode *sourcenode;
     if (arg->hasValue("sourcenode"))
         sourcenode = fgGetNode(arg->getStringValue("sourcenode"), true);
@@ -1414,11 +1512,9 @@ static struct {
   SGCommandMgr::command_t command;
 } built_ins [] = {
     { "null", do_null },
-#if defined(HAVE_PLIB_PSL)
-    { "script", do_script },
-#endif // HAVE_PLIB_PSL
     { "nasal", do_nasal },
     { "exit", do_exit },
+    { "reset", do_reset },
     { "reinit", do_reinit },
     { "suspend", do_reinit },
     { "resume", do_reinit },
@@ -1451,6 +1547,8 @@ static struct {
     { "dialog-update", do_dialog_update },
     { "dialog-apply", do_dialog_apply },
     { "gui-redraw", do_gui_redraw },
+    { "add-model", do_add_model },
+    { "set-cursor", do_set_cursor },
     { "play-audio-sample", do_play_audio_sample },
     { "presets-commit", do_presets_commit },
     { "log-level", do_log_level },
@@ -1463,6 +1561,8 @@ static struct {
     { "savexml", do_save_xml_from_proptree },
     { "press-cockpit-button", do_press_cockpit_button },
     { "release-cockpit-button", do_release_cockpit_button },
+    { "dump-scenegraph", do_dump_scene_graph },
+    { "dump-terrainbranch", do_dump_terrain_branch },
     { 0, 0 }			// zero-terminated
 };
 
