@@ -18,35 +18,28 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: fg_props.cxx,v 1.28.2.4 2007-11-24 08:37:46 mfranz Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
 #include <simgear/compiler.h>
-
 #include <simgear/structure/exception.hxx>
+#include <simgear/props/props_io.hxx>
+
 #include <simgear/magvar/magvar.hxx>
 #include <simgear/timing/sg_time.hxx>
 #include <simgear/misc/sg_path.hxx>
-#include <simgear/scene/material/matlib.hxx>
 #include <simgear/sound/soundmgr_openal.hxx>
 
-#include STL_IOSTREAM
-
-#include <ATC/ATCdisplay.hxx>
 #include <Aircraft/aircraft.hxx>
-#include <Time/tmp.hxx>
-#include <Environment/environment.hxx>
-
+#include <FDM/flight.hxx>
 #include <GUI/gui.h>
 
 #include "globals.hxx"
 #include "fg_props.hxx"
 
-SG_USING_STD(istream);
-SG_USING_STD(ostream);
 
 static bool winding_ccw = true; // FIXME: temporary
 
@@ -54,7 +47,7 @@ static bool fdm_data_logging = false; // FIXME: temporary
 
 static bool frozen = false;	// FIXME: temporary
 
-
+using std::string;
 
 ////////////////////////////////////////////////////////////////////////
 // Default property bindings (not yet handled by any module).
@@ -258,7 +251,7 @@ getDateString ()
 {
   static char buf[64];		// FIXME
   struct tm * t = globals->get_time_params()->getGmt();
-  snprintf(buf, 64, "%.4d-%.2d-%.2dT%.2d:%.2d:%.2d",
+  sprintf(buf, "%.4d-%.2d-%.2dT%.2d:%.2d:%.2d",
 	  t->tm_year + 1900, t->tm_mon + 1, t->tm_mday,
 	  t->tm_hour, t->tm_min, t->tm_sec);
   return buf;
@@ -396,27 +389,6 @@ setWindingCCW (bool state)
     glFrontFace ( GL_CW );
 }
 
-static bool
-getFullScreen ()
-{
-#if defined(FX) && !defined(WIN32)
-  return globals->get_fullscreen();
-#else
-  return false;
-#endif
-}
-
-static void
-setFullScreen (bool state)
-{
-#if defined(FX) && !defined(WIN32)
-  globals->set_fullscreen(state);
-#  if defined(XMESA_FX_FULLSCREEN) && defined(XMESA_FX_WINDOW)
-  XMesaSetFXmode( state ? XMESA_FX_FULLSCREEN : XMESA_FX_WINDOW );
-#  endif
-#endif
-}
-
 static bool
 getFDMDataLogging ()
 {
@@ -545,7 +517,6 @@ FGProperties::bind ()
 
 				// Misc. Temporary junk.
   fgTie("/sim/temp/winding-ccw", getWindingCCW, setWindingCCW, false);
-  fgTie("/sim/temp/full-screen", getFullScreen, setFullScreen);
   fgTie("/sim/temp/fdm-data-logging", getFDMDataLogging, setFDMDataLogging);
 }
 
@@ -613,7 +584,7 @@ FGProperties::update (double dt)
  * Save the current state of the simulator to a stream.
  */
 bool
-fgSaveFlight (ostream &output, bool write_all)
+fgSaveFlight (std::ostream &output, bool write_all)
 {
 
   fgSetBool("/sim/presets/onground", false);
@@ -637,7 +608,7 @@ fgSaveFlight (ostream &output, bool write_all)
  * Restore the current state of the simulator from a stream.
  */
 bool
-fgLoadFlight (istream &input)
+fgLoadFlight (std::istream &input)
 {
   SGPropertyNode props;
   try {
