@@ -18,13 +18,12 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: globals.cxx,v 1.44.2.3 2007-12-10 19:17:56 mfranz Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#include <simgear/scene/model/modellib.hxx>
 #include <simgear/sound/soundmgr_openal.hxx>
 #include <simgear/structure/commands.hxx>
 #include <simgear/misc/sg_path.hxx>
@@ -32,12 +31,13 @@
 #include <simgear/ephemeris/ephemeris.hxx>
 #include <simgear/magvar/magvar.hxx>
 #include <simgear/scene/material/matlib.hxx>
+#include <simgear/structure/subsystem_mgr.hxx>
+#include <simgear/structure/event_mgr.hxx>
 
 #include <Aircraft/controls.hxx>
 #include <Airports/runways.hxx>
-#include <ATC/AIMgr.hxx>
-#include <ATC/ATCmgr.hxx>
-#include <ATC/ATCdisplay.hxx>
+#include <ATCDCL/AIMgr.hxx>
+#include <ATCDCL/ATCmgr.hxx>
 #include <Autopilot/route_mgr.hxx>
 #include <Cockpit/panel.hxx>
 #include <GUI/new_gui.hxx>
@@ -47,6 +47,9 @@
 #include <Navaids/awynet.hxx>
 #include <Scenery/scenery.hxx>
 #include <Scenery/tilemgr.hxx>
+#include <Airports/simple.hxx>
+#include <Navaids/navlist.hxx>
+#include <Navaids/fixlist.hxx>
 
 #include "globals.hxx"
 #include "renderer.hxx"
@@ -71,9 +74,6 @@ FGGlobals::FGGlobals() :
     event_mgr( new SGEventMgr ),
     sim_time_sec( 0.0 ),
     fg_root( "" ),
-#if defined(FX) && defined(XMESA)
-    fullscreen( true ),
-#endif
     warp( 0 ),
     warp_delta( 0 ),
     time_params( NULL ),
@@ -84,17 +84,14 @@ FGGlobals::FGGlobals() :
     current_panel( NULL ),
     soundmgr( NULL ),
     airports( NULL ),
-    runways( NULL ),
     ATC_mgr( NULL ),
-    ATC_display( NULL ),
     AI_mgr( NULL ),
     controls( NULL ),
     viewmgr( NULL ),
     props( new SGPropertyNode ),
     initial_state( NULL ),
     locale( NULL ),
-    commands( new SGCommandMgr ),
-    model_lib( NULL ),
+    commands( SGCommandMgr::instance() ),
     acmodel( NULL ),
     model_mgr( NULL ),
     channel_options_list( NULL ),
@@ -122,7 +119,7 @@ FGGlobals::FGGlobals() :
 FGGlobals::~FGGlobals() 
 {
     delete renderer;
-    // The AIModels manager performs a number of actions upon
+// The AIModels manager performs a number of actions upon
     // Shutdown that implicitly assume that other subsystems
     // are still operational (Due to the dynamic allocation and
     // deallocation of AIModel objects. To ensure we can safely
@@ -139,22 +136,19 @@ FGGlobals::~FGGlobals()
     delete current_panel;
     delete soundmgr;
     delete airports;
-    delete runways;
+
     delete ATC_mgr;
-    delete ATC_display;
     delete AI_mgr;
     delete controls;
     delete viewmgr;
-    delete initial_state;
-    //delete locale; Don't delete locale
-    delete commands;
-    delete model_lib;
+
+//     delete commands;
     delete acmodel;
     delete model_mgr;
     delete channel_options_list;
     delete initial_waypoints;
+    delete tile_mgr;
     delete scenery;
-    //delete tile_mgr; // Don't delete tile manager yet, because loader thread problems
     delete io;
     delete fontcache;
 
@@ -169,25 +163,29 @@ FGGlobals::~FGGlobals()
     delete fixlist;
     delete airwaynet;
     delete multiplayer_mgr;
-
-    delete props;
-
 }
 
 
 // set the fg_root path
 void FGGlobals::set_fg_root (const string &root) {
     fg_root = root;
-    
+
     // append /data to root if it exists
     SGPath tmp( fg_root );
     tmp.append( "data" );
     tmp.append( "version" );
     if ( ulFileExists( tmp.c_str() ) ) {
         fg_root += "/data";
+        SG_LOG(SG_GENERAL, SG_WARN, "Warning: changing bad FG_ROOT/--fg-root to '"
+                << fg_root << '\'');
     }
 
-    fgSetString("/sim/fg-root", fg_root.c_str());   
+    // remove /sim/fg-root before writing to prevent hijacking
+    SGPropertyNode *n = fgGetNode("/sim", true);
+    n->removeChild("fg-root", 0, false);
+    n = n->getChild("fg-root", 0, true);
+    n->setStringValue(fg_root.c_str());
+    n->setAttribute(SGPropertyNode::WRITE, false);
 }
 
 void FGGlobals::set_fg_scenery (const string &scenery) {
@@ -274,7 +272,6 @@ FGGlobals::get_event_mgr () const
 void
 FGGlobals::saveInitialState ()
 {
-  delete initial_state;
   initial_state = new SGPropertyNode();
 
   if (!copyProperties(props, initial_state))
