@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: main.cxx,v 1.227.2.15 2007-12-11 21:32:51 durk Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
@@ -32,20 +32,13 @@
 #  include <signal.h>
 #endif
 
-#ifdef SG_MATH_EXCEPTION_CLASH
-#  include <math.h>
-#endif
-
-#ifdef HAVE_WINDOWS_H
-#  include <windows.h>
-#  include <float.h>
-#endif
+#include <iostream>
 
 #include <plib/netSocket.h>
 
-#include <simgear/props/props.hxx>
-#include <simgear/timing/sg_time.hxx>
-#include <simgear/math/sg_random.h>
+#include <osg/Camera>
+#include <osg/GraphicsContext>
+#include <osgDB/Registry>
 
 // Class references
 #include <simgear/ephemeris/ephemeris.hxx>
@@ -53,6 +46,11 @@
 #include <simgear/scene/material/matlib.hxx>
 #include <simgear/scene/model/animation.hxx>
 #include <simgear/scene/sky/sky.hxx>
+#include <simgear/structure/event_mgr.hxx>
+#include <simgear/props/props.hxx>
+#include <simgear/timing/sg_time.hxx>
+#include <simgear/math/sg_random.h>
+
 #include <Time/light.hxx>
 #include <Include/general.hxx>
 #include <Aircraft/replay.hxx>
@@ -66,32 +64,32 @@
 #include <Sound/beacon.hxx>
 #include <Sound/morse.hxx>
 #include <FDM/flight.hxx>
-// #include <FDM/ADA.hxx>
-#include <ATC/ATCdisplay.hxx>
-#include <ATC/ATCmgr.hxx>
-#include <ATC/AIMgr.hxx>
+#include <ATCDCL/ATCmgr.hxx>
+#include <ATCDCL/AIMgr.hxx>
 #include <Time/tmp.hxx>
 #include <Time/fg_timer.hxx>
 #include <Environment/environment_mgr.hxx>
 #include <GUI/new_gui.hxx>
 #include <MultiPlayer/multiplaymgr.hxx>
 
+#include "CameraGroup.hxx"
 #include "fg_commands.hxx"
 #include "fg_io.hxx"
 #include "renderer.hxx"
 #include "splash.hxx"
 #include "main.hxx"
-
+#include "util.hxx"
+#include "fg_init.hxx"
+#include "WindowSystemAdapter.hxx"
 
 
 static double real_delta_time_sec = 0.0;
 double delta_time_sec = 0.0;
 extern float init_volume;
 
+using namespace flightgear;
 
-#ifdef macintosh
-#  include <console.h>		// -dw- for command line dialog
-#endif
+using std::cerr;
 
 // This is a record containing a bit of global housekeeping information
 FGGeneral general;
@@ -138,9 +136,9 @@ void fgUpdateTimeDepCalcs() {
         double lat = fgGetDouble("/sim/presets/latitude-deg");
         // We require just to have 50 meter scenery availabe around
         // the aircraft.
-        double range = 50.0;
-        if (globals->get_tile_mgr()->scenery_available(lat, lon, range)) {
-            SG_LOG(SG_FLIGHT,SG_INFO, "Finally initializing fdm");
+        double range = 1000.0;
+        if (globals->get_scenery()->scenery_available(lat, lon, range)) {
+            //SG_LOG(SG_FLIGHT, SG_INFO, "Finally initializing fdm");
             cur_fdm_state->init();
             if ( cur_fdm_state->get_bound() ) {
                 cur_fdm_state->unbind();
@@ -179,7 +177,6 @@ void fgUpdateTimeDepCalcs() {
         // do nothing, fdm isn't inited yet
     }
 
-    globals->get_model_mgr()->update(delta_time_sec);
     globals->get_aircraft_model()->update(delta_time_sec);
 
     // Update solar system
@@ -210,6 +207,12 @@ static void fgMainLoop( void ) {
         = fgGetNode("/position/latitude-deg");
     static SGConstPropertyNode_ptr altitude
         = fgGetNode("/position/altitude-ft");
+    static SGConstPropertyNode_ptr vn_fps
+        = fgGetNode("/velocities/speed-north-fps");
+    static SGConstPropertyNode_ptr ve_fps
+        = fgGetNode("/velocities/speed-east-fps");
+    static SGConstPropertyNode_ptr vd_fps
+        = fgGetNode("/velocities/speed-down-fps");
     static SGConstPropertyNode_ptr clock_freeze
         = fgGetNode("/sim/freeze/clock", true);
     static SGConstPropertyNode_ptr cur_time_override
@@ -219,7 +222,7 @@ static void fgMainLoop( void ) {
     static SGPropertyNode_ptr frame_signal
         = fgGetNode("/sim/signals/frame", true);
 
-    frame_signal->setBoolValue(true);
+    frame_signal->fireValueChanged();
     SGCloudLayer::enable_bump_mapping = fgGetBool("/sim/rendering/bump-mapping");
 
     bool scenery_loaded = fgGetBool("sim/sceneryloaded");
@@ -268,7 +271,6 @@ static void fgMainLoop( void ) {
             frame_us = 0.0;
         }
         current_time_stamp.stamp();
-
         /* Convert to ms */
         double elapsed_us = current_time_stamp - last_time_stamp;
         if ( elapsed_us < frame_us ) {
@@ -326,7 +328,6 @@ static void fgMainLoop( void ) {
     }
     last_time_stamp = current_time_stamp;
     globals->inc_sim_time_sec( delta_time_sec );
-    SGAnimation::set_sim_time_sec( globals->get_sim_time_sec() );
 
     // These are useful, especially for Nasal scripts.
     fgSetDouble("/sim/time/delta-realtime-sec", real_delta_time_sec);
@@ -347,14 +348,6 @@ static void fgMainLoop( void ) {
     SG_LOG( SG_ALL, SG_DEBUG, "Running Main Loop");
     SG_LOG( SG_ALL, SG_DEBUG, "======= ==== ====");
 
-#if defined( ENABLE_PLIB_JOYSTICK )
-    // Read joystick and update control settings
-    // if ( fgGetString("/sim/control-mode") == "joystick" )
-    // {
-    //    fgJoystickRead();
-    // }
-#endif
-
     // Fix elevation.  I'm just sticking this here for now, it should
     // probably move eventually
 
@@ -493,12 +486,6 @@ static void fgMainLoop( void ) {
             "Elapsed time is zero ... we're zinging" );
     }
 
-    // Do any I/O channel work that might need to be done
-    globals->get_io()->update( real_delta_time_sec );
-
-    // see if we need to load any deferred-load textures
-    globals->get_matlib()->load_next_deferred();
-
     // Run audio scheduler
 #ifdef ENABLE_AUDIO_SUPPORT
     if ( globals->get_soundmgr()->is_working() ) {
@@ -515,29 +502,8 @@ static void fgMainLoop( void ) {
     //
     double visibility_meters = fgGetDouble("/environment/visibility-m");
     FGViewer *current_view = globals->get_current_view();
-    // Let the scenery center follow the current view position with
-    // 30m increments.
-    //
-    // Having the scenery center near the view position will eliminate
-    // jitter of objects which are placed very near the view position
-    // and haveing it's center near that view position.
-    // So the 3d insruments of the aircraft will not jitter with this.
-    // 
-    // Following the view position exactly would introduce jitter of
-    // the scenery tiles (they would be from their center up to 10000m
-    // to the view and this will introduce roundoff too). By stepping
-    // at 30m incements the roundoff error of the scenery tiles is
-    // still present, but we will make exactly the same roundoff error
-    // at each frame until the center is switched to a new
-    // position. This roundoff is still visible but you will most
-    // propably not notice.
-    double *vp = globals->get_current_view()->get_absolute_view_pos();
-    SGVec3d cntr(vp);
-    if (30.0*30.0 < distSqr(cntr, globals->get_scenery()->get_center()))
-      globals->get_scenery()->set_center( cntr );
 
-    globals->get_tile_mgr()->prep_ssg_nodes( current_view->getSGLocation(),
-                                             visibility_meters );
+    globals->get_tile_mgr()->prep_ssg_nodes( visibility_meters );
     // update tile manager for view...
     SGLocation *view_location = globals->get_current_view()->getSGLocation();
     globals->get_tile_mgr()->update( view_location, visibility_meters );
@@ -550,8 +516,7 @@ static void fgMainLoop( void ) {
         double ref_time, r;
         SGVec3d pt;
         bool valid = cur_fdm_state->is_valid_m(&ref_time, pt.sg(), &r);
-        double *vp = globals->get_current_view()->get_absolute_view_pos();
-        SGVec3d viewpos(vp);
+        SGVec3d viewpos(globals->get_current_view()->get_view_pos());
         if (valid && distSqr(viewpos, pt) < r*r) {
             // Reuse the cache ...
             double lev
@@ -572,107 +537,127 @@ static void fgMainLoop( void ) {
     // run Nasal's settimer() loops right before the view manager
     globals->get_event_mgr()->update(delta_time_sec);
 
+    // pick up model coordidnates that Nasal code may have set relative to the
+    // aircraft's
+    globals->get_model_mgr()->update(delta_time_sec);
+
     // update the view angle as late as possible, but before sound calculations
     globals->get_viewmgr()->update(delta_time_sec);
 
+    // Do any I/O channel work that might need to be done (must come after viewmgr)
+    globals->get_io()->update(real_delta_time_sec);
+
 #ifdef ENABLE_AUDIO_SUPPORT
     // Right now we make a simplifying assumption that the primary
     // aircraft is the source of all sounds and that all sounds are
-    // positioned relative to the current view position.
+    // positioned in the aircraft base
 
-    static sgdVec3 last_visitor_pos = {0, 0, 0};
+    static sgdVec3 last_listener_pos = {0, 0, 0};
     static sgdVec3 last_model_pos = {0, 0, 0};
 
     // get the orientation
-    const sgVec4 *view = current_view->get_VIEW();
-
+    const SGQuatd view_or = current_view->getViewOrientation();
     SGQuatd surf_or = SGQuatd::fromLonLatDeg(
         current_view->getLongitude_deg(), current_view->getLatitude_deg());
-
     SGQuatd model_or = SGQuatd::fromYawPitchRollDeg(
         globals->get_aircraft_model()->get3DModel()->getHeadingDeg(),
         globals->get_aircraft_model()->get3DModel()->getPitchDeg(),
         globals->get_aircraft_model()->get3DModel()->getRollDeg());
 
     // get the up and at vector in the aircraft base
-    sgVec3 u;
-    sgSetVec3 ( u, 0, 0, 1 ); //up
-    sgXformVec3( u, view );
+    // (ok, the up vector is a down vector, but the coordinates
+    // are finally calculated in a left hand system and openal
+    // lives in a right hand system. Therefore we need to pass
+    // the down vector to get correct stereo sound.)
     SGVec3d sgv_up = model_or.rotateBack(
-        surf_or.rotateBack(SGVec3d(u[0], u[1], u[2])));
+        surf_or.rotateBack(view_or.rotate(SGVec3d(0, 1, 0))));
     sgVec3 up;
     sgSetVec3(up, sgv_up[0], sgv_up[1], sgv_up[2]);
-
-    sgSetVec3 ( u, 0, -1, 0 ); //at
-    sgXformVec3( u, view );
     SGVec3d sgv_at = model_or.rotateBack(
-        surf_or.rotateBack(SGVec3d(u[0], u[1], u[2])));
+        surf_or.rotateBack(view_or.rotate(SGVec3d(0, 0, 1))));
     sgVec3 at;
     sgSetVec3(at, sgv_at[0], sgv_at[1], sgv_at[2]);
 
-     // get the location data for the primary FDM (now hardcoded to ac model)...
+    // get the location data for the primary FDM (now hardcoded to ac model)...
     SGLocation *acmodel_loc = NULL;
     acmodel_loc = (SGLocation *)globals->
         get_aircraft_model()->get3DModel()->getSGLocation();
 
-    // calculate speed of visitor and model
+    // Calculate speed of listener and model.  This code assumes the
+    // listener is either tracking the model at the same speed or
+    // stationary.
+
     sgVec3 listener_vel, model_vel;
-    sgdVec3 v;
-    SGVec3d w;
-    sgdVec3 null = {0, 0, 0};
+    SGVec3d SGV3d_help;
+    sgdVec3 sgdv3_help;
+    sgdVec3 sgdv3_null = {0, 0, 0};
 
-    double *abs_view_pos = current_view->get_absolute_view_pos();
-    sgdSubVec3( v, last_visitor_pos, abs_view_pos);
-    sgdAddVec3( last_visitor_pos, null, abs_view_pos);
-    w = model_or.rotateBack( surf_or.rotateBack(SGVec3d(v[0], v[1], v[2])));
-    sgSetVec3( listener_vel, w[0], w[1], w[2]);
+    // the aircraft velocity as reported by the fdm (this will not
+    // vary or be affected by frame rates or timing jitter.)
+    sgVec3 fdm_vel_vec;
+    sgSetVec3( fdm_vel_vec,
+	       vn_fps->getDoubleValue() * SG_FEET_TO_METER,
+	       ve_fps->getDoubleValue() * SG_FEET_TO_METER,
+	       vd_fps->getDoubleValue() * SG_FEET_TO_METER );
+    double fdm_vel = sgLengthVec3(fdm_vel_vec);
 
-    double *model_abs_view_pos = acmodel_loc->get_absolute_view_pos();
-    sgdSubVec3( v, last_model_pos, model_abs_view_pos);
-    sgdAddVec3( last_model_pos, null, model_abs_view_pos);
-    w = model_or.rotateBack( surf_or.rotateBack(SGVec3d(v[0], v[1], v[2])));
-    sgSetVec3( model_vel, w[0], w[1], w[2]);
+    // compute the aircraft velocity vector and scale it to the length
+    // of the fdm velocity vector.  This gives us a vector in the
+    // proper coordinate system, but also with the proper time
+    // invariant magnitude.
+    sgdSubVec3( sgdv3_help,
+                last_model_pos, acmodel_loc->get_absolute_view_pos());
+    sgdAddVec3( last_model_pos, sgdv3_null, acmodel_loc->get_absolute_view_pos());
+    SGV3d_help = model_or.rotateBack(
+        surf_or.rotateBack(SGVec3d(sgdv3_help[0],
+        sgdv3_help[1], sgdv3_help[2])));
+    sgSetVec3( model_vel, SGV3d_help[0], SGV3d_help[1], SGV3d_help[2]);
 
-    if (delta_time_sec > 0) {
-        sgScaleVec3( model_vel, 1 / delta_time_sec );
-        sgScaleVec3( listener_vel, 1 / delta_time_sec );
+    float vel = sgLengthVec3(model_vel);
+    if ( fabs(vel) > 0.0001 ) {
+	if ( fabs(fdm_vel / vel) > 0.0001 ) {
+	    sgScaleVec3( model_vel, fdm_vel / vel );
+	}
     }
 
-    //checking, if the listener pos has moved suddenly
-    if (sgLengthVec3(listener_vel) > 1000)
-    {
-        //check if the relative speed model vs listener has moved suddenly, too
-        sgVec3 delta_vel;
-        sgSubVec3(delta_vel, listener_vel, model_vel );
-        if (sgLengthVec3(delta_vel) > 1000)
-            sgSetVec3(listener_vel, model_vel[0], model_vel[1], model_vel[2] );
-        else
-            globals->get_soundmgr()->set_listener_vel( listener_vel );
+    // check for moving or stationary listener (view position)
+    sgdSubVec3( sgdv3_help,
+                last_listener_pos, (double *)&current_view->get_view_pos());
+    sgdAddVec3( last_listener_pos,
+		sgdv3_null, (double *)&current_view->get_view_pos());
+
+    if ( sgdLengthVec3(sgdv3_help) > 0.2 ) {
+	sgCopyVec3( listener_vel, model_vel );
+    } else {
+	sgSetVec3( listener_vel, 0.0, 0.0, 0.0 );
     }
-    else
-        globals->get_soundmgr()->set_listener_vel( listener_vel );
+
+    globals->get_soundmgr()->set_listener_vel( listener_vel );
 
     // set positional offset for sources
-    sgdVec3 dsource_pos_offset = { abs_view_pos[0], abs_view_pos[1], abs_view_pos[2] };
-
-    sgdSubVec3( dsource_pos_offset, dsource_pos_offset, model_abs_view_pos );
+    sgdVec3 dsource_pos_offset;
+    sgdSubVec3( dsource_pos_offset,
+                (double*) &current_view->get_view_pos(),
+                acmodel_loc->get_absolute_view_pos() );
     SGVec3d sgv_dsource_pos_offset = model_or.rotateBack(
-         surf_or.rotateBack(SGVec3d(dsource_pos_offset[0],
-         dsource_pos_offset[1], dsource_pos_offset[2])));
+        surf_or.rotateBack(SGVec3d(dsource_pos_offset[0],
+        dsource_pos_offset[1], dsource_pos_offset[2])));
 
     sgVec3 source_pos_offset;
     sgSetVec3(source_pos_offset, sgv_dsource_pos_offset[0],
-         sgv_dsource_pos_offset[1], sgv_dsource_pos_offset[2]);
+        sgv_dsource_pos_offset[1], sgv_dsource_pos_offset[2]);
+
     globals->get_soundmgr()->set_source_pos_all( source_pos_offset );
 
     float orient[6];
     for (int i = 0; i < 3; i++) {
         orient[i] = sgv_at[i];
         orient[i + 3] = sgv_up[i];
-     }
-     globals->get_soundmgr()->set_listener_orientation( orient );
+    }
+    globals->get_soundmgr()->set_listener_orientation( orient );
 
     // set the velocity
+    // all sources are defined to be in the model
     globals->get_soundmgr()->set_source_vel_all( model_vel );
 
     // The listener is always positioned at the origin.
@@ -683,44 +668,30 @@ static void fgMainLoop( void ) {
 
     // END Tile Manager udpates
 
-    if (!scenery_loaded && globals->get_tile_mgr()->all_queues_empty() && cur_fdm_state->get_inited()) {
+    if (!scenery_loaded && globals->get_tile_mgr()->isSceneryLoaded()
+        && cur_fdm_state->get_inited()) {
         fgSetBool("sim/sceneryloaded",true);
         fgSetFloat("/sim/sound/volume", init_volume);
         globals->get_soundmgr()->set_volume(init_volume);
     }
 
-    if (fgGetBool("/sim/rendering/specular-highlight")) {
-        glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
-	// glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
-    } else {
-        glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR);
-        // glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
-    }
-
     fgRequestRedraw();
 
     SG_LOG( SG_ALL, SG_DEBUG, "" );
 }
 
-
-// This is the top level master main function that is registered as
-// our idle funciton
-
-// The first few passes take care of initialization things (a couple
-// per pass) and once everything has been initialized fgMainLoop from
-// then on.
-
-static void fgIdleFunction ( void ) {
-    if ( idle_state == 0 ) {
-        idle_state++;
-
-#ifdef GL_EXT_texture_lod_bias
-        // negative values sharpen, positive values blur mipmapped textures
-        glTexEnvf( GL_TEXTURE_FILTER_CONTROL_EXT, GL_TEXTURE_LOD_BIAS_EXT, -0.5 ) ;
-#endif
-        // This seems to be the absolute earliest in the init sequence
-        // that these calls will return valid info.  Too bad it's after
-        // we've already created and sized our window. :-(
+// Operation for querying OpenGL parameters. This must be done in a
+// valid OpenGL context, potentially in another thread.
+namespace
+{
+struct GeneralInitOperation : public GraphicsContextOperation
+{
+    GeneralInitOperation()
+        : GraphicsContextOperation(std::string("General init"))
+    {
+    }
+    void run(osg::GraphicsContext* gc)
+    {
         general.set_glVendor( (char *)glGetString ( GL_VENDOR ) );
         general.set_glRenderer( (char *)glGetString ( GL_RENDERER ) );
         general.set_glVersion( (char *)glGetString ( GL_VERSION ) );
@@ -734,19 +705,45 @@ static void fgIdleFunction ( void ) {
         glGetIntegerv( GL_DEPTH_BITS, &tmp );
         general.set_glDepthBits( tmp );
         SG_LOG ( SG_GENERAL, SG_INFO, "Depth buffer bits = " << tmp );
+    }
+};
+}
 
-        // Initialize the user interface so that we can use fonts
-        guiInit();
-        fgSplashProgress("setting up scenegraph");
+// This is the top level master main function that is registered as
+// our idle funciton
 
+// The first few passes take care of initialization things (a couple
+// per pass) and once everything has been initialized fgMainLoop from
+// then on.
 
+static void fgIdleFunction ( void ) {
+    static osg::ref_ptr<GeneralInitOperation> genOp;
+    if ( idle_state == 0 ) {
+        idle_state++;
+        // Pick some window on which to do queries.
+        // XXX Perhaps all this graphics initialization code should be
+        // moved to renderer.cxx?
+        genOp = new GeneralInitOperation;
+        osg::Camera* guiCamera = getGUICamera(CameraGroup::getDefault());
+        WindowSystemAdapter* wsa = WindowSystemAdapter::getWSA();
+        osg::GraphicsContext* gc = 0;
+        if (guiCamera)
+            gc = guiCamera->getGraphicsContext();
+        if (gc) {
+            gc->add(genOp.get());
+        } else {
+            wsa->windows[0]->gc->add(genOp.get());
+        }
+        guiStartInit(gc);
     } else if ( idle_state == 1 ) {
+        if (genOp.valid()) {
+            if (!genOp->isFinished())
+                return;
+            genOp = 0;
+        }
+        if (!guiFinishInit())
+            return;
         idle_state++;
-
-        // Initialize ssg (from plib).  Needs to come before we do any
-        // other ssg stuff, but after opengl has been initialized.
-        ssgInit();
-
         fgSplashProgress("reading aircraft list");
 
 
@@ -754,23 +751,6 @@ static void fgIdleFunction ( void ) {
         idle_state++;
         // Read the list of available aircraft
         fgReadAircraft();
-
-        // get the address of our OpenGL extensions
-        if (SGIsOpenGLExtensionSupported("GL_EXT_point_parameters") ) {
-            glPointParameterIsSupported = true;
-            glPointParameterfPtr = (glPointParameterfProc)
-                SGLookupFunction("glPointParameterfEXT");
-            glPointParameterfvPtr = (glPointParameterfvProc)
-                SGLookupFunction("glPointParameterfvEXT");
-        } else if ( SGIsOpenGLExtensionSupported("GL_ARB_point_parameters") ) {
-            glPointParameterIsSupported = true;
-            glPointParameterfPtr = (glPointParameterfProc)
-                SGLookupFunction("glPointParameterfARB");
-            glPointParameterfvPtr = (glPointParameterfvProc)
-                SGLookupFunction("glPointParameterfvARB");
-        } else {
-            glPointParameterIsSupported = false;
-        }
         fgSplashProgress("reading airport & navigation data");
 
 
@@ -807,7 +787,7 @@ static void fgIdleFunction ( void ) {
         // Initialize the material manager
         ////////////////////////////////////////////////////////////////////
         globals->set_matlib( new SGMaterialLib );
-        globals->set_model_lib(new SGModelLib);
+        simgear::SGModelLib::init(globals->get_fg_root());
 
 
         ////////////////////////////////////////////////////////////////////
@@ -861,6 +841,13 @@ static void fgIdleFunction ( void ) {
 
         // TODO: move to environment mgr
         thesky = new SGSky;
+        SGPath texture_path(globals->get_fg_root());
+        texture_path.append("Textures");
+        texture_path.append("Sky");
+        for (int i = 0; i < FGEnvironmentMgr::MAX_CLOUD_LAYERS; i++) {
+            SGCloudLayer * layer = new SGCloudLayer(texture_path.str());
+            thesky->add_cloud_layer(layer);
+        }
 
         SGPath sky_tex_path( globals->get_fg_root() );
         sky_tex_path.append( "Textures" );
@@ -880,15 +867,6 @@ static void fgIdleFunction ( void ) {
                        globals->get_ephem()->getStars(),
                        fgGetNode("/environment", true));
 
-
-        SGPath texture_path(globals->get_fg_root());
-        texture_path.append("Textures");
-        texture_path.append("Sky");
-        for (int i = 0; i < FGEnvironmentMgr::MAX_CLOUD_LAYERS; i++) {
-            SGCloudLayer * layer = new SGCloudLayer(texture_path.str());
-            thesky->add_cloud_layer(layer);
-        }
-
         // Initialize MagVar model
         SGMagVar *magvar = new SGMagVar();
         globals->set_mag( magvar );
@@ -915,7 +893,6 @@ static void fgIdleFunction ( void ) {
         // lighting->addKid( airport );
 
         // build our custom render states
-        globals->get_renderer()->build_states();
         fgSplashProgress("initializing subsystems");
 
 
@@ -943,16 +920,6 @@ static void fgIdleFunction ( void ) {
             system ( command.c_str() );
         }
 #endif
-
-        // These are a few miscellaneous things that aren't really
-        // "subsystems" but still need to be initialized.
-
-#ifdef USE_GLIDE
-        if ( strstr ( general.get_glRenderer(), "Glide" ) ) {
-            grTexLodBiasValue ( GR_TMU0, 1.0 ) ;
-        }
-#endif
-
         // This is the top level init routine which calls all the
         // other subsystem initialization routines.  If you are adding
         // a subsystem to flightgear, its initialization call should be
@@ -1011,12 +978,6 @@ static void upper_case_property(const char *name)
 // Main top level initialization
 bool fgMainInit( int argc, char **argv ) {
 
-#if defined( macintosh )
-    freopen ("stdout.txt", "w", stdout );
-    freopen ("stderr.txt", "w", stderr );
-    argc = ccommand( &argv );
-#endif
-
     // set default log levels
     sglog().setLogLevels( SG_ALL, SG_ALERT );
 
@@ -1044,16 +1005,18 @@ bool fgMainInit( int argc, char **argv ) {
     string_list *col = new string_list;
     globals->set_channel_options_list( col );
 
+    fgValidatePath("", false);  // initialize static variables
     upper_case_property("/sim/presets/airport-id");
     upper_case_property("/sim/presets/runway");
     upper_case_property("/sim/tower/airport-id");
+    upper_case_property("/autopilot/route-manager/input");
 
     // Scan the config file(s) and command line options to see if
     // fg_root was specified (ignore all other options for now)
     fgInitFGRoot(argc, argv);
 
     // Check for the correct base package version
-    static char required_version[] = "1.0.0";
+    static char required_version[] = "1.9.0";
     string base_version = fgBasePackageVersion();
     if ( !(base_version == required_version) ) {
         // tell the operator how to use this application
@@ -1070,8 +1033,6 @@ bool fgMainInit( int argc, char **argv ) {
         exit(-1);
     }
 
-    sgUseDisplayList = fgGetBool( "/sim/rendering/use-display-list", true );
-
     // Load the configuration parameters.  (Command line options
     // override config file options.  Config file options override
     // defaults.)
@@ -1081,13 +1042,8 @@ bool fgMainInit( int argc, char **argv ) {
     }
 
     // Initialize the Window/Graphics environment.
-#if !defined(__APPLE__) || defined(OSX_BUNDLE)
-    // Mac OS X command line ("non-bundle") applications call this
-    // from main(), in bootstrap.cxx.  Andy doesn't know why, someone
-    // feel free to add comments...
     fgOSInit(&argc, argv);
     _bootstrap_OSInit++;
-#endif
 
     fgRegisterWindowResizeHandler( &FGRenderer::resize );
     fgRegisterIdleHandler( &fgIdleFunction );
@@ -1104,16 +1060,11 @@ bool fgMainInit( int argc, char **argv ) {
 
     // Clouds3D requires an alpha channel
     // clouds may require stencil buffer
-    fgOSOpenWindow( fgGetInt("/sim/startup/xsize"),
-                    fgGetInt("/sim/startup/ysize"),
-                    fgGetInt("/sim/rendering/bits-per-pixel"),
-                    fgGetBool("/sim/rendering/clouds3d-enable"),
-                    get_stencil_buffer,
-                    fgGetBool("/sim/startup/fullscreen") );
+    fgOSOpenWindow(get_stencil_buffer);
 
     // Initialize the splash screen right away
     fntInit();
-    fgSplashInit(fgGetString("/sim/startup/splash-texture"));
+    fgSplashInit();
 
     // pass control off to the master event handler
     fgOSMainLoop();
