@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: options.cxx,v 1.89.2.8 2007-11-02 18:45:59 curt Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
@@ -35,13 +35,16 @@
 #include <string.h>		// strcmp()
 #include <algorithm>
 
-#include STL_STRING
+#include <iostream>
+#include <string>
 
 #include <plib/ul.h>
 
 #include <simgear/math/sg_random.h>
+#include <simgear/props/props_io.hxx>
 #include <simgear/misc/sgstream.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/scene/material/mat.hxx>
 
 // #include <Include/general.hxx>
 // #include <Airports/simple.hxx>
@@ -59,10 +62,15 @@
 #include "viewmgr.hxx"
 
 
-SG_USING_STD(string);
-SG_USING_STD(sort);
-SG_USING_NAMESPACE(std);
+using std::string;
+using std::sort;
+using std::cout;
+using std::cerr;
+using std::endl;
 
+#ifndef VERSION
+#define VERSION "CVS "__DATE__
+#endif
 
 #define NEW_DEFAULT_MODEL_HZ 120
 
@@ -71,31 +79,21 @@ enum
     FG_OPTIONS_OK = 0,
     FG_OPTIONS_HELP = 1,
     FG_OPTIONS_ERROR = 2,
-    FG_OPTIONS_VERBOSE_HELP = 3,
-    FG_OPTIONS_SHOW_AIRCRAFT = 4
+    FG_OPTIONS_EXIT = 3,
+    FG_OPTIONS_VERBOSE_HELP = 4,
+    FG_OPTIONS_SHOW_AIRCRAFT = 5
 };
 
 static double
 atof( const string& str )
 {
-
-#ifdef __MWERKS__
-    // -dw- if ::atof is called, then we get an infinite loop
-    return std::atof( str.c_str() );
-#else
     return ::atof( str.c_str() );
-#endif
 }
 
 static int
 atoi( const string& str )
 {
-#ifdef __MWERKS__
-    // -dw- if ::atoi is called, then we get an infinite loop
-    return std::atoi( str.c_str() );
-#else
     return ::atoi( str.c_str() );
-#endif
 }
 
 /**
@@ -203,6 +201,8 @@ fgSetDefaults ()
     fgSetBool("/sim/rendering/shading", true);
     fgSetBool("/sim/rendering/skyblend", true);
     fgSetBool("/sim/rendering/textures", true);
+	fgTie( "/sim/rendering/filtering", SGGetTextureFilter, SGSetTextureFilter, false);
+    fgSetInt("/sim/rendering/filtering", 1);
     fgSetBool("/sim/rendering/wireframe", false);
     fgSetBool("/sim/rendering/horizon-effect", false);
     fgSetBool("/sim/rendering/enhanced-lighting", false);
@@ -491,7 +491,7 @@ parse_fov( const string& arg ) {
     if ( fov < FG_FOV_MIN ) { fov = FG_FOV_MIN; }
     if ( fov > FG_FOV_MAX ) { fov = FG_FOV_MAX; }
 
-    fgSetDouble("/sim/current-view/field-of-view", fov);
+    fgSetDouble("/sim/view[0]/config/default-field-of-view-deg", fov);
 
     // printf("parse_fov(): result = %.4f\n", fov);
 
@@ -564,12 +564,8 @@ parse_flightplan(const string& arg)
 
     while ( true ) {
         string line;
-
-#if defined( macintosh )
-        getline( in, line, '\r' );
-#else
         getline( in, line, '\n' );
-#endif
+
         // catch extraneous (DOS) line ending character
         if ( line[line.length() - 1] < 32 )
             line = line.substr( 0, line.length()-1 );
@@ -590,6 +586,15 @@ fgOptLanguage( const char *arg )
     return FG_OPTIONS_OK;
 }
 
+static int
+fgOptWeather( const char *arg )
+{
+    if (arg[0])
+        fgSetString("/environment/weather-scenario", arg);
+    fgSetBool("/environment/params/real-world-weather-fetch", !strcmp(arg, "METAR"));
+    return FG_OPTIONS_OK;
+}
+
 static void
 clearLocation ()
 {
@@ -876,7 +881,7 @@ fgSetupProxy( const char *arg )
 {
     string options = arg;
     string host, port, auth;
-    unsigned int pos;
+    string::size_type pos;
 
     host = port = auth = "";
     if ((pos = options.find("@")) != string::npos) 
@@ -1186,6 +1191,24 @@ fgOptRunway( const char *arg )
     return FG_OPTIONS_OK;
 }
 
+static int
+fgOptParking( const char *arg )
+{
+    cerr << "Processing argument " << arg << endl;
+    fgSetString("/sim/presets/parking", arg );
+    fgSetBool  ("/sim/presets/parking-requested", true );
+    return FG_OPTIONS_OK;
+}
+
+static int
+fgOptVersion( const char *arg )
+{
+    cerr << VERSION << endl;
+    cerr << "FG_ROOT=" << globals->get_fg_root() << endl;
+    cerr << "FG_HOME=" << fgGetString("/sim/fg-home") << endl;
+    return FG_OPTIONS_EXIT;
+}
+
 static map<string,size_t> fgOptionMap;
 
 /*
@@ -1206,7 +1229,7 @@ where:
              value set to the property if has_param is false
  func      : function called if type==OPTION_FUNC. if has_param is true,
              the value is passed to the function as a string, otherwise,
-             0 is passed. 
+             s_param is passed.
 
     For OPTION_DOUBLE and OPTION_INT, the parameter value is converted into a
     double or an integer and set to the property.
@@ -1217,12 +1240,12 @@ where:
 
 enum OptionType { OPTION_BOOL, OPTION_STRING, OPTION_DOUBLE, OPTION_INT, OPTION_CHANNEL, OPTION_FUNC };
 struct OptionDesc {
-    char *option;
+    const char *option;
     bool has_param;
     enum OptionType type;
-    char *property;
+    const char *property;
     bool b_param;
-    char *s_param;
+    const char *s_param;
     int (*func)( const char * );
     } fgOptionArray[] = {
        
@@ -1237,8 +1260,8 @@ struct OptionDesc {
     {"enable-mouse-pointer",         false, OPTION_STRING, "/sim/startup/mouse-pointer", false, "enabled", 0 },
     {"disable-random-objects",       false, OPTION_BOOL,   "/sim/rendering/random-objects", false, "", 0 },
     {"enable-random-objects",        false, OPTION_BOOL,   "/sim/rendering/random-objects", true, "", 0 },
-    {"disable-real-weather-fetch",   false, OPTION_BOOL,   "/environment/params/real-world-weather-fetch", false, "", 0 },
-    {"enable-real-weather-fetch",    false, OPTION_BOOL,   "/environment/params/real-world-weather-fetch", true, "", 0 },
+    {"disable-real-weather-fetch",   false, OPTION_FUNC,   "", false, "", fgOptWeather },
+    {"enable-real-weather-fetch",    false, OPTION_FUNC,   "", false, "METAR", fgOptWeather },
     {"disable-ai-models",            false, OPTION_BOOL,   "/sim/ai/enabled", false, "", 0 },
     {"enable-ai-models",             false, OPTION_BOOL,   "/sim/ai/enabled", true, "", 0 },
     {"disable-freeze",               false, OPTION_BOOL,   "/sim/freeze/master", false, "", 0 },
@@ -1262,15 +1285,14 @@ struct OptionDesc {
     {"disable-sound",                false, OPTION_BOOL,   "/sim/sound/pause", true, "", 0 },
     {"enable-sound",                 false, OPTION_BOOL,   "/sim/sound/pause", false, "", 0 },
     {"airport",                      true,  OPTION_STRING, "/sim/presets/airport-id", false, "", 0 },
-    {"airport-id",                   true,  OPTION_STRING, "/sim/presets/airport-id", false, "", 0 },
     {"runway",                       true,  OPTION_FUNC,   "", false, "", fgOptRunway },
     {"vor",                          true,  OPTION_FUNC,   "", false, "", fgOptVOR },
     {"ndb",                          true,  OPTION_FUNC,   "", false, "", fgOptNDB },
     {"carrier",                      true,  OPTION_FUNC,   "", false, "", fgOptCarrier },
     {"parkpos",                      true,  OPTION_FUNC,   "", false, "", fgOptParkpos },
     {"fix",                          true,  OPTION_FUNC,   "", false, "", fgOptFIX },
-    {"offset-distance",              true,  OPTION_DOUBLE, "/sim/presets/offset-distance", false, "", 0 },
-    {"offset-azimuth",               true,  OPTION_DOUBLE, "/sim/presets/offset-azimuth", false, "", 0 },
+    {"offset-distance",              true,  OPTION_DOUBLE, "/sim/presets/offset-distance-nm", false, "", 0 },
+    {"offset-azimuth",               true,  OPTION_DOUBLE, "/sim/presets/offset-azimuth-deg", false, "", 0 },
     {"lon",                          true,  OPTION_FUNC,   "", false, "", fgOptLon },
     {"lat",                          true,  OPTION_FUNC,   "", false, "", fgOptLat },
     {"altitude",                     true,  OPTION_FUNC,   "", false, "", fgOptAltitude },
@@ -1325,6 +1347,7 @@ struct OptionDesc {
     {"enable-skyblend",              false, OPTION_BOOL,   "/sim/rendering/skyblend", true, "", 0 },
     {"disable-textures",             false, OPTION_BOOL,   "/sim/rendering/textures", false, "", 0 },
     {"enable-textures",              false, OPTION_BOOL,   "/sim/rendering/textures", true, "", 0 },
+    {"texture-filtering",	     false, OPTION_INT,    "/sim/rendering/filtering", 1, "", 0 },
     {"disable-wireframe",            false, OPTION_BOOL,   "/sim/rendering/wireframe", false, "", 0 },
     {"enable-wireframe",             false, OPTION_BOOL,   "/sim/rendering/wireframe", true, "", 0 },
     {"geometry",                     true,  OPTION_FUNC,   "", false, "", fgOptGeometry },
@@ -1391,6 +1414,8 @@ struct OptionDesc {
     {"min-status",                   true,  OPTION_STRING,  "/sim/aircraft-min-status", false, "all", 0 },
     {"livery",                       true,  OPTION_FUNC,   "", false, "", fgOptLivery },
     {"ai-scenario",                  true,  OPTION_FUNC,   "", false, "", fgOptScenario },
+    {"parking-id",                   true,  OPTION_FUNC,   "", false, "", fgOptParking  },
+    {"version",                      false, OPTION_FUNC,   "", false, "", fgOptVersion },
     {0}
 };
 
@@ -1416,22 +1441,29 @@ set_property(const string& arg)
     }
     SGPropertyNode *n = fgGetNode(name.c_str(), true);
 
-    if (type.empty())
-        return n->setUnspecifiedValue(value.c_str());
-    else if (type == "s" || type == "string")
-        return n->setStringValue(value.c_str());
-    else if (type == "d" || type == "double")
-        return n->setDoubleValue(strtod(value.c_str(), 0));
-    else if (type == "f" || type == "float")
-        return n->setFloatValue(atof(value.c_str()));
-    else if (type == "l" || type == "long")
-        return n->setLongValue(strtol(value.c_str(), 0, 0));
-    else if (type == "i" || type == "int")
-        return n->setIntValue(atoi(value.c_str()));
-    else if (type == "b" || type == "bool")
-        return n->setBoolValue(value == "true" || atoi(value.c_str()) != 0);
+    bool writable = n->getAttribute(SGPropertyNode::WRITE);
+    if (!writable)
+        n->setAttribute(SGPropertyNode::WRITE, true);
 
-    return false;
+    bool ret = false;
+    if (type.empty())
+        ret = n->setUnspecifiedValue(value.c_str());
+    else if (type == "s" || type == "string")
+        ret = n->setStringValue(value.c_str());
+    else if (type == "d" || type == "double")
+        ret = n->setDoubleValue(strtod(value.c_str(), 0));
+    else if (type == "f" || type == "float")
+        ret = n->setFloatValue(atof(value.c_str()));
+    else if (type == "l" || type == "long")
+        ret =  n->setLongValue(strtol(value.c_str(), 0, 0));
+    else if (type == "i" || type == "int")
+        ret =  n->setIntValue(atoi(value.c_str()));
+    else if (type == "b" || type == "bool")
+        ret =  n->setBoolValue(value == "true" || atoi(value.c_str()) != 0);
+
+    if (!writable)
+        n->setAttribute(SGPropertyNode::WRITE, false);
+    return ret;
 }
 
 
@@ -1465,11 +1497,12 @@ parse_option (const string& arg)
         }
     } else if ( arg.find( "--" ) == 0 ) {
         size_t pos = arg.find( '=' );
-        string arg_name;
+        string arg_name, arg_value;
         if ( pos == string::npos ) {
             arg_name = arg.substr( 2 );
         } else {
             arg_name = arg.substr( 2, pos - 2 );
+            arg_value = arg.substr( pos + 1);
         }
         map<string,size_t>::iterator it = fgOptionMap.find( arg_name );
         if ( it != fgOptionMap.end() ) {
@@ -1479,9 +1512,9 @@ parse_option (const string& arg)
                     fgSetBool( pt->property, pt->b_param );
                     break;
                 case OPTION_STRING:
-                    if ( pt->has_param && pos != string::npos && pos + 1 < arg.size() ) {
-                        fgSetString( pt->property, arg.substr( pos + 1 ).c_str() );
-                    } else if ( !pt->has_param && pos == string::npos ) {
+                    if ( pt->has_param && !arg_value.empty() ) {
+                        fgSetString( pt->property, arg_value.c_str() );
+                    } else if ( !pt->has_param && arg_value.empty() ) {
                         fgSetString( pt->property, pt->s_param );
                     } else if ( pt->has_param ) {
                         SG_LOG( SG_GENERAL, SG_ALERT, "Option '" << arg << "' needs a parameter" );
@@ -1492,25 +1525,25 @@ parse_option (const string& arg)
                     }
                     break;
                 case OPTION_DOUBLE:
-                    if ( pos != string::npos && pos + 1 < arg.size() ) {
-                        fgSetDouble( pt->property, atof( arg.substr( pos + 1 ) ) );
+                    if ( !arg_value.empty() ) {
+                        fgSetDouble( pt->property, atof( arg_value ) );
                     } else {
                         SG_LOG( SG_GENERAL, SG_ALERT, "Option '" << arg << "' needs a parameter" );
                         return FG_OPTIONS_ERROR;
                     }
                     break;
                 case OPTION_INT:
-                    if ( pos != string::npos && pos + 1 < arg.size() ) {
-                        fgSetInt( pt->property, atoi( arg.substr( pos + 1 ) ) );
+                    if ( !arg_value.empty() ) {
+                        fgSetInt( pt->property, atoi( arg_value ) );
                     } else {
                         SG_LOG( SG_GENERAL, SG_ALERT, "Option '" << arg << "' needs a parameter" );
                         return FG_OPTIONS_ERROR;
                     }
                     break;
                 case OPTION_CHANNEL:
-                    if ( pt->has_param && pos != string::npos && pos + 1 < arg.size() ) {
-                        add_channel( pt->option, arg.substr( pos + 1 ) );
-                    } else if ( !pt->has_param && pos == string::npos ) {
+                    if ( pt->has_param && !arg_value.empty() ) {
+                        add_channel( pt->option, arg_value );
+                    } else if ( !pt->has_param && arg_value.empty() ) {
                         add_channel( pt->option, pt->s_param );
                     } else if ( pt->has_param ) {
                         SG_LOG( SG_GENERAL, SG_ALERT, "Option '" << arg << "' needs a parameter" );
@@ -1521,10 +1554,10 @@ parse_option (const string& arg)
                     }
                     break;
                 case OPTION_FUNC:
-                    if ( pt->has_param && pos != string::npos && pos + 1 < arg.size() ) {
-                        return pt->func( arg.substr( pos + 1 ).c_str() );
-                    } else if ( !pt->has_param && pos == string::npos ) {
-                        return pt->func( 0 );
+                    if ( pt->has_param && !arg_value.empty() ) {
+                        return pt->func( arg_value.c_str() );
+                    } else if ( !pt->has_param && arg_value.empty() ) {
+                        return pt->func( pt->s_param );
                     } else if ( pt->has_param ) {
                         SG_LOG( SG_GENERAL, SG_ALERT, "Option '" << arg << "' needs a parameter" );
                         return FG_OPTIONS_ERROR;
@@ -1578,6 +1611,9 @@ fgParseArgs (int argc, char **argv)
                fgShowAircraft(path, true);
                exit(0);
             }
+
+            else if (result == FG_OPTIONS_EXIT)
+               exit(0);
 	  }
 	} else {
 	  in_options = false;
@@ -1608,20 +1644,9 @@ fgParseOptions (const string& path) {
     SG_LOG( SG_GENERAL, SG_INFO, "Processing config file: " << path );
 
     in >> skipcomment;
-#ifndef __MWERKS__
     while ( ! in.eof() ) {
-#else
-    char c = '\0';
-    while ( in.get(c) && c != '\0' ) {
-	in.putback(c);
-#endif
 	string line;
-
-#if defined( macintosh )
-        getline( in, line, '\r' );
-#else
 	getline( in, line, '\n' );
-#endif
 
         // catch extraneous (DOS) line ending character
         int i;
@@ -1703,21 +1728,21 @@ fgUsage (bool verbose)
                     tmp.append(", -");
                     tmp.append(short_name->getStringValue());
                 }
-
-                char cstr[96];
+				
                 if (tmp.size() <= 25) {
-                    snprintf(cstr, 96, "   --%-27s", tmp.c_str());
+                    msg+= "   --";
+                    msg += tmp;
+                    msg.append( 27-tmp.size(), ' ');
                 } else {
-                    snprintf(cstr, 96, "\n   --%s\n%32c", tmp.c_str(), ' ');
+                    msg += "\n   --";
+                    msg += tmp + '\n';
+                    msg.append(32, ' ');
                 }
-
                 // There may be more than one <description> tag assosiated
                 // with one option
 
-                msg += cstr;
-                vector<SGPropertyNode_ptr>desc =
-                                          option[k]->getChildren("description");
-
+                vector<SGPropertyNode_ptr> desc;
+                desc = option[k]->getChildren("description");
                 if (desc.size() > 0) {
                    for ( unsigned int l = 0; l < desc.size(); l++) {
 
@@ -1732,9 +1757,7 @@ fgUsage (bool verbose)
                          string t_str = trans_desc[m]->getStringValue();
 
                          if ((m > 0) || ((l > 0) && m == 0)) {
-                            snprintf(cstr, 96, "%32c", ' ');
-                            msg += cstr;
-
+                            msg.append( 32, ' ');
                          }
 
                          // If the string is too large to fit on the screen,
@@ -1743,9 +1766,8 @@ fgUsage (bool verbose)
                          while ( t_str.size() > 47 ) {
 
                             unsigned int m = t_str.rfind(' ', 47);
-                            msg += t_str.substr(0, m);
-                            snprintf(cstr, 96, "\n%32c", ' ');
-                            msg += cstr;
+                            msg += t_str.substr(0, m) + '\n';
+                            msg.append( 32, ' ');
 
                             t_str.erase(t_str.begin(), t_str.begin() + m + 1);
                         }
@@ -1756,8 +1778,8 @@ fgUsage (bool verbose)
             }
         }
 
-        SGPropertyNode *name =
-                            locale->getNode(section[j]->getStringValue("name"));
+        SGPropertyNode *name;
+        name = locale->getNode(section[j]->getStringValue("name"));
 
         if (!msg.empty() && name) {
            cout << endl << name->getStringValue() << ":" << endl;
@@ -1840,25 +1862,27 @@ static void fgSearchAircraft(const SGPath &path, string_list &aircraft,
 	     	status = node->getNode("status");
           }
 
-          char cstr[96];
-	  //additionally display status information where it is available
-          
-          if (strlen(dire->d_name) <= 27) {
-             snprintf(cstr, 96, "   %-27s  %s", dire->d_name,
-                      (desc) ? desc->getStringValue() : "");
+          //additionally display status information where it is available
 
-          } else {
-             snprintf(cstr, 96, "   %-27s\n%32c%s", dire->d_name, ' ',
-                      (desc) ? desc->getStringValue() : "");
+          string descStr("   ");
+          descStr += dire->d_name;
+          if (desc) {
+              if (descStr.size() <= 27+3) {
+                descStr.append(29+3-descStr.size(), ' ');
+              } else {
+                descStr += '\n';
+                descStr.append( 32, ' ');
+              }
+              descStr += desc->getStringValue();
           }
-
-	  SGPropertyNode * required_status
+  
+          SGPropertyNode * required_status
                              = fgGetNode ("/sim/aircraft-min-status", true);
 	  
 	  // If the node holds the value "all", then there wasn't any status 
 	  // level specified, so we simply go ahead and output ALL aircraft
 	  if (strcmp(required_status->getStringValue(),"all")==0) {	   
-		  aircraft.push_back(cstr);
+		  aircraft.push_back(descStr);
 		  }
 	  else
 	  {
@@ -1870,7 +1894,7 @@ static void fgSearchAircraft(const SGPath &path, string_list &aircraft,
      	  //Compare (minimally) required status level with actual aircraft status:
 	   if (	getNumMaturity(status->getStringValue() ) >= 
 	  	getNumMaturity(required_status->getStringValue() ) )
-				  aircraft.push_back(cstr); }
+				  aircraft.push_back(descStr); }
 	  			  		  
 	  }
       
