@@ -9,13 +9,14 @@
 
 #include <simgear/compiler.h>
 
+#include <algorithm>
+#include <functional>
 #include <vector>
 
-#include <plib/ssg.h>
+#include <osg/Math>
 
 #include <simgear/scene/model/placement.hxx>
 #include <simgear/scene/model/modellib.hxx>
-#include <simgear/scene/model/shadowvolume.hxx>
 #include <simgear/structure/exception.hxx>
 
 #include <Main/fg_props.hxx>
@@ -24,10 +25,12 @@
 
 #include "modelmgr.hxx"
 
-SG_USING_STD(vector);
+using std::vector;
 
-extern SGShadowVolume *shadows;
+using namespace simgear;
 
+// OSGFIXME
+// extern SGShadowVolume *shadows;
 
 FGModelMgr::FGModelMgr ()
   : _models(fgGetNode("/models", true)),
@@ -43,7 +46,7 @@ FGModelMgr::~FGModelMgr ()
 
   for (unsigned int i = 0; i < _instances.size(); i++) {
     globals->get_scenery()->get_scene_graph()
-      ->removeKid(_instances[i]->model->getSceneGraph());
+      ->removeChild(_instances[i]->model->getSceneGraph());
     delete _instances[i];
   }
 }
@@ -66,17 +69,12 @@ FGModelMgr::add_model (SGPropertyNode * node)
   SGModelPlacement *model = new SGModelPlacement;
   instance->model = model;
   instance->node = node;
-  SGModelLib *model_lib = globals->get_model_lib();
 
   const char *path = node->getStringValue("path", "Models/Geometry/glider.ac");
-  ssgBranch *object;
+  osg::Node *object;
 
   try {
-    object = (ssgBranch *)model_lib->load_model(
-        globals->get_fg_root(),
-        path,
-        globals->get_props(),
-        globals->get_sim_time_sec(), /*cache_object=*/false);
+    object = SGModelLib::loadPagedModel(path, globals->get_props());
   } catch (const sg_throwable& t) {
     SG_LOG(SG_GENERAL, SG_ALERT, "Error loading " << path << ":\n  "
         << t.getFormattedMessage() << t.getOrigin());
@@ -125,10 +123,7 @@ FGModelMgr::add_model (SGPropertyNode * node)
     model->setHeadingDeg(node->getDoubleValue("heading-deg"));
 
       			// Add this model to the global scene graph
-  globals->get_scenery()->get_scene_graph()->addKid(model->getSceneGraph());
-
-  // Register that one at the scenery manager
-  globals->get_scenery()->register_placement_transform(model->getTransform());
+  globals->get_scenery()->get_scene_graph()->addChild(model->getSceneGraph());
 
 
       			// Save this instance for updating
@@ -145,37 +140,70 @@ FGModelMgr::unbind ()
 {
 }
 
+namespace
+{
+double testNan(double val) throw (sg_range_exception)
+{
+    if (osg::isNaN(val))
+        throw sg_range_exception("value is nan");
+    return val;
+}
+
+struct UpdateFunctor : public std::unary_function<FGModelMgr::Instance*, void>
+{
+    void operator()(FGModelMgr::Instance* instance) const
+    {
+        SGModelPlacement* model = instance->model;
+        double lon, lat, elev, roll, pitch, heading;
+
+        try {
+            // Optionally set position from properties
+            if (instance->lon_deg_node != 0)
+                lon = testNan(instance->lon_deg_node->getDoubleValue());
+            if (instance->lat_deg_node != 0)
+                lat = testNan(instance->lat_deg_node->getDoubleValue());
+            if (instance->elev_ft_node != 0)
+                elev = testNan(instance->elev_ft_node->getDoubleValue());
+
+            // Optionally set orientation from properties
+            if (instance->roll_deg_node != 0)
+                roll = testNan(instance->roll_deg_node->getDoubleValue());
+            if (instance->pitch_deg_node != 0)
+                pitch = testNan(instance->pitch_deg_node->getDoubleValue());
+            if (instance->heading_deg_node != 0)
+                heading = testNan(instance->heading_deg_node->getDoubleValue());
+        } catch (const sg_range_exception& e) {
+            const char *path = instance->node->getStringValue("path",
+                                                              "unknown");
+            SG_LOG(SG_GENERAL, SG_INFO, "Instance of model " << path
+                   << " has invalid values");
+            return;
+        }
+        // Optionally set position from properties
+        if (instance->lon_deg_node != 0)
+            model->setLongitudeDeg(lon);
+        if (instance->lat_deg_node != 0)
+            model->setLatitudeDeg(lat);
+        if (instance->elev_ft_node != 0)
+            model->setElevationFt(elev);
+
+        // Optionally set orientation from properties
+        if (instance->roll_deg_node != 0)
+            model->setRollDeg(roll);
+        if (instance->pitch_deg_node != 0)
+            model->setPitchDeg(pitch);
+        if (instance->heading_deg_node != 0)
+            model->setHeadingDeg(heading);
+
+        instance->model->update();
+    }
+};
+}
+
 void
 FGModelMgr::update (double dt)
 {
-  for (unsigned int i = 0; i < _instances.size(); i++) {
-    Instance * instance = _instances[i];
-    SGModelPlacement * model = instance->model;
-
-				// Optionally set position from properties
-    if (instance->lon_deg_node != 0)
-      model->setLongitudeDeg(instance->lon_deg_node->getDoubleValue());
-    if (instance->lat_deg_node != 0)
-      model->setLatitudeDeg(instance->lat_deg_node->getDoubleValue());
-    if (instance->elev_ft_node != 0)
-      model->setElevationFt(instance->elev_ft_node->getDoubleValue());
-
-				// Optionally set orientation from properties
-    if (instance->roll_deg_node != 0)
-      model->setRollDeg(instance->roll_deg_node->getDoubleValue());
-    if (instance->pitch_deg_node != 0)
-      model->setPitchDeg(instance->pitch_deg_node->getDoubleValue());
-    if (instance->heading_deg_node != 0)
-      model->setHeadingDeg(instance->heading_deg_node->getDoubleValue());
-
-    instance->model->update();
-
-    if (shadows && !instance->shadow) {
-      ssgBranch *branch = (ssgBranch *)instance->model->getSceneGraph();
-      shadows->addOccluder(branch, SGShadowVolume::occluderTypeTileObject);
-      instance->shadow = true;
-    }
-  }
+    std::for_each(_instances.begin(), _instances.end(), UpdateFunctor());
 }
 
 void
@@ -197,14 +225,6 @@ FGModelMgr::remove_instance (Instance * instance)
     }
 }
 
-void
-FGModelMgr::draw ()
-{
-//   ssgSetNearFar(_nearplane, _farplane);
-//   ssgCullAndDraw(_scene);
-}
-
-
 
 ////////////////////////////////////////////////////////////////////////
 // Implementation of FGModelMgr::Instance
@@ -225,9 +245,6 @@ FGModelMgr::Instance::Instance ()
 
 FGModelMgr::Instance::~Instance ()
 {
-  // Unregister that one at the scenery manager
-  globals->get_scenery()->unregister_placement_transform(model->getTransform());
-
   delete model;
 }
 
@@ -262,10 +279,11 @@ FGModelMgr::Listener::childRemoved(SGPropertyNode * parent, SGPropertyNode * chi
       continue;
 
     _mgr->_instances.erase(it);
-    ssgBranch *branch = (ssgBranch *)instance->model->getSceneGraph();
-    if (shadows && instance->shadow)
-        shadows->deleteOccluder(branch);
-    globals->get_scenery()->get_scene_graph()->removeKid(branch);
+    osg::Node *branch = instance->model->getSceneGraph();
+    // OSGFIXME
+//     if (shadows && instance->shadow)
+//         shadows->deleteOccluder(branch);
+    globals->get_scenery()->get_scene_graph()->removeChild(branch);
 
     delete instance;
     break;
