@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: ATC-Inputs.cxx,v 1.14 2006-07-05 09:39:46 mfranz Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
@@ -38,16 +38,17 @@
 #include <errno.h>
 #include <math.h>
 
-#include STL_STRING
+#include <string>
 
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/props/props_io.hxx>
 
 #include <Main/fg_props.hxx>
 
 #include "ATC-Inputs.hxx"
 
-SG_USING_STD(string);
+using std::string;
 
 
 
