@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: atlas.cxx,v 1.12.2.1 2007-12-13 20:10:08 durk Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -69,15 +69,11 @@ bool FGAtlas::gen_message() {
     // cout << "generating atlas message" << endl;
 
     static SGPropertyNode *adf_freq
-        = fgGetNode("/instrumentation/kr-87/outputs/selected-khz", true);
-    static SGPropertyNode *nav1_freq
+        = fgGetNode("/instrumentation/adf/outputs/selected-khz", true);
+    static SGPropertyNode *nav_freq
         = fgGetNode("/instrumentation/nav/frequencies/selected-mhz", true);
-    static SGPropertyNode *nav1_sel_radial
+    static SGPropertyNode *nav_sel_radial
         = fgGetNode("/instrumentation/nav/radials/selected-deg", true);
-    static SGPropertyNode *nav2_freq
-        = fgGetNode("/instrumentation/nav[1]/frequencies/selected-mhz", true);
-    static SGPropertyNode *nav2_sel_radial
-        = fgGetNode("/instrumentation/nav[1]/radials/selected-deg", true);
 
     char rmc[256], gga[256], patla[256];
     char rmc_sum[10], gga_sum[10], patla_sum[10];
@@ -142,10 +138,10 @@ bool FGAtlas::gen_message() {
     sprintf( gga_sum, "%02X", calc_atlas_cksum(gga) );
 
     sprintf( patla, "PATLA,%.2f,%.1f,%.2f,%.1f,%.0f",
-	     nav1_freq->getDoubleValue(),
-	     nav1_sel_radial->getDoubleValue(),
-	     nav2_freq->getDoubleValue(),
-	     nav2_sel_radial->getDoubleValue(),
+	     nav_freq->getDoubleValue(),
+	     nav_sel_radial->getDoubleValue(),
+	     nav_freq->getDoubleValue(),
+	     nav_sel_radial->getDoubleValue(),
 	     adf_freq->getDoubleValue() );
     sprintf( patla_sum, "%02X", calc_atlas_cksum(patla) );
 
