@@ -18,12 +18,14 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: garmin.cxx,v 1.7 2006-02-21 01:19:47 mfranz Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
+#include <iostream>
+
 #include <simgear/debug/logstream.hxx>
 #include <simgear/math/sg_geodesy.hxx>
 #include <simgear/io/iochannel.hxx>
@@ -35,7 +37,7 @@
 
 #include "garmin.hxx"
 
-SG_USING_NAMESPACE(std);
+using std::string;
 
 FGGarmin::FGGarmin() {
 }
@@ -167,7 +169,7 @@ bool FGGarmin::gen_message() {
     garmin_sentence += gsa;
     garmin_sentence += "\r\n";
 
-    cout << garmin_sentence;
+    std::cout << garmin_sentence;
 
     length = garmin_sentence.length();
     strncpy( buf, garmin_sentence.c_str(), length );
