@@ -28,7 +28,7 @@
 
 #include <simgear/compiler.h>
 
-#include STL_STRING
+#include <string>
 
 #include <iostream>
 #include <map>
@@ -42,11 +42,11 @@
 
 #include "multiplay.hxx"
 
-SG_USING_STD(string);
+using std::string;
 
 
 // These constants are provided so that the ident command can list file versions.
-const char sFG_MULTIPLAY_BID[] = "$Id: multiplay.cxx,v 1.15 2006-10-10 05:17:07 frohlich Exp $";
+const char sFG_MULTIPLAY_BID[] = "$Id$";
 const char sFG_MULTIPLAY_HID[] = FG_MULTIPLAY_HID;
 
 
@@ -101,13 +101,12 @@ bool FGMultiplay::open() {
     SGPropertyNode* root = globals->get_props();
 
     /// Build up the id to property map
-    unsigned i = 0;
-    while (FGMultiplayMgr::sIdPropertyList[i].name) {
+    
+    for (unsigned i = 0; i < FGMultiplayMgr::numProperties; ++i) {
       const char* name = FGMultiplayMgr::sIdPropertyList[i].name;
       SGPropertyNode* pNode = root->getNode(name);
       if (pNode)
         mPropertyMap[FGMultiplayMgr::sIdPropertyList[i].id] = pNode;
-      ++i;
     }
 
     return is_enabled();
