@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: native_ctrls.cxx,v 1.33 2006-02-21 01:19:47 mfranz Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
@@ -36,8 +36,10 @@
 #include "native_ctrls.hxx"
 
 // FreeBSD works better with this included last ... (?)
-#if defined(WIN32) && !defined(__CYGWIN__)
+#if defined( _MSC_VER )
 #  include <windows.h>
+#elif defined( __MINGW32__ )
+#  include <Winsock2.h>
 #else
 #  include <netinet/in.h>	// htonl() ntohl()
 #endif
@@ -293,7 +295,7 @@ void FGNetCtrls2Props( FGNetCtrls *net, bool honor_freezes,
     int i;
 
     SGPropertyNode * node;
-/***************
+
     if ( net_byte_order ) {
         // convert from network byte order
         net->version = htonl(net->version);
@@ -350,7 +352,7 @@ void FGNetCtrls2Props( FGNetCtrls *net, bool honor_freezes,
         net->speedup = htonl(net->speedup);
         net->freeze = htonl(net->freeze);
     }
-*************/
+
     if ( net->version != FG_NET_CTRLS_VERSION ) {
 	SG_LOG( SG_IO, SG_ALERT,
                 "Version mismatch with raw controls packet format." );
