@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: native_fdm.cxx,v 1.31.2.1 2007-11-07 17:43:39 mfranz Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
@@ -39,8 +39,10 @@
 #include "native_fdm.hxx"
 
 // FreeBSD works better with this included last ... (?)
-#if defined(WIN32) && !defined(__CYGWIN__)
+#if defined( _MSC_VER )
 #  include <windows.h>
+#elif defined( __MINGW32__ )
+#  include <Winsock2.h>
 #else
 #  include <netinet/in.h>	// htonl() ntohl()
 #endif
