@@ -28,6 +28,10 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <simgear/debug/logstream.hxx>
 #include <simgear/io/iochannel.hxx>
 
@@ -38,7 +42,7 @@
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 
-SG_USING_STD(vector);
+using std::vector;
 
 FGOpenGC::FGOpenGC() : 
 	press_node(fgGetNode("/environment/pressure-inhg[0]", true)),
