@@ -18,14 +18,17 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: tilemgr.cxx,v 1.56.2.1 2007-06-24 08:30:56 mfranz Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#include <plib/ssg.h>
+#include <algorithm>
+#include <functional>
+
+#include <osgViewer/Viewer>
 
 #include <simgear/constants.h>
 #include <simgear/debug/logstream.hxx>
@@ -35,43 +38,35 @@
 #include <simgear/math/vector.hxx>
 #include <simgear/structure/exception.hxx>
 #include <simgear/scene/model/modellib.hxx>
-#include <simgear/scene/model/shadowvolume.hxx>
+#include <simgear/scene/tgdb/SGReaderWriterBTGOptions.hxx>
 
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/renderer.hxx>
 #include <Main/viewer.hxx>
 #include <Scripting/NasalSys.hxx>
 
-#include "newcache.hxx"
 #include "scenery.hxx"
+#include "SceneryPager.hxx"
 #include "tilemgr.hxx"
 
-#define TEST_LAST_HIT_CACHE
+using std::for_each;
+using flightgear::SceneryPager;
+using simgear::SGModelLib;
+using simgear::TileEntry;
+using simgear::TileCache;
 
-#if defined(ENABLE_THREADS)
-SGLockedQueue<FGTileEntry *> FGTileMgr::attach_queue;
-SGLockedQueue<FGDeferredModel *> FGTileMgr::model_queue;
-#else
-queue<FGTileEntry *> FGTileMgr::attach_queue;
-queue<FGDeferredModel *> FGTileMgr::model_queue;
-#endif // ENABLE_THREADS
-queue<FGTileEntry *> FGTileMgr::delete_queue;
-
-bool FGTileMgr::tile_filter = true;
-
-extern SGShadowVolume *shadows;
-
-// Constructor
 FGTileMgr::FGTileMgr():
     state( Start ),
-    current_tile( NULL ),
     vis( 16000 )
 {
 }
 
 
-// Destructor
 FGTileMgr::~FGTileMgr() {
+    // remove all nodes we might have left behind
+    osg::Group* group = globals->get_scenery()->get_terrain_branch();
+    group->removeChildren(0, group->getNumChildren());
 }
 
 
@@ -79,31 +74,19 @@ FGTileMgr::~FGTileMgr() {
 int FGTileMgr::init() {
     SG_LOG( SG_TERRAIN, SG_INFO, "Initializing Tile Manager subsystem." );
 
+    _options = new SGReaderWriterBTGOptions;
+    _options->setMatlib(globals->get_matlib());
+    _options->setUseRandomObjects(fgGetBool("/sim/rendering/random-objects", true));
+    _options->setUseRandomVegetation(fgGetBool("/sim/rendering/random-vegetation", true));
+    osgDB::FilePathList &fp = _options->getDatabasePathList();
+    const string_list &sc = globals->get_fg_scenery();
+    fp.clear();
+    std::copy(sc.begin(), sc.end(), back_inserter(fp));
+
+    TileEntry::setModelLoadHelper(this);
+
     tile_cache.init();
 
-#if 0
-
-    // instead it's just a lot easier to let any pending work flush
-    // through, rather than trying to arrest the queue and nuke all
-    // the various work at all the various stages and get everything
-    // cleaned up properly.
-
-    while ( ! attach_queue.empty() ) {
-        attach_queue.pop();
-    }
-
-    while ( ! model_queue.empty() ) {
-#if defined(ENABLE_THREADS)
-        FGDeferredModel* dm = model_queue.pop();
-#else
-        FGDeferredModel* dm = model_queue.front();
-        model_queue.pop();
-#endif
-        delete dm;
-    }
-    loader.reinit();
-#endif
-
     state = Inited;
 
     previous_bucket.make_bad();
@@ -114,22 +97,25 @@ int FGTileMgr::init() {
     return 1;
 }
 
-
 // schedule a tile for loading
 void FGTileMgr::sched_tile( const SGBucket& b, const bool is_inner_ring ) {
     // see if tile already exists in the cache
-    FGTileEntry *t = tile_cache.get_tile( b );
+    TileEntry *t = tile_cache.get_tile( b );
 
-    if ( t == NULL ) {
+    if ( !t ) {
         // make space in the cache
+        SceneryPager* pager = FGScenery::getPagerSingleton();
         while ( (int)tile_cache.get_size() > tile_cache.get_max_cache_size() ) {
             long index = tile_cache.get_oldest_tile();
             if ( index >= 0 ) {
-                FGTileEntry *old = tile_cache.get_tile( index );
-                shadows->deleteOccluderFromTile( (ssgBranch *) old->get_terra_transform() );
-                old->disconnect_ssg_nodes();
-                delete_queue.push( old );
+                TileEntry *old = tile_cache.get_tile( index );
                 tile_cache.clear_entry( index );
+                osg::ref_ptr<osg::Object> subgraph = old->getNode();
+                old->removeFromSceneGraph();
+                delete old;
+                // zeros out subgraph ref_ptr, so subgraph is owned by
+                // the pager and will be deleted in the pager thread.
+                pager->queueDeleteRequest(subgraph);
             } else {
                 // nothing to free ?!? forge ahead
                 break;
@@ -137,17 +123,18 @@ void FGTileMgr::sched_tile( const SGBucket& b, const bool is_inner_ring ) {
         }
 
         // create a new entry
-        FGTileEntry *e = new FGTileEntry( b );
+        TileEntry *e = new TileEntry( b );
 
         // insert the tile into the cache
         if ( tile_cache.insert_tile( e ) ) {
-            // Schedule tile for loading
-            loader.add( e );
+            // update_queues will generate load request
         } else {
             // insert failed (cache full with no available entries to
             // delete.)  Try again later
             delete e;
         }
+        // Attach to scene graph
+        e->addToSceneGraph(globals->get_scenery()->get_terrain_branch());
     } else {
         t->set_inner_ring( is_inner_ring );
     }
@@ -184,7 +171,8 @@ void FGTileMgr::schedule_needed( double vis, const SGBucket& curr_bucket) {
     if ( xrange < 1 ) { xrange = 1; }
     if ( yrange < 1 ) { yrange = 1; }
 
-    // note * 2 at end doubles cache size (for fdm and viewer)
+    // make the cache twice as large to avoid losing terrain when switching
+    // between aircraft and tower views
     tile_cache.set_max_cache_size( (2*xrange + 2) * (2*yrange + 2) * 2 );
     // cout << "xrange = " << xrange << "  yrange = " << yrange << endl;
     // cout << "max cache size = " << tile_cache.get_max_cache_size()
@@ -246,34 +234,65 @@ void FGTileMgr::initialize_queue()
 
     double visibility_meters = fgGetDouble("/environment/visibility-m");
     schedule_needed(visibility_meters, current_bucket);
+}
 
-    // do we really want to lose this? CLO
-#if 0
-    // Now force a load of the center tile and inner ring so we
-    // have something to see in our first frame.
-    int i;
-    for ( i = 0; i < 9; ++i ) {
-        if ( load_queue.size() ) {
-            SG_LOG( SG_TERRAIN, SG_DEBUG, 
-                    "Load queue not empty, loading a tile" );
+osg::Node*
+FGTileMgr::loadTileModel(const string& modelPath, bool cacheModel)
+{
+    osg::Node* result = 0;
+    try {
+        if(cacheModel)
+            result =
+                SGModelLib::loadModel(modelPath, globals->get_props(),
+                                      new FGNasalModelData);
 
-            SGBucket pending = load_queue.front();
-            load_queue.pop_front();
-            load_tile( pending );
+        else
+            result=
+                SGModelLib::loadPagedModel(modelPath, globals->get_props(),
+                                           new FGNasalModelData);
+    } catch (const sg_io_exception& exc) {
+        string m(exc.getMessage());
+        m += " ";
+        m += exc.getLocation().asString();
+        SG_LOG( SG_ALL, SG_ALERT, m );
+    } catch (const sg_exception& exc) { // XXX may be redundant
+        SG_LOG( SG_ALL, SG_ALERT, exc.getMessage());
+    }
+    return result;
+}
+
+// Helper class for STL fun
+class TileLoad : public std::unary_function<TileCache::tile_map::value_type,
+                                            void>
+{
+public:
+    TileLoad(SceneryPager *pager, osg::FrameStamp* framestamp,
+             osg::Group* terrainBranch, osgDB::ReaderWriter::Options* options) :
+        _pager(pager), _framestamp(framestamp), _options(options) {}
+
+    TileLoad(const TileLoad& rhs) :
+        _pager(rhs._pager), _framestamp(rhs._framestamp),
+        _options(rhs._options) {}
+
+    void operator()(TileCache::tile_map::value_type& tilePair)
+    {
+        TileEntry* entry = tilePair.second;
+        if (entry->getNode()->getNumChildren() == 0) {
+            _pager->queueRequest(entry->tileFileName,
+                                 entry->getNode(),
+                                 entry->get_inner_ring() ? 10.0f : 1.0f,
+                                 _framestamp,
+#ifdef FGOSGPAGER25
+                                 entry->getDatabaseRequest(),
+#endif
+                                 _options);
         }
     }
-#endif
-}
-
-/**
- * return current status of queues
- *
- */
-
-bool FGTileMgr::all_queues_empty() {
-	return attach_queue.empty() && model_queue.empty();
-}
-
+private:
+    SceneryPager* _pager;
+    osg::FrameStamp* _framestamp;
+    osgDB::ReaderWriter::Options* _options;
+};
 
 /**
  * Update the various queues maintained by the tilemagr (private
@@ -281,107 +300,14 @@ bool FGTileMgr::all_queues_empty() {
  */
 void FGTileMgr::update_queues()
 {
-    // load the next model in the load queue.  Currently this must
-    // happen in the render thread because model loading can trigger
-    // texture loading which involves use of the opengl api.  Skip any
-    // models belonging to not loaded tiles (i.e. the tile was removed
-    // before we were able to load some of the associated models.)
-    if ( !model_queue.empty() ) {
-        bool processed_one = false;
-
-        while ( model_queue.size() > 200 || processed_one == false ) {
-            processed_one = true;
-
-            if ( model_queue.size() > 200 ) {
-                SG_LOG( SG_TERRAIN, SG_INFO,
-                        "Alert: catching up on model load queue" );
-            }
-
-            // cout << "loading next model ..." << endl;
-            // load the next tile in the queue
-#if defined(ENABLE_THREADS)
-            FGDeferredModel* dm = model_queue.pop();
-#else
-            FGDeferredModel* dm = model_queue.front();
-            model_queue.pop();
-#endif
-
-            // only load the model if the tile still exists in the
-            // tile cache
-            FGTileEntry *t = tile_cache.get_tile( dm->get_bucket() );
-            if ( t != NULL ) {
-                ssgTexturePath( (char *)(dm->get_texture_path().c_str()) );
-                try {
-                    ssgEntity *obj_model =
-                        globals->get_model_lib()->load_model( ".",
-                                                  dm->get_model_path(),
-                                                  globals->get_props(),
-                                                  globals->get_sim_time_sec(),
-                                                  dm->get_cache_state(),
-                                                  new FGNasalModelData );
-                    if ( obj_model != NULL ) {
-                        dm->get_obj_trans()->addKid( obj_model );
-                        shadows->addOccluder( (ssgBranch *) obj_model->getParent(0),
-                            SGShadowVolume::occluderTypeTileObject,
-                            (ssgBranch *) dm->get_tile()->get_terra_transform());
-                    }
-                } catch (const sg_io_exception& exc) {
-					string m(exc.getMessage());
-					m += " ";
-					m += exc.getLocation().asString();
-                    SG_LOG( SG_ALL, SG_ALERT, m );
-                } catch (const sg_exception& exc) { // XXX may be redundant
-                    SG_LOG( SG_ALL, SG_ALERT, exc.getMessage());
-                }
-                
-                dm->get_tile()->dec_pending_models();
-            }
-            delete dm;
-        }
-    }
-    
-    // Notify the tile loader that it can load another tile
-    loader.update();
-
-    if ( !attach_queue.empty() ) {
-#if defined(ENABLE_THREADS)
-        FGTileEntry* e = attach_queue.pop();
-#else
-        FGTileEntry* e = attach_queue.front();
-        attach_queue.pop();
-#endif
-        e->add_ssg_nodes( globals->get_scenery()->get_terrain_branch(),
-                          globals->get_scenery()->get_gnd_lights_root(),
-                          globals->get_scenery()->get_vasi_lights_root(),
-                          globals->get_scenery()->get_rwy_lights_root(),
-                          globals->get_scenery()->get_taxi_lights_root() );
-        // cout << "Adding ssg nodes for "
-    }
-
-    if ( !delete_queue.empty() ) {
-        // cout << "delete queue = " << delete_queue.size() << endl;
-        bool processed_one = false;
-
-        while ( delete_queue.size() > 30 || processed_one == false ) {
-            processed_one = true;
-
-            if ( delete_queue.size() > 30 ) {
-                // uh oh, delete queue is blowing up, we aren't clearing
-                // it fast enough.  Let's just panic, well not panic, but
-                // get real serious and agressively free up some tiles so
-                // we don't explode our memory usage.
-
-                SG_LOG( SG_TERRAIN, SG_WARN,
-                        "Warning: catching up on tile delete queue" );
-            }
-
-            FGTileEntry* e = delete_queue.front();
-            if ( e->free_tile() ) {
-                delete_queue.pop();
-                delete e;
-            }
-        }
-    }
+    SceneryPager* pager = FGScenery::getPagerSingleton();
+    osg::FrameStamp* framestamp
+        = globals->get_renderer()->getViewer()->getFrameStamp();
+    tile_cache.set_current_time(framestamp->getReferenceTime());
+    for_each(tile_cache.begin(), tile_cache.end(),
+             TileLoad(pager,
+                      framestamp,
+                      globals->get_scenery()->get_terrain_branch(), _options.get()));
 }
 
 
@@ -422,7 +348,7 @@ int FGTileMgr::update( SGLocation *location, double visibility_meters )
     // Note that we need keep track of both viewer buckets and fdm buckets.
     if ( state == Running ) {
         SG_LOG( SG_TERRAIN, SG_DEBUG, "State == Running" );
-        if (!(current_bucket == previous_bucket )) {
+        if (current_bucket != previous_bucket) {
             // We've moved to a new bucket, we need to schedule any
             // needed tiles for loading.
             SG_LOG( SG_TERRAIN, SG_INFO, "FGTileMgr::update()" );
@@ -432,10 +358,11 @@ int FGTileMgr::update( SGLocation *location, double visibility_meters )
         SG_LOG( SG_TERRAIN, SG_INFO, "State == Start || Inited" );
 //        initialize_queue();
         state = Running;
-
-        // load the next tile in the load queue (or authorize the next
-        // load in the case of the threaded tile pager)
-        loader.update();
+        if (current_bucket != previous_bucket
+            && current_bucket.get_chunk_lon() != -1000) {
+               SG_LOG( SG_TERRAIN, SG_INFO, "FGTileMgr::update()" );
+               schedule_needed(visibility_meters, current_bucket);
+        }
     }
 
     update_queues();
@@ -446,35 +373,18 @@ int FGTileMgr::update( SGLocation *location, double visibility_meters )
     return 1;
 }
 
-
-// timer event driven call to scheduler for the purpose of refreshing the tile timestamps
-void FGTileMgr::refresh_view_timestamps() {
-    SG_LOG( SG_TERRAIN, SG_INFO,
-            "Refreshing timestamps for " << current_bucket.get_center_lon()
-            << " " << current_bucket.get_center_lat() );
-    if ( longitude >= -180.0 && longitude <= 180.0 
-         && latitude >= -90.0 && latitude <= 90.0 )
-    {
-        schedule_needed(fgGetDouble("/environment/visibility-m"), current_bucket);
-    }
-}
-
-void FGTileMgr::prep_ssg_nodes( SGLocation *location, float vis ) {
+void FGTileMgr::prep_ssg_nodes(float vis) {
 
     // traverse the potentially viewable tile list and update range
     // selector and transform
 
-    float *up = location->get_world_up();
-
-    FGTileEntry *e;
+    TileEntry *e;
     tile_cache.reset_traversal();
 
-    const double *vp = location->get_absolute_view_pos();
-    Point3D viewpos(vp[0], vp[1], vp[2]);
     while ( ! tile_cache.at_end() ) {
         // cout << "processing a tile" << endl;
         if ( (e = tile_cache.get_current()) ) {
-            e->prep_ssg_node( viewpos, up, vis);
+            e->prep_ssg_node(vis);
         } else {
             SG_LOG(SG_INPUT, SG_ALERT, "warning ... empty tile in cache");
         }
@@ -482,17 +392,6 @@ void FGTileMgr::prep_ssg_nodes( SGLocation *location, float vis ) {
     }
 }
 
-bool FGTileMgr::set_tile_filter( bool f ) {
-    bool old = tile_filter;
-    tile_filter = f;
-    return old;
-}
-
-int FGTileMgr::tile_filter_cb( ssgEntity *, int )
-{
-  return tile_filter ? 1 : 0;
-}
-
 bool FGTileMgr::scenery_available(double lat, double lon, double range_m)
 {
   // sanity check (unfortunately needed!)
@@ -500,7 +399,7 @@ bool FGTileMgr::scenery_available(double lat, double lon, double range_m)
     return false;
   
   SGBucket bucket(lon, lat);
-  FGTileEntry *te = tile_cache.get_tile(bucket);
+  TileEntry *te = tile_cache.get_tile(bucket);
   if (!te || !te->is_loaded())
     return false;
 
@@ -517,7 +416,7 @@ bool FGTileMgr::scenery_available(double lat, double lon, double range_m)
       // We have already checked for the center tile.
       if ( x != 0 || y != 0 ) {
         SGBucket b = sgBucketOffset( lon, lat, x, y );
-        FGTileEntry *te = tile_cache.get_tile(b);
+        TileEntry *te = tile_cache.get_tile(b);
         if (!te || !te->is_loaded())
           return false;
       }
@@ -527,3 +426,22 @@ bool FGTileMgr::scenery_available(double lat, double lon, double range_m)
   // Survived all tests.
   return true;
 }
+
+namespace
+{
+struct IsTileLoaded :
+        public std::unary_function<TileCache::tile_map::value_type, bool>
+{
+    bool operator()(const TileCache::tile_map::value_type& tilePair) const
+    {
+        return tilePair.second->is_loaded();
+    }
+};
+}
+
+bool FGTileMgr::isSceneryLoaded()
+{
+    return (std::find_if(tile_cache.begin(), tile_cache.end(),
+                         std::not1(IsTileLoaded()))
+            == tile_cache.end());
+}
