@@ -23,11 +23,13 @@
 #include <simgear/scene/material/mat.hxx>
 #include <simgear/structure/commands.hxx>
 #include <simgear/math/sg_geodesy.hxx>
+#include <simgear/structure/event_mgr.hxx>
 
 #include <Airports/runways.hxx>
 #include <Airports/simple.hxx>
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/util.hxx>
 #include <Scenery/scenery.hxx>
 
 #include "NasalSys.hxx"
@@ -122,7 +124,7 @@ FGNasalScript* FGNasalSys::parseScript(const char* src, const char* name)
 
     char buf[256];
     if(!name) {
-        snprintf(buf, 256, "FGNasalScript@%p", (void *)script);
+        sprintf(buf, "FGNasalScript@%p", (void *)script);
         name = buf;
     }
 
@@ -328,6 +330,8 @@ static naRef f_interpolate(naContext c, naRef me, int argc, naRef* args)
         ->get_group(SGSubsystemMgr::INIT)->get_subsystem("interpolator"))
         ->interpolate(node, nPoints, values, deltas);
 
+    delete[] values;
+    delete[] deltas;
     return naNil();
 }
 
@@ -345,6 +349,12 @@ static naRef f_srand(naContext c, naRef me, int argc, naRef* args)
     return naNum(0);
 }
 
+static naRef f_abort(naContext c, naRef me, int argc, naRef* args)
+{
+    abort();
+    return naNil();
+}
+
 // Return an array listing of all files in a directory
 static naRef f_directory(naContext c, naRef me, int argc, naRef* args)
 {
@@ -372,7 +382,7 @@ static naRef f_directory(naContext c, naRef me, int argc, naRef* args)
 // <pi>        ... callback function with two args: target, data
 //                 (pi = "processing instruction")
 // All four callback functions are optional and default to nil.
-// The function returns nil on error, and the file name otherwise.
+// The function returns nil on error, or the validated file name otherwise.
 static naRef f_parsexml(naContext c, naRef me, int argc, naRef* args)
 {
     if(argc < 1 || !naIsString(args[0]))
@@ -382,7 +392,12 @@ static naRef f_parsexml(naContext c, naRef me, int argc, naRef* args)
         if(!(naIsNil(args[i]) || naIsFunc(args[i])))
             naRuntimeError(c, "parsexml(): callback argument not a function");
 
-    const char* file = naStr_data(args[0]);
+    const char* file = fgValidatePath(naStr_data(args[0]), false);
+    if(!file) {
+        naRuntimeError(c, "parsexml(): reading '%s' denied "
+                "(unauthorized access)", naStr_data(args[0]));
+        return naNil();
+    }
     std::ifstream input(file);
     NasalXMLVisitor visitor(c, argc, args);
     try {
@@ -392,7 +407,7 @@ static naRef f_parsexml(naContext c, naRef me, int argc, naRef* args)
                 file, e.getFormattedMessage().c_str());
         return naNil();
     }
-    return args[0];
+    return naStr_fromdata(naNewString(c), const_cast<char*>(file), strlen(file));
 }
 
 // Return UNIX epoch time in seconds.
@@ -410,7 +425,6 @@ static naRef f_systime(naContext c, naRef me, int argc, naRef* args)
     do { t = time(0); gettimeofday(&td, 0); } while(t != time(0));
     return naNum(t + 1e-6 * td.tv_usec);
 #endif
-
 }
 
 // Convert a cartesian point to a geodetic lat/lon/altitude.
@@ -518,13 +532,16 @@ static naRef f_airportinfo(naContext c, naRef me, int argc, naRef* args)
         lat = latn->getDoubleValue();
         lon = lonn->getDoubleValue();
     }
+
+    double maxRange = 360.0; // expose this? or pick a smaller value?
+
     if(argc == 0) {
-        apt = aptlst->search(lon, lat, airport);
+        apt = aptlst->search(lon, lat, maxRange, airport);
     } else if(argc == 1 && naIsString(args[0])) {
         const char *s = naStr_data(args[0]);
-        if(!strcmp(s, "airport")) apt = aptlst->search(lon, lat, airport);
-        else if(!strcmp(s, "seaport")) apt = aptlst->search(lon, lat, seaport);
-        else if(!strcmp(s, "heliport")) apt = aptlst->search(lon, lat, heliport);
+        if(!strcmp(s, "airport")) apt = aptlst->search(lon, lat, maxRange, airport);
+        else if(!strcmp(s, "seaport")) apt = aptlst->search(lon, lat, maxRange, seaport);
+        else if(!strcmp(s, "heliport")) apt = aptlst->search(lon, lat, maxRange, heliport);
         else apt = aptlst->search(s);
     } else {
         naRuntimeError(c, "airportinfo() with invalid function arguments");
@@ -536,33 +553,26 @@ static naRef f_airportinfo(naContext c, naRef me, int argc, naRef* args)
     string name = apt->getName();
 
     // set runway hash
-    FGRunwayList *rwylst = globals->get_runways();
-    FGRunway rwy;
     naRef rwys = naNewHash(c);
-    if(rwylst->search(id, &rwy)) {
-        do {
-            if(rwy._id != id) break;
-            if(rwy._type[0] != 'r') continue;
+    for (unsigned int r=0; r<apt->numRunways(); ++r) {
+        FGRunway* rwy(apt->getRunwayByIndex(r));
 
-            naRef rwyid = naStr_fromdata(naNewString(c),
-                    const_cast<char *>(rwy._rwy_no.c_str()),
-                    rwy._rwy_no.length());
+        naRef rwyid = naStr_fromdata(naNewString(c),
+                      const_cast<char *>(rwy->ident().c_str()),
+                      rwy->ident().length());
 
-            naRef rwydata = naNewHash(c);
+        naRef rwydata = naNewHash(c);
 #define HASHSET(s,l,n) naHash_set(rwydata, naStr_fromdata(naNewString(c),s,l),n)
-            HASHSET("id", 2, rwyid);
-            HASHSET("lat", 3, naNum(rwy._lat));
-            HASHSET("lon", 3, naNum(rwy._lon));
-            HASHSET("heading", 7, naNum(rwy._heading));
-            HASHSET("length", 6, naNum(rwy._length * SG_FEET_TO_METER));
-            HASHSET("width", 5, naNum(rwy._width * SG_FEET_TO_METER));
-            HASHSET("threshold1", 10, naNum(rwy._displ_thresh1 * SG_FEET_TO_METER));
-            HASHSET("threshold2", 10, naNum(rwy._displ_thresh2 * SG_FEET_TO_METER));
-            HASHSET("stopway1", 8, naNum(rwy._stopway1 * SG_FEET_TO_METER));
-            HASHSET("stopway2", 8, naNum(rwy._stopway2 * SG_FEET_TO_METER));
+        HASHSET("id", 2, rwyid);
+        HASHSET("lat", 3, naNum(rwy->latitude()));
+        HASHSET("lon", 3, naNum(rwy->longitude()));
+        HASHSET("heading", 7, naNum(rwy->headingDeg()));
+        HASHSET("length", 6, naNum(rwy->lengthM()));
+        HASHSET("width", 5, naNum(rwy->widthM()));
+        HASHSET("threshold", 9, naNum(rwy->_displ_thresh * SG_FEET_TO_METER));
+        HASHSET("stopway", 7, naNum(rwy->_stopway * SG_FEET_TO_METER));
 #undef HASHSET
-            naHash_set(rwys, rwyid, rwydata);
-        } while(rwylst->next(&rwy));
+        naHash_set(rwys, rwyid, rwydata);
     }
 
     // set airport hash
@@ -595,6 +605,7 @@ static struct { const char* name; naCFunction func; } funcs[] = {
     { "_interpolate",  f_interpolate },
     { "rand",  f_rand },
     { "srand",  f_srand },
+    { "abort", f_abort },
     { "directory", f_directory },
     { "parsexml", f_parsexml },
     { "systime", f_systime },
@@ -708,7 +719,7 @@ void FGNasalSys::loadPropertyScripts()
         int j = 0;
         SGPropertyNode *fn;
         bool file_specified = false;
-        while ( (fn = n->getChild("file", j)) != NULL ) {
+        while((fn = n->getChild("file", j)) != NULL) {
             file_specified = true;
             const char* file = fn->getStringValue();
             SGPath p(globals->get_fg_root());
@@ -777,7 +788,9 @@ void FGNasalSys::loadModule(SGPath file, const char* module)
 // used to pass an associated property node to the module, which can then
 // be accessed via cmdarg().  (This is, for example, used by XML dialogs.)
 void FGNasalSys::createModule(const char* moduleName, const char* fileName,
-                              const char* src, int len, const SGPropertyNode* arg)
+                              const char* src, int len,
+                              const SGPropertyNode* cmdarg,
+                              int argc, naRef* args)
 {
     naRef code = parse(fileName, src, len);
     if(naIsNil(code))
@@ -792,9 +805,9 @@ void FGNasalSys::createModule(const char* moduleName, const char* fileName,
     if(!naHash_get(_globals, modname, &locals))
         locals = naNewHash(_context);
 
-    _cmdArg = (SGPropertyNode*)arg;
+    _cmdArg = (SGPropertyNode*)cmdarg;
 
-    call(code, 0, 0, locals);
+    call(code, argc, args, locals);
     hashset(_globals, moduleName, locals);
 }
 
@@ -1011,7 +1024,6 @@ void FGNasalListener::call(SGPropertyNode* which, naRef mode)
     arg[1] = _nas->propNodeGhost(_node);
     arg[2] = mode;                  // value changed, child added/removed
     arg[3] = naNum(_node != which); // child event?
-    _nas->_cmdArg = _node;
     _nas->call(_code, 4, arg, naNil());
     _active--;
 }
@@ -1078,14 +1090,16 @@ bool FGNasalListener::changed(SGPropertyNode* node)
 // is removed from the scene graph.
 
 void FGNasalModelData::modelLoaded(const string& path, SGPropertyNode *prop,
-                                   ssgBranch *)
+                                   osg::Node *)
 {
-    SGPropertyNode *n = prop->getNode("nasal"), *load;
-    if(!n)
+    if(!prop)
+        return;
+    SGPropertyNode *nasal = prop->getNode("nasal");
+    if(!nasal)
         return;
 
-    load = n->getNode("load");
-    _unload = n->getNode("unload");
+    SGPropertyNode *load = nasal->getNode("load");
+    _unload = nasal->getNode("unload");
     if(!load && !_unload)
         return;
 
@@ -1093,7 +1107,13 @@ void FGNasalModelData::modelLoaded(const string& path, SGPropertyNode *prop,
     if(_props)
         _module += ':' + _props->getPath();
     const char *s = load ? load->getStringValue() : "";
-    nasalSys->createModule(_module.c_str(), _module.c_str(), s, strlen(s), _props);
+
+    naRef arg[2];
+    arg[0] = nasalSys->propNodeGhost(_root);
+    arg[1] = nasalSys->propNodeGhost(prop);
+    nasalSys->createModule(_module.c_str(), _module.c_str(), s, strlen(s),
+                           _root, 2, arg);
+    _props = 0;
 }
 
 FGNasalModelData::~FGNasalModelData()
@@ -1109,7 +1129,7 @@ FGNasalModelData::~FGNasalModelData()
 
     if(_unload) {
         const char *s = _unload->getStringValue();
-        nasalSys->createModule(_module.c_str(), _module.c_str(), s, strlen(s), _props);
+        nasalSys->createModule(_module.c_str(), _module.c_str(), s, strlen(s), _root);
     }
     nasalSys->deleteModule(_module.c_str());
 }
