@@ -3,39 +3,19 @@
 //
 // This file is in the Public Domain and comes with no warranty.
 
-#include "pitot.hxx"
+#include <simgear/constants.h>
+
 #include <Main/fg_props.hxx>
 #include <Main/util.hxx>
 
+#include "pitot.hxx"
 
 
 PitotSystem::PitotSystem ( SGPropertyNode *node )
     :
-    num(0),
-    name("pitot")
+    _name(node->getStringValue("name", "pitot")),
+    _num(node->getIntValue("number", 0))
 {
-    int i;
-    for ( i = 0; i < node->nChildren(); ++i ) {
-        SGPropertyNode *child = node->getChild(i);
-        string cname = child->getName();
-        string cval = child->getStringValue();
-        if ( cname == "name" ) {
-            name = cval;
-        } else if ( cname == "number" ) {
-            num = child->getIntValue();
-        } else {
-            SG_LOG( SG_SYSTEMS, SG_WARN, "Error in systems config logic" );
-            if ( name.length() ) {
-                SG_LOG( SG_SYSTEMS, SG_WARN, "Section = " << name );
-            }
-        }
-    }
-}
-
-PitotSystem::PitotSystem ( int i )
-{
-    num = i;
-    name = "pitot";
 }
 
 PitotSystem::~PitotSystem ()
@@ -46,13 +26,14 @@ void
 PitotSystem::init ()
 {
     string branch;
-    branch = "/systems/" + name;
+    branch = "/systems/" + _name;
 
-    SGPropertyNode *node = fgGetNode(branch.c_str(), num, true );
+    SGPropertyNode *node = fgGetNode(branch.c_str(), _num, true );
     _serviceable_node = node->getChild("serviceable", 0, true);
     _pressure_node = fgGetNode("/environment/pressure-inhg", true);
     _density_node = fgGetNode("/environment/density-slugft3", true);
     _velocity_node = fgGetNode("/velocities/airspeed-kt", true);
+    _slip_angle = fgGetNode("/orientation/side-slip-rad", true);
     _total_pressure_node = node->getChild("total-pressure-inhg", 0, true);
 }
 
@@ -74,10 +55,6 @@ PitotSystem::unbind ()
 # define PSFTOINHG (1/INHGTOPSF)
 #endif
 
-#ifndef KTTOFPS
-# define KTTOFPS 1.68781
-#endif
-
 
 void
 PitotSystem::update (double dt)
@@ -87,7 +64,10 @@ PitotSystem::update (double dt)
                                 // velocity in the body axis.
         double p = _pressure_node->getDoubleValue() * INHGTOPSF;
         double r = _density_node->getDoubleValue();
-        double v = _velocity_node->getDoubleValue() * KTTOFPS;
+        double v = _velocity_node->getDoubleValue() * SG_KT_TO_FPS;
+        v *= cos(_slip_angle->getDoubleValue());
+        if (v < 0.0)
+            v = 0.0;
         double q = 0.5 * r * v * v; // dynamic
         _total_pressure_node->setDoubleValue((p + q) * PSFTOINHG);
     }
